SET ECHO ON TERM OFF FEED OFF;
SPOOL 04_tacpkg.log;
REM
REM $Header: 224270.1 tacpkg.sql 11.4.4.1 2012/01/02 csierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   tacpkg.sql
REM
REM DESCRIPTION
REM   This script creates the packages owned by schema TRCANLZR
REM
REM PRE-REQUISITES
REM   1. This script must be executed connected as TRCANLZR
REM
REM PARAMETERS
REM   1. None
REM
REM EXECUTION
REM   1. Navigate to trca/install directory
REM   2. Start SQL*Plus connecting as TRCANLZR
REM   3. Execute script tacpkg.sql
REM
REM EXAMPLE
REM   # cd trca/install
REM   # sqlplus trcanlzr
REM   SQL> start tacpkg.sql
REM
REM NOTES
REM   1. This script is executed automatically by tacreate.sql
REM   2. For possible errors see tacpkg.log file
REM
-- begin common
DEF _SQLPLUS_RELEASE
SELECT USER FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
SELECT * FROM v$instance;
SELECT name, value FROM v$parameter2 WHERE name LIKE '%dump_dest';
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
-- end common
WHENEVER SQLERROR CONTINUE;
ALTER SESSION SET PLSQL_CODE_TYPE = INTERPRETED;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
SET ECHO OFF TERM ON;
ALTER SESSION SET NLS_NUMERIC_CHARACTERS = ".,";
ALTER SESSION SET NLS_LENGTH_SEMANTICS = CHAR;
DECLARE
  rdbms_release NUMBER;
BEGIN
  IF USER NOT IN ('TRCANLZR', 'SQLTXPLAIN') THEN
    RAISE_APPLICATION_ERROR(-20100, 'Install failed - TRCANLZR objects should be created connected as TRCANLZR or SQLTXPLAIN, not as '||USER);
  END IF;
  SELECT TO_NUMBER(SUBSTR(version, 1, INSTR(version, '.', 1, 2) - 1))
    INTO rdbms_release
    FROM v$instance;
  IF rdbms_release < 9.2 THEN
    RAISE_APPLICATION_ERROR(-20200, 'Install failed - TRCANLZR should be installed in 9i(9.2) or higher, not in '||rdbms_release);
  END IF;
END;
/

SET TERM ON;
PRO tool_owner: "&&tool_owner."
PRO role_name: "&&role_name."
--
SET TERM ON;
PRO Creating Package Specs TRCA$G
SET TERM OFF;
@@tacpkgg.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$P
SET TERM OFF;
@@tacpkgp.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$T
SET TERM OFF;
@@tacpkgt.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$I
SET TERM OFF;
@@tacpkgi.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$E
SET TERM OFF;
@@tacpkge.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$R
SET TERM OFF;
@@tacpkgr.pks
--
SET TERM ON;
PRO Creating Package Specs TRCA$X
SET TERM OFF;
@@tacpkgx.pks
--
SET TERM ON;
PRO Creating Views
SET TERM OFF;
@@tacvw.sql
--
SET TERM ON;
PRO Creating Package Body TRCA$G
SET TERM OFF;
@@tacpkgg.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$P
SET TERM OFF;
@@tacpkgp.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$T
SET TERM OFF;
@@tacpkgt.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$I
SET TERM OFF;
@@tacpkgi.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$E
SET TERM OFF;
@@tacpkge.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$R
SET TERM OFF;
@@tacpkgr.pkb
--
SET TERM ON;
PRO Creating Package Body TRCA$X
SET TERM OFF;
@@tacpkgx.pkb
--
SET TERM ON;
PRO Creating Grants on Packages
SET TERM OFF;
--
GRANT EXECUTE ON trca$g TO &&role_name.;
GRANT EXECUTE ON trca$p TO &&role_name.;
GRANT EXECUTE ON trca$t TO &&role_name.;
GRANT EXECUTE ON trca$i TO &&role_name.;
GRANT EXECUTE ON trca$e TO &&role_name.;
GRANT EXECUTE ON trca$r TO &&role_name.;
GRANT EXECUTE ON trca$x TO &&role_name.;
--
SET HEA ON LIN 2000 PAGES 1000 TRIMS ON TIM OFF SERVEROUT ON SIZE 1000000;
SHOW PARAMETERS NLS;
EXEC trca$p.set_nls;
EXEC trca$g.general_initialization;

SET TERM ON;
COL tool_version FOR A16 HEA 'Tool Version';
SELECT trca$g.get_param('tool_version', 'I') tool_version FROM DUAL;
COL install_date FOR A16 HEA 'Install Date';
SELECT trca$g.get_param('install_date', 'I') install_date FROM DUAL;
COL column_value FOR A128 HEA 'Directories';
SELECT column_value FROM TABLE(trca$g.directories);
COL column_value FOR A128 HEA 'Libraries';
SELECT column_value FROM TABLE(trca$g.packages);
WHENEVER SQLERROR CONTINUE;
SET FEED 6;
PRO TACPKG completed.
--SPOOL OFF;
