SET ECHO ON TERM OFF FEED OFF;
SPOOL 01_tacreate.log;
REM
REM $Header: 224270.1 tacreate.sql 11.4.4.1 2012/01/02 csierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   tacreate.sql
REM
REM DESCRIPTION
REM   This script installs the TRCANLZR tool into its own schema.
REM
REM PRE-REQUISITES
REM   1. This script must be executed connected INTERNAL (SYS) as
REM      SYSDBA
REM   2. During the installation you will be asked to enter the
REM      follwing:
REM        o  Connect Identifier - Optional
REM           Some restricted-access systems may need to specify
REM           a connect identifier like "@PROD".
REM           This optional parameter allows to enter it. Else,
REM           enter nothing and just hit the "Enter" key.
REM        o  TRCANLZR password - Required and it has no default
REM        o  TRCANLZR default tablespace - You will be presented
REM           with a list, then you will have to enter one tablespace
REM           name from that list
REM        o  TRCANLZR temporary tablespace - Similar as above
REM        o  Type of object for large staging tables - Enter "T" is
REM           you want large tables to be created as PERMANENT, or
REM           "N" if you prefer GLOBAL TEMPORARY (recommended)
REM
REM PARAMETERS
REM   1. None inline. During the installation you will be asked for
REM      the values of the parameters described under pre-requisites
REM      section above
REM
REM EXECUTION
REM   1. Navigate to trca/install directory
REM   2. Start SQL*Plus connecting INTERNAL (SYS) as SYSDBA
REM   3. Execute script tacreate.sql and respond to values requested
REM
REM EXAMPLE
REM   # cd trca/install
REM   # sqlplus /nolog
REM   SQL> connect / as sysdba
REM   SQL> start tacreate.sql
REM
REM NOTES
REM   1. For possible errors see *.log files
REM   2. If you want to make all TRCANLZR staging tables permanent,
REM      answer Y when asked for "large_tables_as_permanent".
REM
-- begin common
DEF _SQLPLUS_RELEASE
SELECT USER FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
SELECT * FROM v$instance;
SELECT name, value FROM v$parameter2 WHERE name LIKE '%dump_dest';
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
-- end common
SET ECHO OFF TERM ON;
DEF tool_owner = TRCANLZR
DEF role_name = PUBLIC
WHENEVER SQLERROR EXIT SQL.SQLCODE;
ALTER SESSION SET NLS_NUMERIC_CHARACTERS = ".,";
DECLARE
  rdbms_release NUMBER;
BEGIN
  IF USER <> 'SYS' THEN
    RAISE_APPLICATION_ERROR(-20100, 'Install failed - TRCANLZR should be installed connected as SYS, not as '||USER);
  END IF;
  SELECT TO_NUMBER(SUBSTR(version, 1, INSTR(version, '.', 1, 2) - 1))
    INTO rdbms_release
    FROM v$instance;
  IF rdbms_release < 9.2 THEN
    RAISE_APPLICATION_ERROR(-20200, 'Install failed - TRCANLZR should be installed in 9i(9.2) or higher, not in '||rdbms_release);
  END IF;
END;
/
WHENEVER SQLERROR CONTINUE;

-- drops TRCA objects owned by TRCANLZR
@@tadrop.sql
SPOOL OFF;
-- creates or recreates user TRCANLZR
@@tacusr.sql
-- create SQLT and TRCA directories
@@tasqdirset.sql
-- connects as TRCANLZR
SET VER OFF;
CONNECT trcanlzr/&&trcanlzr_password.&&connect_identifier.
UNDEFINE trcanlzr_password trcanlzr_password2
SET VER ON FEED 6;
-- verifies that TRCANLZR can actually read and write files (1st pass)
@@tautltest.sql
-- creates TRCA schema objects owned by TRCANLZR
@@tacobj.sql
-- creates TRCA set of packages owned by TRCANLZR
@@tacpkg.sql

PRO
PRO Taking a snapshot of some Data Dictionary objects, please wait...
PRO
EXEC trca$t.refresh_trca$_dict_from_this;
PRO
PRO Snapshot of some Data Dictionary objects completed.
PRO

-- verifies that TRCANLZR can actually read and write files (2nd pass)
@@tautltest.sql

UNDEFINE tool_owner connect_identifier
PRO
PRO TACREATE completed. Installation completed successfully.
