SPO 09_tautltest.log;
SET TERM OFF ECHO ON SERVEROUT ON SIZE 1000000;
REM
REM $Header: 224270.1 tautltest.sql 11.4.3.1 2011/06/17 csierra $
REM
REM Copyright (c) 2000-2000, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   tautltest.sql
REM
REM DESCRIPTION
REM   This script tests that UTL_FILE works properly as required
REM   by Trace Analyzer to read and write files.
REM
REM PRE-REQUISITES
REM   1. This script must be executed connected as TRCANLZR
REM
REM PARAMETERS
REM   1. None
REM
REM EXECUTION
REM   1. Navigate to trca/install directory
REM   2. Start SQL*Plus connecting as TRCANLZR
REM   3. Execute script tautltest.sql
REM
REM EXAMPLE
REM   # cd trca/install
REM   # sqlplus trcanlzr
REM   SQL> start tautltest.sql
REM
REM NOTES
REM   1. This script is executed automatically by tacreate.sql
REM   2. If installing in RAC, perform the installation locally
REM      and run TRCA from the same node where it was installed.
REM   3. For possible errors see tautltest.log file
REM
SET ECHO OFF;
-- begin common
DEF _SQLPLUS_RELEASE
SELECT USER FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
SELECT * FROM v$instance;
SELECT name, value FROM v$parameter2 WHERE name LIKE '%dump_dest';
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
-- end common
WHENEVER SQLERROR EXIT SQL.SQLCODE;
ALTER SESSION SET NLS_NUMERIC_CHARACTERS = ".,";

DECLARE
  rdbms_release NUMBER;
  l_open_mode VARCHAR2(2);

  PROCEDURE open_write_close (
    p2_location IN VARCHAR2,
    p2_filename IN VARCHAR2 )
  IS
    out_file_type UTL_FILE.file_type;
  BEGIN
    DBMS_OUTPUT.PUT_LINE('Test WRITING into file "'||p2_filename||'" in directory "'||p2_location||'" started.');
    out_file_type :=
    SYS.UTL_FILE.FOPEN (
       location     => p2_location,
       filename     => p2_filename,
       open_mode    => l_open_mode,
       max_linesize => 32767 );

    IF rdbms_release < 10 THEN
      SYS.UTL_FILE.PUT_LINE (
        file   => out_file_type,
        buffer => 'Hello World!');
    ELSE
      SYS.UTL_FILE.PUT_RAW (
        file   => out_file_type,
        buffer => UTL_RAW.CAST_TO_RAW('Hello World!'||CHR(10)));
    END IF;

    SYS.UTL_FILE.FCLOSE(file => out_file_type);
    DBMS_OUTPUT.PUT_LINE('Test WRITING into file "'||p2_filename||'" in directory "'||p2_location||'" completed.');
  END open_write_close;

  PROCEDURE open_read_close (
    p2_directory_alias IN VARCHAR2,
    p2_file_name IN VARCHAR2 )
  IS
    l_file BFILE;
    l_file_len INTEGER := NULL;
    l_file_offset INTEGER;
    l_chunk_raw RAW(32767);
    l_chunk VARCHAR2(32767);
  BEGIN
    DBMS_OUTPUT.PUT_LINE('Test READING file "'||p2_file_name||'" from directory "'||p2_directory_alias||'" started.');
    l_file := BFILENAME(p2_directory_alias, p2_file_name);
    DBMS_LOB.FILEOPEN (file_loc => l_file);
    l_file_len := DBMS_LOB.GETLENGTH(file_loc => l_file);
    l_file_offset := 1;

    DBMS_LOB.READ (
      file_loc => l_file,
      amount   => l_file_len,
      offset   => l_file_offset,
      buffer   => l_chunk_raw );

    l_chunk := UTL_RAW.CAST_TO_VARCHAR2 (r => l_chunk_raw);
    DBMS_LOB.FILECLOSE (file_loc => l_file);
    DBMS_OUTPUT.PUT_LINE('Test READING file "'||p2_file_name||'" from directory "'||p2_directory_alias||'" completed.');
  END open_read_close;

  PROCEDURE read_attributes (
    p2_directory_alias IN VARCHAR2,
    p2_file_name IN VARCHAR2 )
  IS
    l_file_exists     BOOLEAN;
    l_file_length     NUMBER;
    l_file_block_size NUMBER;
  BEGIN
    SYS.UTL_FILE.FGETATTR (
      location     => p2_directory_alias,
      filename     => p2_file_name,
      fexists      => l_file_exists,
      file_length  => l_file_length,
      block_size   => l_file_block_size );

    IF l_file_exists THEN
      DBMS_OUTPUT.PUT_LINE('File "'||p2_file_name||'" exists in directory "'||p2_directory_alias||'".');
    ELSE
      DBMS_OUTPUT.PUT_LINE('File "'||p2_file_name||'" does not exists in directory "'||p2_directory_alias||'".');
      RAISE_APPLICATION_ERROR(-20100, 'Install failed - UTL_FILE.FGETATTR not capable of reading file attributes.');
    END IF;
  END read_attributes;
BEGIN
  SELECT TO_NUMBER(SUBSTR(version, 1, INSTR(version, '.', 1, 2) - 1))
    INTO rdbms_release
    FROM v$instance;

  IF rdbms_release < 10 THEN
    l_open_mode := 'W';
  ELSE
    l_open_mode := 'WB';
  END IF;

  open_write_close('TRCA$STAGE', 'tautltest.txt');
  open_read_close('TRCA$STAGE', 'tautltest.txt');
  read_attributes('TRCA$STAGE', 'tautltest.txt');

  DBMS_OUTPUT.PUT_LINE('TRCA UTL_FILE WRITE/READ Test passed!');
END;
/

SET TERM ON;
WHENEVER SQLERROR CONTINUE;
PRO
PRO TAUTLTEST completed.
SPO OFF;
