SPO sqltcompare.log;
SET DEF ON TERM OFF ECHO ON FEED OFF VER OFF HEA ON LIN 200 PAGES 100 TRIMS ON TI OFF TIMI OFF APPI OFF SERVEROUT ON SIZE 1000000 NUMF "" SQLP SQL>;
SET SERVEROUT ON SIZE UNL;
REM
REM $Header: 215187.1 sqltcompare.sql 11.4.4.5 2012/05/02 carlos.sierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   sqlt/run/sqltcompare.sql
REM
REM DESCRIPTION
REM   This script compares two prior executions of SQLT and
REM   identifies differences in execution plans, CBO statistics,
REM   metadata, CBO initialization parameters and bug fix control.
REM
REM PRE-REQUISITES
REM   1. Execute any standard method of SQLT in both SOURCES.
REM   2. Import SQLT repository into test system as per readme.
REM
REM PARAMETERS
REM   1. Statement IDs as per SQLT in SOURCEs system (required)
REM      A list of statement ids is presented.
REM   2. Plan Hash Value for both statement ids (required).
REM      A list of known plans is presented.
REM
REM EXECUTION
REM   1. Navigate to sqlt/run directory.
REM   2. Connect into SQL*Plus as SQLTXPLAIN or application user
REM      or SYSDBA.
REM   3. Execute script sqltcompare.sql passing statement ids and
REM      plan hash values (parameters can be passed inline or until
REM      requested).
REM
REM EXAMPLE
REM   # cd sqlt/run
REM   # sqlplus sqltxplain
REM   SQL> START sqltcompare.sql [statement id1] [statement id2] [plan hash value1] [plan hash value2];
REM   SQL> START sqltcompare.sql 32263 92366 8673881838 1452771383;
REM   SQL> START sqltcompare.sql 32263 92366;
REM   SQL> START sqltcompare.sql;
REM
REM NOTES
REM   1. For possible errors see sqltcompare.log
REM
-- begin common
DEF _SQLPLUS_RELEASE
SELECT USER FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
SELECT * FROM v$instance;
SELECT name, value FROM v$parameter2 WHERE name LIKE '%dump_dest';
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
-- end common
COL libraries FOR A64;
SELECT SUBSTR(text, INSTR(text, ' ', 1, 3) + 1,  INSTR(text, ' ', 1, 6) - INSTR(text, ' ', 1, 3)) libraries
  FROM all_source
 WHERE owner = 'SQLTXPLAIN'
   AND line = 2
   AND text LIKE '%$Header%'
 ORDER BY 1;
SELECT status||' '||object_type||' '||object_name libraries
  FROM all_objects
 WHERE owner = 'SQLTXPLAIN'
   AND object_type LIKE 'PACKAGE%'
 ORDER BY 1;
COL lib_count NEW_V lib_count FOR 999;
COL role_count NEW_V role_count FOR 999;
SELECT COUNT(*) lib_count FROM all_objects WHERE owner = 'SQLTXPLAIN' AND object_type = 'PACKAGE' AND status = 'VALID';
SELECT COUNT(*) role_count FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT granted_role FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT user FROM dual;
SET TERM ON ECHO OFF;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
BEGIN
  IF &&role_count. < 1 THEN
    RAISE_APPLICATION_ERROR(-20109, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role. Request: "GRANT SQLT_USER_ROLE TO '||USER||';" to your DBA.');
  END IF;
  IF &&lib_count. < 17 THEN
    RAISE_APPLICATION_ERROR(-20110, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role or SQLT is not properly installed. Review installation NN_*.log files.');
  END IF;
END;
/
PRO ... please wait ...
SET TERM OFF ECHO ON;
EXEC sqltxplain.sqlt$migrate;
WHENEVER SQLERROR CONTINUE;
SET TERM ON ECHO OFF;
SELECT LPAD(s.statement_id, 5, '0') staid,
       SUBSTR(s.method, 1, 3) method,
       SUBSTR(s.instance_name_short, 1, 8) instance,
       SUBSTR(s.sql_text, 1, 60) sql_text
  FROM sqltxplain.sqlt$_sql_statement s
 WHERE USER IN ('SYS', 'SYSTEM', 'SQLTXPLAIN', s.username)
 ORDER BY
       s.statement_id;
PRO
PRO Parameter 1:
PRO STATEMENT_ID1 (required)
PRO
DEF statement_id1 = '&1';
PRO
PRO Parameter 2:
PRO STATEMENT_ID2 (required)
PRO
DEF statement_id2 = '&2';
PRO
PRO
EXEC sqltxplain.sqlt$t.sqlt_plan_hash_value(TO_NUMBER(TRIM('&&statement_id1.')));
EXEC sqltxplain.sqlt$t.sqlt_plan_hash_value(TO_NUMBER(TRIM('&&statement_id2.')));
COL attribute FOR A9;
SELECT p.plan_hash_value, p.sqlt_plan_hash_value, p.statement_id,
       DECODE(p.plan_hash_value, s.best_plan_hash_value, '[B]')||
       DECODE(p.plan_hash_value, s.worst_plan_hash_value, '[W]')||
       DECODE(p.plan_hash_value, s.xecute_plan_hash_value, '[X]') attribute
  FROM (
SELECT DISTINCT plan_hash_value, sqlt_plan_hash_value, statement_id
  FROM sqltxplain.sqlt$_plan_extension
 WHERE statement_id = TO_NUMBER(TRIM('&&statement_id1.'))) p,
       sqltxplain.sqlt$_sql_statement s
 WHERE p.statement_id = s.statement_id
 ORDER BY
       p.plan_hash_value;
PRO
PRO Parameter 3:
PRO PLAN_HASH_VALUE1 (required if more than one)
PRO
DEF plan_hash_value1 = '&3';
PRO
SELECT p.plan_hash_value, p.sqlt_plan_hash_value, p.statement_id,
       DECODE(p.plan_hash_value, s.best_plan_hash_value, '[B]')||
       DECODE(p.plan_hash_value, s.worst_plan_hash_value, '[W]')||
       DECODE(p.plan_hash_value, s.xecute_plan_hash_value, '[X]') attribute
  FROM (
SELECT DISTINCT plan_hash_value, sqlt_plan_hash_value, statement_id
  FROM sqltxplain.sqlt$_plan_extension
 WHERE statement_id = TO_NUMBER(TRIM('&&statement_id2.'))) p,
       sqltxplain.sqlt$_sql_statement s
 WHERE p.statement_id = s.statement_id
 ORDER BY
       p.plan_hash_value;
PRO
PRO Parameter 4:
PRO PLAN_HASH_VALUE2 (required if more than one)
PRO
DEF plan_hash_value2 = '&4';
PRO
PRO
PRO Values passed to sqltcompare:
PRO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PRO STATEMENT_ID1   : "&&statement_id1."
PRO STATEMENT_ID2   : "&&statement_id2."
PRO PLAN_HASH_VALUE1: "&&plan_hash_value1."
PRO PLAN_HASH_VALUE2: "&&plan_hash_value2."
PRO
EXEC DBMS_APPLICATION_INFO.SET_MODULE('sqltcompare', 'script');
PRO
PRO ... please wait ...
PRO
WHENEVER SQLERROR EXIT SQL.SQLCODE;
SET TERM ON ECHO OFF VER OFF;
EXEC sqltxplain.sqlt$c.compare_report(TO_NUMBER(TRIM('&&statement_id1.')), TO_NUMBER(TRIM('&&statement_id2.')), TO_NUMBER(TRIM('&&plan_hash_value1.')), TO_NUMBER(TRIM('&&plan_hash_value2.')));
WHENEVER SQLERROR CONTINUE;
SET TERM OFF ECHO OFF FEED OFF VER OFF SHOW OFF HEA OFF LIN 2000 NEWP NONE PAGES 0 SQLC MIX TAB ON TRIMS ON TI OFF TIMI OFF ARRAY 100 NUMF "" SQLP SQL> SUF sql BLO . RECSEP OFF APPI OFF SERVEROUT ON SIZE 1000000 FOR TRU;
SET SERVEROUT ON SIZE UNL FOR TRU;
PRO No fatal errors!
COL filename NEW_V filename FOR A256;
SPO OFF;
--
DEF filename = 'sqlt_s&&statement_id1._s&&statement_id2._compare.html';
SPO &&filename.;
SELECT * FROM TABLE(sqltxplain.sqlt$r.display_file('&&filename.', TO_NUMBER(TRIM('&&statement_id1.')), TO_NUMBER(TRIM('&&statement_id2.'))));
SPO OFF;
--
SET TERM OFF;
CL COL;
EXEC DBMS_APPLICATION_INFO.SET_MODULE(NULL, NULL);
UNDEFINE 1 2 3 4 lib_count role_count statement_id1 statement_id2 plan_hash_value1 plan_hash_value2;
SET DEF ON TERM ON ECHO OFF FEED 6 VER ON SHOW OFF HEA ON LIN 80 NEWP 1 PAGES 14 SQLC MIX TAB ON TRIMS OFF TI OFF TIMI OFF ARRAY 15 NUMF "" SQLP SQL> SUF sql BLO . RECSEP WR APPI OFF SERVEROUT OFF;
PRO
PRO &&filename. has been generated
PRO
PRO SQLTCOMPARE completed.
