SET DEF ON ECHO ON TERM OFF APPI OFF SERVEROUT ON SIZE 1000000 NUMF "" SQLP SQL>;
SET SERVEROUT ON SIZE UNL;
SPO trcanlzr_error.log;
REM
REM $Header: 224270.1 sqltrcaxtr.sql 11.4.4.5 2012/05/02 carlos.sierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   sqltrcaxtr.sql
REM
REM DESCRIPTION
REM   This script parses a sql trace file generated by event 10046
REM   levels 1, 4, 8 or 12. Then it produces a comprehensive report
REM   which includes top sql and sql genealogy according to the sql
REM   statements found in the trace provided.
REM   After the trace is analyzed it proceeds with a SQLT XTRACT on
REM   each of the Top SQL statements.
REM
REM PRE-REQUISITES
REM   1. Execute directly from the sqlt/run directory and not
REM      from some other directory passing path and script name.
REM
REM PARAMETERS
REM   1. Trace file name on UDUMP.
REM
REM EXECUTION
REM   1. Execute directly from the sqlt/run directory and not
REM      from some other directory passing path and script name.
REM   2. Start SQL*Plus connecting as application user that generated
REM      trace to be analyzed
REM   3. Execute script sqltrcaxtr.sql passing name of trace file or
REM      the name of the text file that contains multiple traces (one
REM      per line).
REM
REM EXAMPLES
REM   # cd sqlt/run
REM   # sqlplus [apps user]
REM   SQL> start sqltrcaxtr.sql [name of your trace file]
REM   SQL> start sqltrcaxtr.sql largesql.trc  <== your trace file
REM   SQL> start sqltrcaxtr.sql control_file.txt  <== your text file
REM   Notice that a single file must end with ".trc" while a control
REM   file must end with something else like ".txt".
REM
REM NOTES
REM   1. For best results, use Trace Analyzer in the same system
REM      where the trace was generated, and connected as the same
REM      user that generated the trace.
REM   2. To analyze multiple files, create a text file that contains
REM      the names of the trace files to be analyzed, and provide
REM      this filename as the inline parameter to the Trace Analyzer.
REM      This small text file with names of traces (one per line),
REM      should also be located in the the UDUMP directory.
REM   3. For possible errors see trcanlzr_error.log generated under
REM      the SQL*Plus default directory.
REM
COL libraries FOR A64;
SELECT SUBSTR(text, INSTR(text, ' ', 1, 3) + 1,  INSTR(text, ' ', 1, 6) - INSTR(text, ' ', 1, 3)) libraries
  FROM all_source
 WHERE owner = 'SQLTXPLAIN'
   AND line = 2
   AND text LIKE '%$Header%'
 ORDER BY 1;
SELECT status||' '||object_type||' '||object_name libraries
  FROM all_objects
 WHERE owner = 'SQLTXPLAIN'
   AND object_type LIKE 'PACKAGE%'
 ORDER BY 1;
COL lib_count NEW_V lib_count FOR 999;
COL role_count NEW_V role_count FOR 999;
SELECT COUNT(*) lib_count FROM all_objects WHERE owner = 'SQLTXPLAIN' AND object_type = 'PACKAGE' AND status = 'VALID';
SELECT COUNT(*) role_count FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT granted_role FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT user FROM dual;
SET TERM ON ECHO OFF VER OFF;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
BEGIN
  IF &&role_count. < 1 THEN
    RAISE_APPLICATION_ERROR(-20109, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role. Request: "GRANT SQLT_USER_ROLE TO '||USER||';" to your DBA.');
  END IF;
  IF &&lib_count. < 17 THEN
    RAISE_APPLICATION_ERROR(-20110, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role or SQLT is not properly installed. Review installation NN_*.log files.');
  END IF;
END;
/
PRO
PRO WARNING:
PRO ~~~~~~~
PRO Execute sqltrcaxtr.sql connecting directly from the
PRO sqlt/run directory and not from some other directory.
PRO
HOS pwd
PRO If directory above ends with sqlt/run please enter
PRO keyword RUN when asked for "directory".
PRO
PRO &&directory.
PRO
BEGIN
  IF UPPER(NVL(TRIM('&&directory.'), 'NULL')) <> 'RUN' THEN
    RAISE_APPLICATION_ERROR(-20100, 'Directory verification keyword "&&directory." was not RUN. Abort!');
  END IF;
END;
/
WHENEVER SQLERROR CONTINUE;
SET VER ON;
PRO
PRO Parameter 1:
PRO Trace Filename or control_file.txt (required)
PRO
DEF input_filename = &1;
PRO
PRO Value passed to sqltrcaxtr.sql:
PRO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PRO TRACE_FILENAME: &&input_filename.
PRO
EXEC DBMS_APPLICATION_INFO.SET_MODULE('sqltrcaxtr', 'script');
PRO
PRO Analyzing &&input_filename.
PRO
SET TERM OFF HEA ON LIN 2000 PAGES 1000 TRIMS ON TI OFF TIMI OFF;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
DEF _SQLPLUS_RELEASE
COL connected_user NEW_V connected_user FOR A30;
SELECT user connected_user FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
COL library FOR A64 HEA 'Libraries';
SELECT object_type||' '||object_name||' ('||status||')' library
  FROM all_objects
 WHERE owner = 'SQLTXPLAIN'
   AND object_type IN ('PACKAGE', 'PACKAGE', 'PROCEDURE', 'FUNCTION')
 ORDER BY
       object_type, object_name;
SET TERM ON ECHO OFF;
PRO
PRO NOTE:
PRO If you get one of these errors it means SQLTXPLAIN is not installed:
PRO PLS-00201: identifier 'SQLTXPLAIN.SQLT$A' must be declared
PRO ORA-00904: "SQLTXPLAIN"."SQLT$A"."VALIDATE_USER": invalid identifier;
PRO Review NN_*.log files created during installation and fix errors reported.
PRO
EXEC sqltxplain.sqlt$a.validate_user(USER);
CL SCR;
PRO ... please wait ...
SET TERM OFF;
EXEC sqltxplain.trca$p.set_nls;
EXEC sqltxplain.trca$g.general_initialization;
VAR v_tool_execution_id NUMBER;
EXEC :v_tool_execution_id := sqltxplain.trca$p.get_tool_execution_id;
COL tool_execution_id NEW_V tool_execution_id FOR A17;
SELECT TO_CHAR(:v_tool_execution_id) tool_execution_id FROM DUAL;
COL tool_version FOR A16 HEA 'Tool Version';
SELECT sqltxplain.trca$g.get_param('tool_version', 'I') tool_version FROM DUAL;
COL install_date FOR A16 HEA 'Install Date';
SELECT sqltxplain.trca$g.get_param('install_date', 'I') install_date FROM DUAL;
COL host_name FOR A128 HEA 'Host Name';
SELECT sqltxplain.trca$g.get_param('tool_host_name', 'I') host_name FROM DUAL;
COL column_value FOR A128 HEA 'Directories';
SELECT column_value FROM TABLE(sqltxplain.trca$g.directories);
COL column_value FOR A128 HEA 'Libraries';
SELECT column_value FROM TABLE(sqltxplain.trca$g.packages);
PRO
PRO In case of unexpected termination, review file
PRO trca_e&&tool_execution_id..log from output directory
SET TERM ON;
PRO To monitor progress, login as SQLTXPLAIN into another session and execute:
PRO SQL> SELECT * FROM trca$_log_v;;
PRO
PRO ... analyzing trace(s) ...
PRO
SET TERM ON ECHO OFF VER OFF;
EXEC sqltxplain.trca$i.trcanlzr(p_file_name => '&&input_filename.', x_tool_execution_id => :v_tool_execution_id);
SET TERM OFF;
COL trace_file_name NEW_V trace_file_name FOR A640;
SELECT sqltxplain.trca$g.get_1st_trace_path_n_name trace_file_name FROM DUAL;
SET TERM ON;
PRO
PRO
PRO Trace Analyzer completed.
PRO Review first trcanlzr_error.log file for possible fatal errors.
PRO Review next trca_e&&tool_execution_id..log for parsing messages and totals.
PRO
PRO Copying now generated files into local directory
PRO
SET TERM OFF ECHO OFF DEF ON FEED OFF FLU OFF HEA OFF LIN 2000 NEWP NONE PAGES 0 SHOW OFF SQLC MIX TAB OFF TRIMS ON VER OFF TI OFF TIMI OFF ARRAY 100 SQLP SQL> BLO . RECSEP OFF APPI OFF SERVEROUT ON SIZE 1000000 FOR TRU;
SET SERVEROUT ON SIZE UNL;
COL column_value FOR A2000;
WHENEVER OSERROR CONTINUE;
WHENEVER SQLERROR CONTINUE;
PRO No fatal errors!
SPO trca_e&&tool_execution_id..txt;
SELECT column_value FROM TABLE(sqltxplain.trca$g.display_file(:v_tool_execution_id, 'TEXT'));
SPO trca_e&&tool_execution_id..html;
SELECT column_value FROM TABLE(sqltxplain.trca$g.display_file(:v_tool_execution_id, 'HTML'));
SPO trca_e&&tool_execution_id..log;
SELECT column_value FROM TABLE(sqltxplain.trca$g.display_file(:v_tool_execution_id, 'LOG'));
SPO OFF;
HO tkprof &&trace_file_name. trca_e&&tool_execution_id..tkprof
HO zip -mT trca_e&&tool_execution_id. trca_e&&tool_execution_id.* trcanlzr_error.log
HO zip -d trca_e&&tool_execution_id. trcanlzr_error.log
HO unzip -l trca_e&&tool_execution_id.
SET TERM ON;
PRO
PRO File trca_e&&tool_execution_id..zip has been created.
PRO
CL COL;
UNDEFINE 1
PRO SQLTRCAXTR completed.
PRO
PRO TRCA Top SQL
PRO ~~~~~~~~~~~~
SELECT NVL(sqlid, TO_CHAR(hv)) trca_top_sql
  FROM sqltxplain.trca$_sql_v
 WHERE tool_execution_id = :v_tool_execution_id
   AND (top_sql = 'Y' OR top_sql_et = 'Y' OR top_sql_ct = 'Y')
   AND hv > 0
 ORDER BY
       NVL(rank, 0) + NVL(rank_et, 0) + NVL(rank_ct, 0);
PRO
PRO Proceeding now with XTRACT for Top SQL
PRO
EXEC DBMS_APPLICATION_INFO.SET_MODULE(NULL, NULL);
SET TERM ON ECHO OFF DEF ON FEED 6 VER ON HEA ON LIN 80 NEWP 1 PAGES 14 SHOW OFF SQLC MIX TAB ON TRIMS OFF TI OFF TIMI OFF ARRAY 15 NUMF "" SQLP SQL> SUF sql BLO . RECSEP WR APPI OFF SERVEROUT OFF;
@@sqltrcaset.sql &&tool_execution_id


