SPO sqltxecute.log;
SET DEF ON;
SET DEF ^ TERM OFF ECHO ON VER OFF SERVEROUT ON SIZE 1000000 NUMF "" SQLP SQL>;
SET SERVEROUT ON SIZE UNL;
SET ESC ! SQLBL ON;
REM
REM $Header: 215187.1 sqltxecute.sql 11.4.4.6 2012/06/02 carlos.sierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   sqlt/run/sqltxecute.sql
REM
REM DESCRIPTION
REM   Collects SQL tuning diagnostics data and generates a set of
REM   diagnostics files. It inputs one SCRIPT which contains one SQL
REM   statement. If the SQL contains binds, the SCRIPT should also
REM   include binds declaration and assignment. For an example of
REM   its format see sqlt/input/sample/script1.sql provided.
REM
REM PRE-REQUISITES
REM   1. Use a dedicated SQL*Plus connection (not a shared one).
REM   2. User has been granted SQLT_USER_ROLE.
REM
REM PARAMETERS
REM   1. SCRIPT name which contains SQL and its binds (required)
REM   2. SQLTXPLAIN password (required).
REM
REM EXECUTION
REM   1. Place your SCRIPT with one SQL into sqlt/input directory
REM   2. Navigate to sqlt main directory.
REM   3. Start SQL*Plus connecting as the application user of the SQL.
REM   4. Execute script sqltxecute.sql passing directory path and
REM      name of SCRIPT with one SQL and its bind variables.
REM   5. Enter SQLTXPLAIN password when asked for it.
REM
REM EXAMPLE
REM   # cd sqlt
REM   # sqlplus apps
REM   SQL> START [path]sqltxecute.sql [path]scriptname
REM   SQL> START run/sqltxecute.sql input/sample/script1.sql
REM
REM NOTES
REM   1. For possible errors see sqltxecute.log and sqltxecute2.log.
REM   2. For better output execute this script connected as the
REM      application user who issued the SQL.
REM
COL libraries FOR A64;
SELECT SUBSTR(text, INSTR(text, ' ', 1, 3) + 1,  INSTR(text, ' ', 1, 6) - INSTR(text, ' ', 1, 3)) libraries
  FROM all_source
 WHERE owner = 'SQLTXPLAIN'
   AND line = 2
   AND text LIKE '%$Header%'
 ORDER BY 1;
SELECT status||' '||object_type||' '||object_name libraries
  FROM all_objects
 WHERE owner = 'SQLTXPLAIN'
   AND object_type LIKE 'PACKAGE%'
 ORDER BY 1;
COL lib_count NEW_V lib_count FOR 999;
COL role_count NEW_V role_count FOR 999;
SELECT COUNT(*) lib_count FROM all_objects WHERE owner = 'SQLTXPLAIN' AND object_type = 'PACKAGE' AND status = 'VALID';
SELECT COUNT(*) role_count FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT granted_role FROM user_role_privs WHERE granted_role IN ('SQLT_USER_ROLE', 'DBA');
SELECT user FROM dual;
SET TERM ON ECHO OFF;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
BEGIN
  IF ^^role_count. < 1 THEN
    RAISE_APPLICATION_ERROR(-20109, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role. Request: "GRANT SQLT_USER_ROLE TO '||USER||';" to your DBA.');
  END IF;
  IF ^^lib_count. < 17 THEN
    RAISE_APPLICATION_ERROR(-20110, 'User "'||USER||'" lacks required "SQLT_USER_ROLE" role or SQLT is not properly installed. Review installation NN_*.log files.');
  END IF;
END;
/
WHENEVER SQLERROR CONTINUE;
PRO
PRO Parameter 1:
PRO SCRIPT name which contains SQL and its binds (required)
PRO
DEF script_with_sql = '^1';
PRO
PRO Paremeter 2:
PRO SQLTXPLAIN password (required)
PRO
DEF sqltxplain_password = '^2';
WHENEVER SQLERROR EXIT SQL.SQLCODE;
BEGIN
  IF '^^sqltxplain_password.' IS NULL THEN
    RAISE_APPLICATION_ERROR(-20104, 'No password specified for user SQLTXPLAIN');
  END IF;
  IF '^^sqltxplain_password.' LIKE '% %' THEN
    RAISE_APPLICATION_ERROR(-20105, 'Password for user SQLTXPLAIN cannot contain spaces');
  END IF;
END;
/
PRO
PRO Value passed to sqltxecute:
PRO ~~~~~~~~~~~~~~~~~~~~~~~~~~
PRO SCRIPT_WITH_SQL: "^^script_with_sql."
PRO
EXEC DBMS_APPLICATION_INFO.SET_MODULE('sqltxecute', 'script');
PRO
PRO ... please wait ...
PRO
PRO In case of a disconnect review sqltxecute.log
PRO
--
-- begin common
--
SET TERM OFF;
DEF _SQLPLUS_RELEASE
SET FEED OFF VER OFF SHOW OFF HEA ON LIN 1280 NEWP 1 PAGES 960 SQLC MIX TAB ON TRIMS ON TI OFF TIMI OFF SQLP SQL> BLO . RECSEP OFF APPI OFF SERVEROUT ON SIZE 1000000 FOR WOR;
SET SERVEROUT ON SIZE UNL FOR WOR;
COL connected_user NEW_V connected_user FOR A30;
SELECT user connected_user FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
VAR v_statement_id VARCHAR2(32);
WHENEVER SQLERROR EXIT SQL.SQLCODE;
SET TERM ON ECHO OFF;
PRO
PRO NOTE:
PRO If you get one of these errors it means SQLTXPLAIN is not installed:
PRO PLS-00201: identifier 'SQLTXPLAIN.SQLT$A' must be declared
PRO ORA-00904: "SQLTXPLAIN"."SQLT$A"."GET_STATEMENT_ID_C": invalid identifier;
PRO Review NN_*.log files created during installation and fix errors reported.
PRO
EXEC :v_statement_id := sqltxplain.sqlt$a.get_statement_id_c;
EXEC sqltxplain.sqlt$a.trace_on(:v_statement_id);
CL SCR;
PRO ... please wait ...
SET TERM OFF;
PRINT v_statement_id;
COL statement_id NEW_V statement_id FOR A32;
SELECT :v_statement_id statement_id FROM DUAL;
COL unique_id NEW_V unique_id FOR A32;
SELECT 'sqlt_s'||:v_statement_id unique_id FROM DUAL;
COL libraries FOR A64;
SELECT column_value libraries FROM TABLE(sqltxplain.sqlt$r.libraries_versions);
EXEC sqltxplain.sqlt$a.validate_tool_version('11.4.4.6');
COL install_date FOR A20;
SELECT sqltxplain.sqlt$a.get_param('install_date') install_date FROM DUAL;
COL host_name FOR A80;
SELECT sqltxplain.sqlt$a.get_host_name_short host_name FROM DUAL;
COL udump_path NEW_V udump_path FOR A256;
SELECT sqltxplain.sqlt$a.get_udump_full_path udump_path FROM DUAL;
COL bdump_path NEW_V bdump_path FOR A256;
SELECT sqltxplain.sqlt$a.get_bdump_full_path bdump_path FROM DUAL;
COL user_dump_dest FOR A256;
SELECT sqltxplain.sqlt$a.get_v$parameter('user_dump_dest') user_dump_dest FROM DUAL;
COL background_dump_dest FOR A256;
SELECT sqltxplain.sqlt$a.get_v$parameter('background_dump_dest') background_dump_dest FROM DUAL;
COL directories FOR A256;
WHENEVER SQLERROR CONTINUE;
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
EXEC sqltxplain.sqlt$a.reset_directories;
--
-- end common
--
COL file_with_output NEW_V file_with_output;
SELECT sqltxplain.sqlt$a.get_filename_with_output(:v_statement_id, '^^script_with_sql.') file_with_output FROM DUAL;
COL prev_sql_id NEW_V prev_sql_id;
COL prev_child_number NEW_V prev_child_number;
SELECT prev_sql_id, prev_child_number FROM sys.sqlt$_my_v$session;
EXEC sqltxplain.sqlt$i.xecute_begin(p_statement_id => :v_statement_id);
SET TERM ON;
PRO
PRO NOTE:
PRO You used the XECUTE method connected as ^^connected_user..
PRO
PRO In case of a session disconnect please verify the following:
PRO 1. There are no errors in sqltxecute.log or sqltxecute2.log.
PRO 2. Your SQL contains token "!^!^unique_id" within a comment.
PRO 3. Your SQL ends with a semi-colon ";".
PRO 4. You connected as the application user that issued original SQL.
PRO 5. Script ^^script_with_sql. can execute stand-alone connected as ^^connected_user.
PRO 6. User ^^connected_user. has been granted SQLT_USER_ROLE.
PRO
PRO In case of errors ORA-03113, ORA-03114 or ORA-07445 please just
PRO re-try this SQLT method. This tool handles some of the errors behind
PRO a disconnect when executed a second time.
PRO
PRO To actually diagnose the problem behind the disconnect, read ALERT
PRO log and provide referenced traces to Support. After the root cause
PRO of the disconnect is fixed then reset SQLT corresponding parameter.
PRO
PRO ... executing ^^script_with_sql. ...
PRO
PRO In case of a disconnect review sqltxecute2.log and ^^file_with_output.
PRO
SET TERM OFF ECHO ON HEA ON LIN 2000 PAGES 1000 TRIMS ON TI OFF TIMI OFF SERVEROUT OFF;
SPO ^^file_with_output.;
-- This file ^^file_with_output. contains the output of ^^script_with_sql.
--
-- If you want to include this spool file with output of your script into zip file,
-- or delete it automatically, use SQLT parameter xecute_script_output.
--
-- To permanently set this tool parameter connect as SQLTXPLAIN and issue:
-- SQL> EXEC sqlt$a.set_param('xecute_script_output', 'ZIP');
-- To temporarily set this tool parameter for a session connect as the application user and issue:
-- SQL> EXEC SQLTXPLAIN.sqlt$a.set_sess_param('xecute_script_output', 'ZIP');
--
-- Valid values are these 3 below. Default is KEEP.
-- ZIP (to include in zip file),
-- KEEP (to generate spool file and leave it in local directory without including it in zip file),
-- DELETE (to delete this spool file without including it in zip file).
--
SAVEPOINT sqlt_xecute_savepoint;
@^^script_with_sql.
SPO sqltxecute2.log;
SET TERM ON ECHO OFF;
PRO To monitor progress, login as SQLTXPLAIN into another session and execute:
PRO SQL> SELECT * FROM sqlt$_log_v;;
PRO
PRO ... collecting diagnostics details ...
PRO
PRO In case of a disconnect review sqltxecute2.log
PRO
SET TERM OFF;
L
SELECT prev_sql_id, prev_child_number FROM sys.sqlt$_my_v$session;
ROLLBACK TO SAVEPOINT sqlt_xecute_savepoint;
SELECT sql_text FROM sys.sqlt$_my_v$sql WHERE sql_id = '^^prev_sql_id.' AND child_number = TO_NUMBER('^^prev_child_number.');
SET SERVEROUT ON SIZE 1000000;
SET SERVEROUT ON SIZE UNL;
SET TERM ON;
EXEC sqltxplain.sqlt$i.xecute_end(p_statement_id => :v_statement_id, p_string => 'sqlt_s'||:v_statement_id, p_sql_id => '^^prev_sql_id.', p_child_number => '^^prev_child_number.', p_input_filename => '^^script_with_sql.', p_password => 'Y');
WHENEVER SQLERROR CONTINUE;
SET TERM OFF ECHO OFF FEED OFF VER OFF SHOW OFF HEA OFF LIN 2000 NEWP NONE PAGES 0 SQLC MIX TAB ON TRIMS ON TI OFF TIMI OFF ARRAY 100 NUMF "" SQLP SQL> SUF sql BLO . RECSEP OFF APPI OFF SERVEROUT ON SIZE 1000000 FOR TRU;
SET SERVEROUT ON SIZE UNL FOR TRU;
PRO No fatal errors!
COL column_value FOR A2000;
COL filename NEW_V filename FOR A256;
SPO OFF;
--
-- consumed by xtrcxec.sql:
DEF zipfile2 = '^^unique_id.';
--
-- begin common
--
@@sqltgetfile.sql REMOTE_DRIVER
@^^filename.
@@sqltgetfile.sql MAIN_REPORT
@@sqltgetfile.sql LITE_REPORT
@@sqltgetfile.sql README_REPORT_HTML
@@sqltgetfile.sql README_REPORT_TXT
@@sqltgetfile.sql METADATA_SCRIPT
@@sqltgetfile.sql SYSTEM_STATS_SCRIPT
@@sqltgetfile.sql SET_CBO_ENV_SCRIPT
@@sqltgetfile.sql CUSTOM_SQL_PROFILE
@@sqltgetfile.sql STA_REPORT_MEM
@@sqltgetfile.sql STA_SCRIPT_MEM
@@sqltgetfile.sql STA_REPORT_TXT
@@sqltgetfile.sql STA_SCRIPT_TXT
@@sqltgetfile.sql STA_REPORT_AWR
@@sqltgetfile.sql STA_SCRIPT_AWR
@@sqltgetfile.sql SQL_DETAIL_ACTIVE
@@sqltgetfile.sql SQL_MONITOR_ACTIVE
@@sqltgetfile.sql SQL_MONITOR_HTML
@@sqltgetfile.sql SQL_MONITOR_TEXT
@@sqltgetfile.sql 10053_EXPLAIN
@@sqltgetfile.sql 10053_EXTRACT
@@sqltgetfile.sql BDE_CHK_CBO_REPORT
@@sqltgetfile.sql IMPORT_SCRIPT
@@sqltgetfile.sql EXPORT_PARFILE
@@sqltgetfile.sql PLAN
@@sqltgetfile.sql 10053
@@sqltgetfile.sql FLUSH
@@sqltgetfile.sql PURGE
@@sqltgetfile.sql RESTORE
@@sqltgetfile.sql DEL_HGRM
@@sqltgetfile.sql TC_SQL
@@sqltgetfile.sql XPRESS_SH
@@sqltgetfile.sql XPRESS_SQL
@@sqltgetfile.sql SETUP
@@sqltgetfile.sql SEL
@@sqltgetfile.sql SEL_AUX
@@sqltgetfile.sql TCB_DRIVER
@^^filename.
@@sqltgetfile.sql EXPORT_DRIVER
@^^filename.
--
-- end common
--
COL file_10046_10053_udump NEW_V file_10046_10053_udump FOR A256;
SELECT NVL(file_10046_10053_udump, 'missing_file.txt') file_10046_10053_udump FROM sqltxplain.sqlt$_sql_statement WHERE statement_id = :v_statement_id;
@@sqltgetfile.sql 10046_10053_EXECUTE
@@sqltgetfile.sql TRACE_10046
@@sqltgetfile.sql TRACE_10053
@@sqltgetfile.sql TRCA_HTML
@@sqltgetfile.sql TRCA_TXT
@@sqltgetfile.sql TRCA_LOG
@@sqltgetfile.sql TRCA_PX_HTML
@@sqltgetfile.sql TRCA_PX_TXT
@@sqltgetfile.sql TRCA_PX_LOG
@@sqltgetfile.sql TKPROF_PX_DRIVER
@^^filename.
@@sqltgetfile.sql SCRIPT_OUTPUT_DRIVER
@^^filename.
SPO ^^unique_id._xecute.log;
GET sqltxecute.log
.
GET sqltxecute2.log
.
SPO OFF;
HOS cp ^^script_with_sql. q.sql
HOS copy ^^script_with_sql. q.sql
--
-- begin common
--
EXEC sqltxplain.sqlt$a.trace_off;
HOS tkprof ^^udump_path.*_S^^statement_id._SQLT_TRACE.trc ^^unique_id._sqlt_tkprof_nosort.txt >> sqltxhost.log
HOS tkprof ^^udump_path.*_S^^statement_id._SQLT_TRACE.trc ^^unique_id._sqlt_tkprof_sort.txt sort=prsela exeela fchela >> sqltxhost.log
HOS zip -mT ^^unique_id._log ^^unique_id._sqlt_tkprof_*.txt >> sqltxhost.log
--
HOS zip -jT ^^unique_id._opatch $ORACLE_HOME/cfgtoollogs/opatch/opatch* >> sqltxhost.log
HOS unzip -l ^^unique_id._opatch >> sqltxhost.log
--
HOS chmod 777 xpress.sh
HOS zip -mT ^^unique_id._tc ^^unique_id._system_stats.sql ^^unique_id._metadata.sql ^^unique_id._set_cbo_env.sql ^^unique_id._readme.txt >> sqltxhost.log
HOS zip -mT ^^unique_id._tc q.sql plan.sql 10053.sql flush.sql tc.sql xpress.sql xpress.sh setup.sql sel.sql sel_aux.sql >> sqltxhost.log
HOS zip -mT ^^unique_id._tc ^^unique_id._purge.sql ^^unique_id._restore.sql ^^unique_id._del_hgrm.sql >> sqltxhost.log
HOS zip -jT ^^unique_id._tc ^^unique_id._opatch.zip >> sqltxhost.log
HOS unzip -l ^^unique_id._tc >> sqltxhost.log
--
HOS zip -juT ^^unique_id._trc ^^udump_path.*_s^^statement_id._*.trc >> sqltxhost.log
HOS zip -juT ^^unique_id._trc ^^bdump_path.*_s^^statement_id._*.trc >> sqltxhost.log
HOS unzip -l ^^unique_id._trc >> sqltxhost.log
--
-- end common
--
HOS tkprof ^^udump_path.^^file_10046_10053_udump. ^^unique_id._tkprof_nosort.txt >> sqltxhost.log
HOS tkprof ^^udump_path.^^file_10046_10053_udump. ^^unique_id._tkprof_sort.txt sort=prsela exeela fchela >> sqltxhost.log
--
HOS zip -jT ^^unique_id._xecute ^^script_with_sql. >> sqltxhost.log
HOS zip -m ^^unique_id._xecute sqltxecute.log sqltxecute2.log missing_file.txt >> sqltxhost.log
HOS zip -d ^^unique_id._xecute sqltxecute.log sqltxecute2.log missing_file.txt >> sqltxhost.log
--
-- begin common
--
HOS unzip -l ^^unique_id._xecute >> sqltxhost.log
--
HOS echo "SIEBEL_ORA_BIND_PEEK" >> sqltxhost.log
HOS echo $SIEBEL_ORA_BIND_PEEK >> sqltxhost.log
HOS echo "ORACLE_HOME" >> sqltxhost.log
HOS echo $ORACLE_HOME >> sqltxhost.log
HOS echo "NLS_LANG" >> sqltxhost.log
HOS echo $NLS_LANG >> sqltxhost.log
HOS echo "UDUMP" >> sqltxhost.log
HOS ls -dl ^^udump_path. >> sqltxhost.log
HOS echo "BDUMP" >> sqltxhost.log
HOS ls -dl ^^bdump_path. >> sqltxhost.log
HOS echo "WHO" >> sqltxhost.log
HOS who >> sqltxhost.log
--
-- end common
--
HOS zip -mT ^^unique_id._log ^^unique_id._xecute.log sqltxhost.log
HOS zip -mT ^^unique_id._xecute ^^unique_id.*
--
SET TERM ON;
HOS unzip -l ^^unique_id._xecute
PRO File ^^unique_id._xecute.zip for ^^script_with_sql. has been created.
SET TERM OFF;
CL COL;
EXEC DBMS_APPLICATION_INFO.SET_MODULE(NULL, NULL);
UNDEFINE 1 2 lib_count role_count udump_path bdump_path connected_user statement_id unique_id filename prev_sql_id prev_child_number script_with_sql file_with_output file_10046_10053_udump;
UNDEFINE sqltxplain_password;
SET DEF ON TERM ON ECHO OFF FEED 6 VER ON SHOW OFF HEA ON LIN 80 NEWP 1 PAGES 14 SQLC MIX TAB ON TRIMS OFF TI OFF TIMI OFF ARRAY 15 NUMF "" SQLP SQL> SUF sql BLO . RECSEP WR APPI OFF SERVEROUT OFF;
SET ESC OFF SQLBL OFF;
PRO
PRO SQLTXECUTE completed.
