SPOOL 03_tacobj.log;
SET ECHO ON TERM OFF VER OFF NUMF "";
REM
REM $Header: 224270.1 tacobj.sql 11.4.4.2 2012/02/02 csierra $
REM
REM Copyright (c) 2000-2012, Oracle Corporation. All rights reserved.
REM
REM AUTHOR
REM   carlos.sierra@oracle.com
REM
REM SCRIPT
REM   tacobj.sql
REM
REM DESCRIPTION
REM   This script creates sequences, tables, and indexes owned by schema
REM   TRCANLZR
REM
REM PRE-REQUISITES
REM   1. This script must be executed connected as TRCANLZR
REM
REM PARAMETERS
REM   1. None
REM
REM EXECUTION
REM   1. Navigate to trca/install directory
REM   2. Start SQL*Plus connecting as TRCANLZR
REM   3. Execute script sqtacobj.sql
REM
REM EXAMPLE
REM   # cd trca/install
REM   # sqlplus trcanlzr
REM   SQL> start tacobj.sql
REM
REM NOTES
REM   1. This script is executed automatically by tacreate.sql
REM   2. For possible errors see tacobj.log file
REM   3. If you want to make all TRCANLZR staging tables permanent,
REM      answer P when asked for "temporary_or_permanent".
REM      Be aware that most large objects have no indexes.
REM
SET ECHO OFF TERM OFF;
-- begin common
DEF _SQLPLUS_RELEASE
SELECT USER FROM DUAL;
SELECT TO_CHAR(SYSDATE, 'YYYY-MM-DD HH24:MI:SS') current_time FROM DUAL;
SELECT * FROM v$version;
SELECT * FROM v$instance;
SELECT name, value FROM v$parameter2 WHERE name LIKE '%dump_dest';
SELECT directory_name||' '||directory_path directories FROM dba_directories WHERE directory_name LIKE 'SQLT$%' OR directory_name LIKE 'TRCA$%' ORDER BY 1;
-- end common
WHENEVER SQLERROR CONTINUE;
ALTER SESSION SET PLSQL_CODE_TYPE = INTERPRETED;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
ALTER SESSION SET NLS_NUMERIC_CHARACTERS = ".,";
ALTER SESSION SET NLS_LENGTH_SEMANTICS = CHAR;
DECLARE
  rdbms_release NUMBER;
BEGIN
  IF USER NOT IN ('TRCANLZR', 'SQLTXPLAIN') THEN
    RAISE_APPLICATION_ERROR(-20100, 'Install failed - TRCANLZR objects should be created connected as TRCANLZR or SQLTXPLAIN, not as '||USER);
  END IF;
  SELECT TO_NUMBER(SUBSTR(version, 1, INSTR(version, '.', 1, 2) - 1))
    INTO rdbms_release
    FROM v$instance;
  IF rdbms_release < 9.2 THEN
    RAISE_APPLICATION_ERROR(-20200, 'Install failed - TRCANLZR should be installed in 9i(9.2) or higher, not in '||rdbms_release);
  END IF;
END;
/
DEFINE on_commit_preserve_rows = "ON COMMIT PRESERVE ROWS";
DEFINE global_temporary        = "GLOBAL TEMPORARY";
COLUMN on_commit_preserve_rows NEW_VALUE on_commit_preserve_rows;
COLUMN global_temporary NEW_VALUE global_temporary;
SET TERM ON;
SELECT NULL global_temporary, NULL on_commit_preserve_rows
  FROM DUAL
 WHERE SUBSTR(TRIM(UPPER(NVL('&&temporary_or_permanent.', 'T'))), 1, 1) = 'P';
SET ECHO ON TERM OFF;

/* ------------------------------------------------------------------------- */

COL start_with NEW_V start_with NOPRI;
SELECT (NVL(ABS(MOD(SYS.DBMS_RANDOM.RANDOM, 80000)), 0) + 10000) start_with FROM DUAL;

CREATE SEQUENCE trca$_tool_execution_id_s START WITH &&start_with. NOCACHE;
CREATE SEQUENCE trca$_trace_id_s NOCACHE;
CREATE SEQUENCE trca$_cursor_id_s;
CREATE SEQUENCE trca$_statement_id_s;
CREATE SEQUENCE trca$_call_id_s;
CREATE SEQUENCE trca$_dep_id_s;
CREATE SEQUENCE trca$_exec_id_s;
CREATE SEQUENCE trca$_session_id_s;
CREATE SEQUENCE trca$_gap_id_s;
CREATE SEQUENCE trca$_header_id_s;

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_control (
  refresh_date               VARCHAR2(255),
  database_id                VARCHAR2(255),
  database_name              VARCHAR2(255),
  instance_id                VARCHAR2(255),
  instance_name              VARCHAR2(255),
  host_name                  VARCHAR2(255),
  platform                   VARCHAR2(255),
  rdbms_version              VARCHAR2(255),
  db_files                   VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_CONTROL.bad'
  LOGFILE 'TRCA_CONTROL.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_CONTROL.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_control');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- external dict table snaphot of sys.file$ generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_file (
  file#                      VARCHAR2(255),
  ts#                        VARCHAR2(255),
  relfile#                   VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_FILE.bad'
  LOGFILE 'TRCA_FILE.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_FILE.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_file');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of sys.file$ seeded by trca$t.refresh_trca$_dict_from
-- to be used by trca$_dba_extents only
CREATE GLOBAL TEMPORARY TABLE trca$_file$ (
  file#                      NUMBER,
  ts#                        NUMBER,
  relfile#                   NUMBER
) ON COMMIT PRESERVE ROWS;

/* ------------------------------------------------------------------------- */

-- external dict table snaphot of dba_segments generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_segments (
  uid#                       VARCHAR2(255),
  owner                      VARCHAR2(255),
  segment_name               VARCHAR2(255),
  partition_name             VARCHAR2(255),
  segment_type               VARCHAR2(255),
  tablespace_id              VARCHAR2(255),
  header_block               VARCHAR2(255),
  relative_fno               VARCHAR2(255),
  managed                    VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_SEGMENTS.bad'
  LOGFILE 'TRCA_SEGMENTS.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_SEGMENTS.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_segments');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_segments seeded by trca$t.refresh_trca$_dict_from
-- to be used by trca$_dba_extents only
CREATE GLOBAL TEMPORARY TABLE trca$_segments (
  uid#                       NUMBER,
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30),
  segment_type               VARCHAR2(20),
  tablespace_id              NUMBER,
  header_block               NUMBER,
  relative_fno               NUMBER,
  managed                    NUMBER
) ON COMMIT PRESERVE ROWS;

/* ------------------------------------------------------------------------- */

-- external dict table snaphot of sys.uet$ generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_extents_dm (
  relative_fno               VARCHAR2(255), -- segfile#
  header_block               VARCHAR2(255), -- segblock#
  tablespace_id              VARCHAR2(255), -- ts#
  relfile#                   VARCHAR2(255), -- file#
  block_id                   VARCHAR2(255), -- block#
  blocks                     VARCHAR2(255)  -- length
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_EXTENTS_DM.bad'
  LOGFILE 'TRCA_EXTENTS_DM.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_EXTENTS_DM.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_extents_dm');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

 -- dict snaphot of sys.uet$ seeded by trca$t.refresh_trca$_dict_from
 -- to be used by trca$_dba_extents only
 CREATE GLOBAL TEMPORARY TABLE trca$_extents_dm (
   relative_fno               NUMBER, -- segfile#
   header_block               NUMBER, -- segblock#
   tablespace_id              NUMBER, -- ts#
   relfile#                   NUMBER, -- file#
   block_id                   NUMBER, -- block#
   blocks                     NUMBER  -- length
 ) ON COMMIT PRESERVE ROWS;

/* ------------------------------------------------------------------------- */

-- external dict table snaphot of sys.x_$ktfbue generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_extents_lm (
  relative_fno               VARCHAR2(255), -- ktfbuesegfno
  header_block               VARCHAR2(255), -- ktfbuesegbno
  tablespace_id              VARCHAR2(255), -- ktfbuesegtsn
  relfile#                   VARCHAR2(255), -- ktfbuefno
  block_id                   VARCHAR2(255), -- ktfbuebno
  blocks                     VARCHAR2(255)  -- ktfbueblks
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_EXTENTS_LM.bad'
  LOGFILE 'TRCA_EXTENTS_LM.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_EXTENTS_LM.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_extents_lm');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

 -- dict snaphot of sys.x_$ktfbue seeded by trca$t.refresh_trca$_dict_from
 -- to be used by trca$_dba_extents only
 CREATE GLOBAL TEMPORARY TABLE trca$_extents_lm (
   relative_fno              NUMBER, -- ktfbuesegfno
   header_block              NUMBER, -- ktfbuesegbno
   tablespace_id             NUMBER, -- ktfbuesegtsn
   relfile#                  NUMBER, -- ktfbuefno
   block_id                  NUMBER, -- ktfbuebno
   blocks                    NUMBER  -- ktfbueblks
 ) ON COMMIT PRESERVE ROWS;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_extents seeded by trca$t.refresh_trca$_extents
CREATE TABLE trca$_extents (
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30),
  segment_type               VARCHAR2(20),
  file_id                    NUMBER NOT NULL,
  block_id_from              NUMBER NOT NULL,
  block_id_to                NUMBER NOT NULL
);

CREATE UNIQUE INDEX trca$_extents_u1 ON trca$_extents(file_id, block_id_from, block_id_to);

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_users seeded by trca$t.refresh_trca$_users
CREATE TABLE trca$_users (
  user_id                    NUMBER NOT NULL,
  user_name                  VARCHAR2(30)
);

CREATE UNIQUE INDEX trca$_users_u1 ON trca$_users(user_id);

/* ------------------------------------------------------------------------- */

-- Trace Analyzer configuration parameters
CREATE TABLE trca$_tool_parameter (
  id                         INTEGER      NOT NULL, -- pk
  hidden                     CHAR(1)      NOT NULL, -- (N)o, (Y)es
  user_updateable            CHAR(1)      NOT NULL, -- (N)o, (Y)es
  name                       VARCHAR2(32) NOT NULL,
  description                VARCHAR2(64) NOT NULL,
  value_type                 CHAR(1)      NOT NULL, -- (N)umber, (C)har
  value                      VARCHAR2(128),
  default_value              VARCHAR2(128),
  low_value                  NUMBER,
  high_value                 NUMBER,
  value1                     VARCHAR2(128),
  value2                     VARCHAR2(128),
  value3                     VARCHAR2(128),
  value4                     VARCHAR2(128),
  value5                     VARCHAR2(128),
  instructions               VARCHAR2(128)
);

CREATE UNIQUE INDEX trca$_tool_parameter_pk ON trca$_tool_parameter(id);
ALTER TABLE trca$_tool_parameter ADD (CONSTRAINT trca$_tool_parameter_pk PRIMARY KEY (id));

CREATE UNIQUE INDEX trca$_tool_parameter_u1 ON trca$_tool_parameter(name);

DECLARE
  ins_id INTEGER := 0;
  PROCEDURE ins (
    p_hidden          CHAR,
    p_user_updateable CHAR,
    p_name            VARCHAR2,
    p_description     VARCHAR2,
    p_value_type      CHAR,
    p_value           VARCHAR2,
    p_default_value   VARCHAR2,
    p_low_value       NUMBER,
    p_high_value      NUMBER,
    p_value1          VARCHAR2,
    p_value2          VARCHAR2,
    p_value3          VARCHAR2,
    p_value4          VARCHAR2,
    p_value5          VARCHAR2,
    p_instructions    VARCHAR2 )
  IS
  BEGIN
    ins_id := ins_id + 1;
    INSERT INTO trca$_tool_parameter VALUES (
      ins_id,
      p_hidden,
      p_user_updateable,
      p_name,
      p_description,
      p_value_type,
      p_value,
      p_default_value,
      p_low_value,
      p_high_value,
      p_value1,
      p_value2,
      p_value3,
      p_value4,
      p_value5,
      p_instructions );
  END;
BEGIN
  DELETE trca$_tool_parameter;
  --  hid  upd  name                         description                       typ  value                         default       low    high         val1  val2  val3  val4  val5  instructions
  --  ~~~  ~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ~~~~~~~~~~~~~ ~~~~~~ ~~~~~~~~~~~~ ~~~~~ ~~~~~ ~~~~~ ~~~~~ ~~~~~ ~~~~~~~~~~~~
  ins('Y', 'N', 'tool_owner',                'Tool Schema Owner',              'C', '&&tool_owner.',              NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, NULL);
  ins('Y', 'N', 'tool_name',                 'Tool Name',                      'C', 'Trace Analyzer',             NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, NULL);
  ins('Y', 'N', 'tool_version',              'Tool Version',                   'C', '11.4.4.2',                   NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, NULL);
  ins('Y', 'N', 'install_date',              'Tool Install Date',              'C', TO_CHAR(SYSDATE, 'YYYYMMDD'), NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'YYYYMMDD');
  ins('Y', 'N', 'interop_version',           'Inter Operability Version',      'N', '1',                          '1',          1,     9999,        NULL, NULL, NULL, NULL, NULL, '1-9999');

  ins('Y', 'N', 'dict_refresh_date',         'Tool Dict Refresh Date',         'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'YYYYMMDD');
  ins('Y', 'N', 'dict_refresh_days',         'Tool Dict Refresh Days',         'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, NULL);
  ins('Y', 'N', 'dict_database_id',          'Tool Dict Database Id',          'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$database.dbid');
  ins('Y', 'N', 'dict_database_name',        'Tool Dict Database Name',        'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$database.name');
  ins('Y', 'N', 'dict_instance_id',          'Tool Dict Instance Id',          'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.instance_number');
  ins('Y', 'N', 'dict_instance_name',        'Tool Dict Instance Name',        'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.instance_name');
  ins('Y', 'N', 'dict_host_name',            'Tool Dict Host Name',            'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.host_name');
  ins('Y', 'N', 'dict_platform',             'Tool Dict Platform',             'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'product_component_version.product');
  ins('Y', 'N', 'dict_rdbms_version',        'Tool Dict RDBMS Version',        'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.version');
  ins('Y', 'N', 'dict_db_files',             'Tool Dict DB Files',             'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$parameter2 db_files');

  ins('Y', 'N', 'tool_database_id',          'Tool Database Id',               'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$database.dbid');
  ins('Y', 'N', 'tool_database_name',        'Tool Database Name',             'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$database.name');
  ins('Y', 'N', 'tool_instance_id',          'Tool Instance Id',               'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.instance_number');
  ins('Y', 'N', 'tool_instance_name',        'Tool Instance Name',             'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.instance_name');
  ins('Y', 'N', 'tool_host_name',            'Tool Host Name',                 'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.host_name');
  ins('Y', 'N', 'tool_platform',             'Tool Platform',                  'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'product_component_version.product');
  ins('Y', 'N', 'tool_rdbms_version',        'Tool RDBMS Version',             'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$instance.version');
  ins('Y', 'N', 'tool_rdbms_version_short',  'Tool RDBMS Version Short',       'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'rdbms_version');
  ins('Y', 'N', 'tool_rdbms_release',        'Tool RDBMS Release',             'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'rdbms_version_short');
  ins('Y', 'N', 'tool_product_version',      'Tool Product Version',           'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'product_component_version.product.status');
  ins('Y', 'N', 'tool_db_files',             'Tool DB Files',                  'N', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'v$parameter2 db_files');

  ins('N', 'Y', 'top_sql_th',                'Top SQL Threshold',              'N', '10',                         '10',         0,     100,         NULL, NULL, NULL, NULL, NULL, '0-100');
  ins('N', 'Y', 'top_exec_th',               'Top Execution Threshold',        'N', '10',                         '10',         0,     100,         NULL, NULL, NULL, NULL, NULL, '0-100');
  ins('N', 'Y', 'hot_block_th',              'Hot Block Threshold',            'N', '5',                          '5',          0,     100,         NULL, NULL, NULL, NULL, NULL, '0-100');
  ins('N', 'Y', 'aggregate',                 'Aggregate',                      'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'perform_count_star',        'Perform COUNT(*)',               'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'count_star_th',             'COUNT(*) Threshold',             'N', '1000000',                    '1000000',    0,     999999999,   NULL, NULL, NULL, NULL, NULL, '0-999999999');
  ins('N', 'Y', 'errors_th',                 'Errors Threshold',               'N', '100',                        '100',        0,     1000,        NULL, NULL, NULL, NULL, NULL, '0-1000');
  ins('N', 'Y', 'gaps_th',                   'Gaps Threshold',                 'N', '100',                        '100',        0,     1000,        NULL, NULL, NULL, NULL, NULL, '0-1000');
  ins('N', 'Y', 'include_internal_sql',      'Include Internal SQL',           'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_non_top_sql',       'Include non-Top SQL',            'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_init_ora',          'Include Initialization Params',  'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_waits',             'Include Waits',                  'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_binds',             'Include Bind Variables',         'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_fetches',           'Include Fetch Calls',            'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_expl_plans',        'Include Explain Plans',          'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'include_segments',          'Include Segments',               'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'detail_non_top_sql',        'Detail non-top SQL',             'C', 'N',                          'N',          NULL,  NULL,        'N',  'Y',  NULL, NULL, NULL, 'N, Y');
  ins('N', 'Y', 'time_granularity',          'Time Granularity',               'N', '1000000',                    '1000000',    NULL,  NULL,        '1000000', '100', NULL, NULL, NULL, '1000000-100');
  ins('N', 'Y', 'wait_time_th',              'Wait Time Threshold',            'N', '0.01',                       '0.01',       0.0001,1.0,         NULL, NULL, NULL, NULL, NULL, '0.0001-1.0');
  ins('N', 'Y', 'response_time_th',          'Response Time Threshold',        'N', '0.1',                        '0.1',        0.001, 10.0,        NULL, NULL, NULL, NULL, NULL, '0.001-10.0');
  ins('N', 'Y', 'trace_file_max_size_bytes', 'Trace File Max Size in Bytes',   'N', '999999999',                  '999999999',  99999, 99999999999, NULL, NULL, NULL, NULL, NULL, '99999-999999999999');
  ins('N', 'Y', 'copy_file_max_size_bytes',  'Copy File Max Size in Bytes',    'N', '99999999',                   '99999999',   99999, 99999999999, NULL, NULL, NULL, NULL, NULL, '99999-999999999999');
  ins('N', 'Y', 'gen_html_report',           'Generate HTML Report',           'C', 'Y',                          'Y',          NULL, NULL,         'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'gen_text_report',           'Generate Text Report',           'C', 'Y',                          'Y',          NULL, NULL,         'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'split_10046_10053_trc',     'Split traces generated by SQLT', 'C', 'Y',                          'Y',          NULL, NULL,         'Y',  'N',  NULL, NULL, NULL, 'Y, N');
  ins('N', 'Y', 'gather_cbo_stats',          'Gather CBO Stats Staging Objs',  'N', '10',                         '10',         0,    100,          NULL, NULL, NULL, NULL, NULL, '0-100');
  ins('N', 'Y', 'capture_extents',           'Capture Extents in Repository',  'C', 'S',                          'S',          NULL,  NULL,        'S',  'P',  'N',  NULL, NULL, '(S)erial, (P)arallel, (N)o');
  ins('N', 'Y', 'refresh_dict_repository',   'Refresh Dictionary Repository',  'C', 'Y',                          'Y',          NULL,  NULL,        'Y',  'N',  NULL, NULL, NULL, 'Y, N');

  ins('N', 'N', 'input1_dir',                'Input Directory 1',              'C', 'TRCA$INPUT1',                'UDUMP',      NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'Set by trca/utl/tacdiri1.sql');
  ins('N', 'N', 'input2_dir',                'Input Directory 2',              'C', 'TRCA$INPUT2',                'BDUMP',      NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'Set by trca/utl/tacdiri2.sql');
  ins('N', 'N', 'stage_dir',                 'Stage Directory',                'C', 'TRCA$STAGE',                 'UDUMP',      NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'Set by trca/utl/tacdirs.sql');
  ins('Y', 'N', 'input1_directory',          'Input Directory Path 1',         'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'TRCA$INPUT1');
  ins('Y', 'N', 'input2_directory',          'Input Directory Path 2',         'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'TRCA$INPUT2');
  ins('Y', 'N', 'stage_directory',           'Stage Directory Path',           'C', NULL,                         NULL,         NULL,  NULL,        NULL, NULL, NULL, NULL, NULL, 'TRCA$STAGE');
END;
/

/* ------------------------------------------------------------------------- */

-- lookup table for oracle data types
CREATE TABLE trca$_data_type (
  id                          INTEGER      NOT NULL,
  name                        VARCHAR2(128),
  description                 VARCHAR2(128)
);

CREATE UNIQUE INDEX trca$_data_type_pk ON trca$_data_type(id);
ALTER TABLE trca$_data_type ADD (CONSTRAINT trca$_data_type_pk PRIMARY KEY (id));

CREATE OR REPLACE PROCEDURE ins (
  p_id   IN INTEGER,
  p_name IN VARCHAR2,
  p_desc IN VARCHAR2 )
IS
BEGIN
  INSERT INTO trca$_data_type (
    id,
    name,
    description
  ) VALUES (
    p_id,
    p_name,
    p_desc
  );
END;
/

SHOW ERRORS;

BEGIN
  ins(  1, 'VARCHAR2', 'char[n]');
  ins(  2, 'NUMBER', 'unsigned char[21]');
  ins(  3, 'signed INTEGER', 'signed char, short, int, long');
  ins(  4, 'FLOAT', 'float, double');
  ins(  5, 'NULL-terminated STRING', 'char[n+1]');
  ins(  6, 'VARNUM', 'char[22]');
  ins(  7, 'Packed Decimal Numeric (Cobol)', NULL);
  ins(  8, 'LONG', 'char[n]');
  ins(  9, 'VARCHAR', 'char[n+sizeof(short integer)]');
  ins( 10, 'Null/empty PCC Descriptor entry', NULL);
  ins( 11, 'Internal (native format) rowid', 'unsigned char[12]');
  ins( 12, 'DATE', 'char[7]');
  ins( 13, 'internal date format', NULL);
  ins( 14, 'internal julian format', NULL);
  ins( 15, 'VARRAW', 'unsigned char[n+sizeof(short integer)]');
  ins( 16, 'Date input format', NULL);
  ins( 17, 'Date output format', NULL);
  ins( 18, 'Date time zone', NULL);
  ins( 19, 'Day name', NULL);
  ins( 20, 'Date precision code', NULL);
  ins( 21, 'Single precision floating point', 'float');
  ins( 22, 'Double precision floating point', 'double');
  ins( 23, 'RAW', 'unsigned char[n]');
  ins( 24, 'LONG RAW', 'unsigned char[n]');
  ins( 29, 'internal integer', NULL);
  ins( 38, 'network form', NULL);
  ins( 39, 'native form', NULL);
  ins( 68, 'UNSIGNED INT', 'unsigned');
  ins( 69, 'ROWID', '10 bytes');
  ins( 77, 'archive op', NULL);
  ins( 78, 'media recovery start', NULL);
  ins( 79, 'media recovery record tablespace', NULL);
  ins( 80, 'media recovery get starting log sequence #', NULL);
  ins( 81, 'media recovery recover using offline log', NULL);
  ins( 82, 'media recovery cancel', NULL);
  ins( 83, 'version number', NULL);
  ins( 84, 'logon, w/extra information', NULL);
  ins( 85, 'OINIT, w/extra information', NULL);
  ins( 86, 'bundled call', NULL);
  ins( 87, 'array bind describe info', NULL);
  ins( 88, 'reserved for os2/msdos', NULL);
  ins( 89, 'loader buffer transfer', NULL);
  ins( 90, 'loader function call', NULL);
  ins( 91, 'Longer longs (char)', 'struct { ub4 len; char s[n]; }');
  ins( 92, 'Oracle session id', NULL);
  ins( 93, 'new network uac type', NULL);
  ins( 94, 'LONG VARCHAR', 'char[n+sizeof(integer)]');
  ins( 95, 'LONG VARRAW', 'unsigned char[n+sizeof(integer)]');
  ins( 96, 'CHAR', 'char[n]');
  ins( 97, 'CHARZ', 'char[n+1]');
  ins( 98, 'Dataatype for pisdef for deferred upi', NULL);
  ins( 99, 'RPC between transaction managers', NULL);
  ins(100, 'BINARY_FLOAT', '4 bytes');
  ins(101, 'BINARY_DOUBLE', '8 bytes');
  ins(102, 'REF cursor type', NULL);
  ins(103, 'direct path Export', NULL);
  ins(104, 'ROWID descriptor', 'OCIRowid');
  ins(105, 'label type (Trusted Oracle)', NULL);
  ins(106, 'OS label type (Trusted Oracle)', NULL);
  ins(107, 'for KOD', NULL);
  ins(108, 'NAMED DATATYPE', 'struct');
  ins(109, 'Internal Named Type', NULL);
  ins(110, 'REF', 'OCIRef');
  ins(111, 'REF', 'N/A');
  ins(112, 'Character LOB descriptor', 'OCILobLocator (see note 2)');
  ins(113, 'Binary LOB descriptor', 'OCILobLocator (see note 2)');
  ins(114, 'Binary FILE descriptor', 'OCILobLocator');
  ins(115, 'Character file LOB (CFILE)', NULL);
  ins(116, 'Result set type', NULL);
  ins(117, 'Reuse for Cursor /w describe info', NULL);
  ins(118, 'structure value type', NULL);
  ins(119, 'internal structure value type', NULL);
  ins(120, 'new version8 OAC', NULL);
  ins(122, 'named collection type (varray or nested table)', NULL);
  ins(123, 'named array type', NULL);
  ins(124, 'new v8 udsdef', NULL);
  ins(125, 'new v8 execute structure', NULL);
  ins(126, 'LOB and FILE operations except file create', NULL);
  ins(127, 'FILE create operation', NULL);
  ins(128, 'new v8 describe any', NULL);
  ins(129, 'Used for recursive open calls', NULL);
  ins(130, 'Datatype for pisdef for bundled PL/SQL calls', NULL);
  ins(131, 'transaction start, attach and detach operation', NULL);
  ins(132, 'transaction end and recover operation', NULL);
  ins(133, 'old describe callback', NULL);
  ins(134, 'Cursor close all piggyback function', NULL);
  ins(135, 'warning message', NULL);
  ins(136, 'Object form. Used on server side only', NULL);
  ins(137, 'Load Header', NULL);
  ins(138, 'Typed Object Header', NULL);
  ins(139, 'failover info', NULL);
  ins(140, 'V8 Session switching piggyback', NULL);
  ins(141, 'COR Header', NULL);
  ins(146, 'Character Lob Value', NULL);
  ins(147, 'Binary Lob Value', NULL);
  ins(148, 'v8 rxhdef', NULL);
  ins(149, 'name,pref', NULL);
  ins(150, 'New generic logon call', NULL);
  ins(151, 'keyword value pair', NULL);
  ins(152, 'cobol: display trailing', NULL);
  ins(153, 'cobol: display unsigned', NULL);
  ins(154, 'cobol: display overpunch', NULL);
  ins(155, 'OCI STRING type', 'OCIString');
  ins(156, 'OCI DATE type', 'OCIDate');
  ins(158, 'V8 descibe any', NULL);
  ins(159, 'top level descriptor for V8.0 describe any', NULL);
  ins(160, 'header descriptor for V8.0 describe any', NULL);
  ins(161, 'list descriptor for V8 describe any', NULL);
  ins(162, 'table descriptor for V8.0 describe any', NULL);
  ins(163, 'view descriptor for V8 describe any', NULL);
  ins(164, 'procedure descriptor for V8 describe any', NULL);
  ins(165, 'function descriptor for V8 describe any', NULL);
  ins(166, 'package descriptor for V8 describe any', NULL);
  ins(167, 'synonym descriptor for V8 describe any', NULL);
  ins(168, 'sequence descriptor for V8 describe any', NULL);
  ins(169, 'column descriptor for V8 describe any', NULL);
  ins(170, 'argument descriptor for V8 describe any', NULL);
  ins(171, 'Oracle Transaction service: Commit remote sites', NULL);
  ins(172, 'display overpunch leading', NULL);
  ins(173, 'function descriptor for V8 describe any', NULL);
  ins(174, 'AQ Enqueue structure', NULL);
  ins(175, 'fast keyword value pair', NULL);
  ins(176, 'AQ Dequeue structure', NULL);
  ins(177, 'AQ Message properties structure', NULL);
  ins(178, 'TIME', NULL);
  ins(179, 'TIME WITH TIME ZONE', NULL);
  ins(180, 'TIMESTAMP', '11 bytes');
  ins(181, 'TIMESTAMP WITH TIME ZONE', '13 bytes');
  ins(182, 'INTERVAL YEAR TO MONTH', '5 bytes');
  ins(183, 'INTERVAL DAY TO SECOND', '11 bytes');
  ins(184, 'ANSI DATE descriptor', 'OCIDateTime');
  ins(185, 'TIME in structured format', NULL);
  ins(186, 'TIME WITH TIME ZONE in structured format', NULL);
  ins(187, 'TIMESTAMP descriptor', 'OCIDateTime');
  ins(188, 'TIMESTAMP WITH TIME ZONE descriptor', 'OCIDateTime');
  ins(189, 'INTERVAL YEAR TO MONTH descriptor', 'OCIInterval');
  ins(190, 'INTERVAL DAY TO SECOND descriptor', 'OCIInterval');
  ins(191, 'LDI input format', NULL);
  ins(192, 'LDI output forma', NULL);
  ins(193, 'Remote archive file server', NULL);
  ins(194, 'V8.1 row header definition', NULL);
  ins(195, 'Desriptor rep. for DTYCLOB - internal use only', NULL);
  ins(196, 'Descriptor rep for DTYBLOB - internal use only', NULL);
  ins(197, 'Descriptor rep for DTYBFIL - internal use only', NULL);
  ins(198, 'kernel programmatic notification', NULL);
  ins(199, 'notification registration info', NULL);
  ins(200, 'Database descriptor for V8 describe any', NULL);
  ins(201, 'Schema descriptor for V8 describe any', NULL);
  ins(202, 'KPCDS structur for 8.1 onward', NULL);
  ins(203, 'Header descriptor for 8.1 onward', NULL);
  ins(204, 'table descriptor for 8.1 onward', NULL);
  ins(205, 'future use', NULL);
  ins(206, 'AQ message properties structure for Oracle 8.1', NULL);
  ins(207, 'V8.1 oerdef structure', NULL);
  ins(208, 'UROWID', '3950 bytes');
  ins(209, 'partial sort record for parallel aggregates', NULL);
  ins(210, 'aq listen', NULL);
  ins(211, 'OTs: Commit remote sites for version >= 8.1.3.0.0', NULL);
  ins(214, 'for object transfer', NULL);
  ins(215, 'sb4 IDL piece', NULL);
  ins(216, 'ub2 IDL piece', NULL);
  ins(217, 'ub1 IDL piece', NULL);
  ins(218, 'txt IDL piece', NULL);
  ins(219, 'segment of sb4 IDL pieces', NULL);
  ins(220, 'segment of ub2 IDL pieces', NULL);
  ins(221, 'segment of ub1 IDL pieces', NULL);
  ins(222, 'segment of txt IDL pieces', NULL);
  ins(223, 'top most structure for IDL piece (diana or pcode)', NULL);
  ins(224, 'structure for transfering single dependency', NULL);
  ins(225, 'dependency segment', NULL);
  ins(226, 'array of arrays of dependencies', NULL);
  ins(227, 'KOD operations post 8.1', NULL);
  ins(228, 'Direct Path Prepare descriptor', NULL);
  ins(229, 'Direct Path Load Stream descriptor', NULL);
  ins(230, 'Direct Path Misc Operations descriptor', NULL);
  ins(231, 'TIMESTAMP WITH LOCAL TIME ZONE', '11 bytes');
  ins(232, 'TIMESTAMP WITH LOCAL TIME ZONE descriptor', 'OCIDateTime');
  ins(249, 'nchar lob', NULL);
  ins(250, 'pl/sql record (or %rowtype)', NULL);
  ins(251, 'pl/sql indexed table', NULL);
  ins(252, 'pl/sql boolean', NULL);
END;
/

DROP PROCEDURE ins;

/* ------------------------------------------------------------------------- */

-- same content than v$event_name as per 11.2.0.1
CREATE TABLE trca$_event_name (
  event#                      INTEGER      NOT NULL, -- pk
  name                        VARCHAR2(64) NOT NULL,
  wait_class                  VARCHAR2(64) NOT NULL,
  idle                        CHAR(1)      NOT NULL, -- Y/N
  parameter1                  VARCHAR2(64),
  parameter2                  VARCHAR2(64),
  parameter3                  VARCHAR2(64)
);

CREATE UNIQUE INDEX trca$_event_name_pk ON trca$_event_name(event#);
ALTER TABLE trca$_event_name ADD (CONSTRAINT trca$_event_name_pk PRIMARY KEY (event#));

CREATE OR REPLACE PROCEDURE ins (
  p_event#     IN INTEGER,
  p_name       IN VARCHAR2,
  p_wait_class IN VARCHAR2,
  p_idle       IN VARCHAR2,
  p_parameter1 IN VARCHAR2,
  p_parameter2 IN VARCHAR2,
  p_parameter3 IN VARCHAR2 )
IS
BEGIN
  INSERT INTO trca$_event_name (
    event#,
    name,
    wait_class,
    idle,
    parameter1,
    parameter2,
    parameter3
  ) VALUES (
    p_event#,
    p_name,
    p_wait_class,
    p_idle,
    p_parameter1,
    p_parameter2,
    p_parameter3
  );
END;
/

SET DEF OFF;
BEGIN
  DELETE trca$_event_name;
  ins(   0, 'null event'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(   1, 'pmon timer'                                                 , 'Idle'           , 'Y', 'duration'                                , ''                           , ''                       );
  ins(   2, 'logout restrictor'                                          , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins(   3, 'VKTM Logical Idle Wait'                                     , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(   4, 'VKTM Init Wait for GSGA'                                    , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(   5, 'IORM Scheduler Slave Idle Wait'                             , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(   6, 'Parameter File I/O'                                         , 'User I/O'       , 'N', 'blkno'                                   , '#blks'                      , 'read/write'             );
  ins(   7, 'rdbms ipc message'                                          , 'Idle'           , 'Y', 'timeout'                                 , ''                           , ''                       );
  ins(   8, 'remote db operation'                                        , 'Network'        , 'N', 'clientid'                                , 'operation'                  , 'timeout'                );
  ins(   9, 'remote db file read'                                        , 'Network'        , 'N', 'clientid'                                , 'count'                      , 'intr'                   );
  ins(  10, 'remote db file write'                                       , 'Network'        , 'N', 'clientid'                                , 'count'                      , 'intr'                   );
  ins(  11, 'Disk file operations I/O'                                   , 'User I/O'       , 'N', 'FileOperation'                           , 'fileno'                     , 'filetype'               );
  ins(  12, 'Disk file I/O Calibration'                                  , 'User I/O'       , 'N', 'count'                                   , ''                           , ''                       );
  ins(  13, 'Disk file Mirror Read'                                      , 'User I/O'       , 'N', 'fileno'                                  , 'blkno'                      , 'filetype'               );
  ins(  14, 'Disk file Mirror/Media Repair Write'                        , 'User I/O'       , 'N', 'fileno'                                  , 'blkno'                      , 'filetype'               );
  ins(  15, 'direct path sync'                                           , 'User I/O'       , 'N', 'File number'                             , 'Flags'                      , ''                       );
  ins(  16, 'Datapump dump file I/O'                                     , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  17, 'dbms_file_transfer I/O'                                     , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  18, 'DG Broker configuration file I/O'                           , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  19, 'Data file init write'                                       , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  20, 'Log file init write'                                        , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  21, 'Log archive I/O'                                            , 'System I/O'     , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  22, 'RMAN backup & recovery I/O'                                 , 'System I/O'     , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  23, 'Standby redo I/O'                                           , 'System I/O'     , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  24, 'Network file transfer'                                      , 'System I/O'     , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins(  25, 'Backup: MML initialization'                                 , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  26, 'Backup: MML v1 open backup piece'                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  27, 'Backup: MML v1 read backup piece'                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  28, 'Backup: MML v1 write backup piece'                          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  29, 'Backup: MML v1 close backup piece'                          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  30, 'Backup: MML v1 query backup piece'                          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  31, 'Backup: MML v1 delete backup piece'                         , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  32, 'Backup: MML create a backup piece'                          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  33, 'Backup: MML commit backup piece'                            , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  34, 'Backup: MML command to channel'                             , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  35, 'Backup: MML shutdown'                                       , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  36, 'Backup: MML obtain textual error'                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  37, 'Backup: MML query backup piece'                             , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  38, 'Backup: MML extended initialization'                        , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  39, 'Backup: MML read backup piece'                              , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  40, 'Backup: MML delete backup piece'                            , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  41, 'Backup: MML restore backup piece'                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  42, 'Backup: MML write backup piece'                             , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  43, 'Backup: MML proxy initialize backup'                        , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  44, 'Backup: MML proxy cancel'                                   , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  45, 'Backup: MML proxy commit backup piece'                      , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  46, 'Backup: MML proxy session end'                              , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  47, 'Backup: MML datafile proxy backup?'                         , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  48, 'Backup: MML datafile proxy restore?'                        , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  49, 'Backup: MML proxy initialize restore'                       , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  50, 'Backup: MML proxy start data movement'                      , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  51, 'Backup: MML data movement done?'                            , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  52, 'Backup: MML proxy prepare to start'                         , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  53, 'Backup: MML obtain a direct buffer'                         , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  54, 'Backup: MML release a direct buffer'                        , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  55, 'Backup: MML get base address'                               , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  56, 'Backup: MML query for direct buffers'                       , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  57, 'io done'                                                    , 'System I/O'     , 'N', 'msg ptr'                                 , ''                           , ''                       );
  ins(  58, 'i/o slave wait'                                             , 'Idle'           , 'Y', 'msg ptr'                                 , ''                           , ''                       );
  ins(  59, 'RMAN Disk slave I/O'                                        , 'System I/O'     , 'N', 'wait count'                              , 'wait flags'                 , 'timeout'                );
  ins(  60, 'RMAN Tape slave I/O'                                        , 'System I/O'     , 'N', 'tape operation'                          , 'operation flags'            , 'timeout'                );
  ins(  61, 'DBWR slave I/O'                                             , 'System I/O'     , 'N', 'wait count'                              , 'wait flags'                 , 'timeout'                );
  ins(  62, 'LGWR slave I/O'                                             , 'System I/O'     , 'N', 'wait count'                              , 'wait flags'                 , 'timeout'                );
  ins(  63, 'Archiver slave I/O'                                         , 'System I/O'     , 'N', 'wait count'                              , 'wait flags'                 , 'timeout'                );
  ins(  64, 'VKRM Idle'                                                  , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(  65, 'wait for unread message on broadcast channel'               , 'Idle'           , 'Y', 'channel context'                         , 'channel handle'             , ''                       );
  ins(  66, 'wait for unread message on multiple broadcast channels'     , 'Idle'           , 'Y', 'channel context'                         , 'channel handle count'       , ''                       );
  ins(  67, 'class slave wait'                                           , 'Idle'           , 'Y', 'slave id'                                , ''                           , ''                       );
  ins(  68, 'KSV master wait'                                            , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(  69, 'PING'                                                       , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(  70, 'os thread startup'                                          , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins(  71, 'watchdog main loop'                                         , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(  72, 'DIAG idle wait'                                             , 'Idle'           , 'Y', 'component'                               , 'where'                      , 'wait time(millisec)'    );
  ins(  73, 'ges remote message'                                         , 'Idle'           , 'Y', 'waittime'                                , 'loop'                       , 'p3'                     );
  ins(  74, 'gcs remote message'                                         , 'Idle'           , 'Y', 'waittime'                                , 'poll'                       , 'event'                  );
  ins(  75, 'heartbeat monitor sleep'                                    , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins(  76, 'GCR sleep'                                                  , 'Idle'           , 'Y', 'lock retry count'                        , ''                           , ''                       );
  ins(  77, 'SGA: MMAN sleep for component shrink'                       , 'Idle'           , 'Y', 'component id'                            , 'current size'               , 'target size'            );
  ins(  78, 'retry contact SCN lock master'                              , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins(  79, 'control file sequential read'                               , 'System I/O'     , 'N', 'file#'                                   , 'block#'                     , 'blocks'                 );
  ins(  80, 'control file single write'                                  , 'System I/O'     , 'N', 'file#'                                   , 'block#'                     , 'blocks'                 );
  ins(  81, 'control file parallel write'                                , 'System I/O'     , 'N', 'files'                                   , 'block#'                     , 'requests'               );
  ins(  82, 'control file backup creation'                               , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  83, 'Shared IO Pool Memory'                                      , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins(  84, 'Shared IO Pool IO Completion'                               , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins(  85, 'enq: PW - flush prewarm buffers'                            , 'Application'    , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(  86, 'latch: cache buffers chains'                                , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins(  87, 'free buffer waits'                                          , 'Configuration'  , 'N', 'file#'                                   , 'block#'                     , 'set-id#'                );
  ins(  88, 'local write wait'                                           , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins(  89, 'checkpoint completed'                                       , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins(  90, 'write complete waits'                                       , 'Configuration'  , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins(  91, 'write complete waits: flash cache'                          , 'Configuration'  , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins(  92, 'buffer read retry'                                          , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins(  93, 'buffer busy waits'                                          , 'Concurrency'    , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins(  94, 'gc buffer busy acquire'                                     , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins(  95, 'gc buffer busy release'                                     , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins(  96, 'read by other session'                                      , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins(  97, 'multiple dbwriter suspend/resume for file offline'          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins(  98, 'recovery read'                                              , 'System I/O'     , 'N', ''                                        , ''                           , ''                       );
  ins(  99, 'pi renounce write complete'                                 , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins( 100, 'db flash cache single block physical read'                  , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 101, 'db flash cache multiblock physical read'                    , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 102, 'db flash cache write'                                       , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 103, 'db flash cache invalidate wait'                             , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins( 104, 'db flash cache dynamic disabling wait'                      , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 105, 'enq: RO - contention'                                       , 'Application'    , 'N', 'name|mode'                               , '2'                          , '0'                      );
  ins( 106, 'enq: RO - fast object reuse'                                , 'Application'    , 'N', 'name|mode'                               , '2'                          , '0'                      );
  ins( 107, 'enq: KO - fast object checkpoint'                           , 'Application'    , 'N', 'name|mode'                               , '2'                          , '0'                      );
  ins( 108, 'MRP redo arrival'                                           , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 109, 'RFS sequential i/o'                                         , 'System I/O'     , 'N', ''                                        , ''                           , ''                       );
  ins( 110, 'RFS random i/o'                                             , 'System I/O'     , 'N', ''                                        , ''                           , ''                       );
  ins( 111, 'RFS write'                                                  , 'System I/O'     , 'N', ''                                        , ''                           , ''                       );
  ins( 112, 'ARCH wait for net re-connect'                               , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 113, 'LGWR wait on ATTACH'                                        , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 114, 'ARCH wait on ATTACH'                                        , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 115, 'ARCH wait for netserver start'                              , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 116, 'LNS wait on ATTACH'                                         , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 117, 'LNS wait on SENDREQ'                                        , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 118, 'LNS wait on DETACH'                                         , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 119, 'LGWR wait on SENDREQ'                                       , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 120, 'LGWR wait on DETACH'                                        , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 121, 'ARCH wait on SENDREQ'                                       , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 122, 'ARCH wait on DETACH'                                        , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 123, 'ARCH wait for netserver init 2'                             , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 124, 'LNS wait on LGWR'                                           , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 125, 'LGWR wait on LNS'                                           , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 126, 'ARCH wait for flow-control'                                 , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 127, 'ARCH wait for netserver detach'                             , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 128, 'LNS ASYNC archive log'                                      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 129, 'LNS ASYNC dest activation'                                  , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 130, 'LNS ASYNC end of log'                                       , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 131, 'log file sequential read'                                   , 'System I/O'     , 'N', 'log#'                                    , 'block#'                     , 'blocks'                 );
  ins( 132, 'log file single write'                                      , 'System I/O'     , 'N', 'log#'                                    , 'block#'                     , 'blocks'                 );
  ins( 133, 'log file parallel write'                                    , 'System I/O'     , 'N', 'files'                                   , 'blocks'                     , 'requests'               );
  ins( 134, 'latch: redo writing'                                        , 'Configuration'  , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 135, 'latch: redo copy'                                           , 'Configuration'  , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 136, 'log buffer space'                                           , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 137, 'log file switch (checkpoint incomplete)'                    , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 138, 'log file switch (private strand flush incomplete)'          , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 139, 'log file switch (archiving needed)'                         , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 140, 'switch logfile command'                                     , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 141, 'log file switch completion'                                 , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 142, 'log file sync'                                              , 'Commit'         , 'N', 'buffer#'                                 , 'sync scn'                   , ''                       );
  ins( 143, 'simulated log write delay'                                  , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 144, 'LGWR real time apply sync'                                  , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 145, 'db file sequential read'                                    , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , 'blocks'                 );
  ins( 146, 'db file scattered read'                                     , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , 'blocks'                 );
  ins( 147, 'db file single write'                                       , 'User I/O'       , 'N', 'file#'                                   , 'block#'                     , 'blocks'                 );
  ins( 148, 'db file parallel write'                                     , 'System I/O'     , 'N', 'requests'                                , 'interrupt'                  , 'timeout'                );
  ins( 149, 'db file async I/O submit'                                   , 'System I/O'     , 'N', 'requests'                                , 'interrupt'                  , 'timeout'                );
  ins( 150, 'db file parallel read'                                      , 'User I/O'       , 'N', 'files'                                   , 'blocks'                     , 'requests'               );
  ins( 151, 'enq: MV - datafile move'                                    , 'Administrative' , 'N', 'name|mode'                               , 'type'                       , 'file #'                 );
  ins( 152, 'gc current request'                                         , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'id#'                    );
  ins( 153, 'gc cr request'                                              , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 154, 'gc cr disk request'                                         , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 155, 'gc cr multi block request'                                  , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 156, 'gc current multi block request'                             , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'id#'                    );
  ins( 157, 'gc block recovery request'                                  , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 158, 'gc cr block 2-way'                                          , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 159, 'gc cr block 3-way'                                          , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 160, 'gc cr block busy'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 161, 'gc cr block congested'                                      , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 162, 'gc cr failure'                                              , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 163, 'gc cr block lost'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 164, 'gc cr block unknown'                                        , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 165, 'gc current block 2-way'                                     , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 166, 'gc current block 3-way'                                     , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 167, 'gc current block busy'                                      , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 168, 'gc current block congested'                                 , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 169, 'gc current retry'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 170, 'gc current block lost'                                      , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 171, 'gc current split'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 172, 'gc current block unknown'                                   , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 173, 'gc cr grant 2-way'                                          , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 174, 'gc cr grant busy'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 175, 'gc cr grant congested'                                      , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 176, 'gc cr grant unknown'                                        , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 177, 'gc cr disk read'                                            , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 178, 'gc current grant 2-way'                                     , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 179, 'gc current grant busy'                                      , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 180, 'gc current grant congested'                                 , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 181, 'gc current grant unknown'                                   , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 182, 'gc freelist'                                                , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 183, 'gc remaster'                                                , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 184, 'gc quiesce'                                                 , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 185, 'gc object scan'                                             , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 186, 'gc recovery'                                                , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 187, 'gc flushed buffer'                                          , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 188, 'gc current cancel'                                          , 'Cluster'        , 'N', 'le'                                      , ''                           , ''                       );
  ins( 189, 'gc cr cancel'                                               , 'Cluster'        , 'N', 'le'                                      , ''                           , ''                       );
  ins( 190, 'gc assume'                                                  , 'Cluster'        , 'N', 'le'                                      , ''                           , ''                       );
  ins( 191, 'gc domain validation'                                       , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 192, 'gc recovery free'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 193, 'gc recovery quiesce'                                        , 'Cluster'        , 'N', 'file#'                                   , 'block#'                     , 'class#'                 );
  ins( 194, 'gc claim'                                                   , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 195, 'gc cancel retry'                                            , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 196, 'direct path read'                                           , 'User I/O'       , 'N', 'file number'                             , 'first dba'                  , 'block cnt'              );
  ins( 197, 'direct path read temp'                                      , 'User I/O'       , 'N', 'file number'                             , 'first dba'                  , 'block cnt'              );
  ins( 198, 'direct path write'                                          , 'User I/O'       , 'N', 'file number'                             , 'first dba'                  , 'block cnt'              );
  ins( 199, 'direct path write temp'                                     , 'User I/O'       , 'N', 'file number'                             , 'first dba'                  , 'block cnt'              );
  ins( 200, 'parallel recovery slave idle wait'                          , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 201, 'LogMiner builder: idle'                                     , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 202, 'LogMiner builder: memory'                                   , 'Queueing'       , 'N', 'Session ID'                              , ''                           , ''                       );
  ins( 203, 'LogMiner builder: DDL'                                      , 'Queueing'       , 'N', 'Session ID'                              , ''                           , ''                       );
  ins( 204, 'LogMiner builder: branch'                                   , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 205, 'LogMiner preparer: idle'                                    , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 206, 'LogMiner preparer: memory'                                  , 'Queueing'       , 'N', 'Session ID'                              , ''                           , ''                       );
  ins( 207, 'LogMiner reader: buffer'                                    , 'Queueing'       , 'N', 'Session ID'                              , ''                           , ''                       );
  ins( 208, 'LogMiner reader: log (idle)'                                , 'Idle'           , 'Y', 'Session ID'                              , 'Thread'                     , 'Sequence'               );
  ins( 209, 'LogMiner reader: redo (idle)'                               , 'Idle'           , 'Y', 'Session ID'                              , 'Thread'                     , 'Sequence'               );
  ins( 210, 'LogMiner client: transaction'                               , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 211, 'LogMiner: other'                                            , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 212, 'LogMiner: activate'                                         , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 213, 'LogMiner: reset'                                            , 'Idle'           , 'Y', 'Caller ID'                               , ''                           , ''                       );
  ins( 214, 'LogMiner: find session'                                     , 'Idle'           , 'Y', 'Session ID'                              , ''                           , ''                       );
  ins( 215, 'LogMiner: internal'                                         , 'Idle'           , 'Y', 'Caller ID'                               , ''                           , ''                       );
  ins( 216, 'Logical Standby Apply Delay'                                , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 217, 'wait for possible quiesce finish'                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 218, 'parallel recovery coordinator waits for slave cleanup'      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 219, 'flashback log file write'                                   , 'System I/O'     , 'N', 'log#'                                    , 'block#'                     , 'Bytes'                  );
  ins( 220, 'flashback log file read'                                    , 'System I/O'     , 'N', 'log#'                                    , 'block#'                     , 'Bytes'                  );
  ins( 221, 'flashback buf free by RVWR'                                 , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 222, 'flashback log file sync'                                    , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 223, 'parallel recovery control message reply'                    , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 224, 'cell smart table scan'                                      , 'User I/O'       , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 225, 'cell smart index scan'                                      , 'User I/O'       , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 226, 'cell statistics gather'                                     , 'User I/O'       , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 227, 'cell smart incremental backup'                              , 'System I/O'     , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 228, 'cell smart file creation'                                   , 'User I/O'       , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 229, 'cell smart restore from backup'                             , 'System I/O'     , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 230, 'parallel recovery slave next change'                        , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 231, 'concurrent I/O completion'                                  , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 232, 'datafile copy range completion'                             , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 233, 'enq: TM - contention'                                       , 'Application'    , 'N', 'name|mode'                               , 'object #'                   , 'table/partition'        );
  ins( 234, 'enq: ST - contention'                                       , 'Configuration'  , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 235, 'undo segment extension'                                     , 'Configuration'  , 'N', 'segment#'                                , ''                           , ''                       );
  ins( 236, 'undo segment tx slot'                                       , 'Configuration'  , 'N', 'segment#'                                , ''                           , ''                       );
  ins( 237, 'switch undo - offline'                                      , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 238, 'alter rbs offline'                                          , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 239, 'enq: TX - row lock contention'                              , 'Application'    , 'N', 'name|mode'                               , 'usn<<16 | slot'             , 'sequence'               );
  ins( 240, 'enq: TX - allocate ITL entry'                               , 'Configuration'  , 'N', 'name|mode'                               , 'usn<<16 | slot'             , 'sequence'               );
  ins( 241, 'enq: TX - index contention'                                 , 'Concurrency'    , 'N', 'name|mode'                               , 'usn<<16 | slot'             , 'sequence'               );
  ins( 242, 'enq: TW - contention'                                       , 'Administrative' , 'N', 'name|mode'                               , '0'                          , 'operation'              );
  ins( 243, 'PX Deq: Txn Recovery Start'                                 , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 244, 'PX Deq: Txn Recovery Reply'                                 , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 245, 'latch: Undo Hint Latch'                                     , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 246, 'statement suspended, wait error to be cleared'              , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 247, 'latch: In memory undo latch'                                , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 248, 'latch: MQL Tracking Latch'                                  , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 249, 'fbar timer'                                                 , 'Idle'           , 'Y', 'sleep time'                              , 'failed'                     , ''                       );
  ins( 250, 'Archive Manager file transfer I/O'                          , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins( 251, 'securefile chain update'                                    , 'Concurrency'    , 'N', 'seghdr'                                  , 'fsb'                        , ''                       );
  ins( 252, 'enq: HW - contention'                                       , 'Configuration'  , 'N', 'name|mode'                               , 'table space #'              , 'block'                  );
  ins( 253, 'enq: SS - contention'                                       , 'Configuration'  , 'N', 'name|mode'                               , 'tablespace #'               , 'dba'                    );
  ins( 254, 'sort segment request'                                       , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 255, 'smon timer'                                                 , 'Idle'           , 'Y', 'sleep time'                              , 'failed'                     , ''                       );
  ins( 256, 'PX Deq: Metadata Update'                                    , 'Idle'           , 'Y', 'ktelc_wait1s'                            , ''                           , ''                       );
  ins( 257, 'Space Manager: slave idle wait'                             , 'Idle'           , 'Y', 'Slave ID'                                , ''                           , ''                       );
  ins( 258, 'enq: SQ - contention'                                       , 'Configuration'  , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 259, 'PX Deq: Index Merge Reply'                                  , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 260, 'PX Deq: Index Merge Execute'                                , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 261, 'PX Deq: Index Merge Close'                                  , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 262, 'enq: HV - contention'                                       , 'Concurrency'    , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 263, 'PX Deq: kdcph_mai'                                          , 'Idle'           , 'Y', 'kdcph_mai'                               , ''                           , ''                       );
  ins( 264, 'PX Deq: kdcphc_ack'                                         , 'Idle'           , 'Y', 'kdcphc_ack'                              , ''                           , ''                       );
  ins( 265, 'index (re)build online start'                               , 'Administrative' , 'N', 'object'                                  , 'mode'                       , 'wait'                   );
  ins( 266, 'index (re)build online cleanup'                             , 'Administrative' , 'N', 'object'                                  , 'mode'                       , 'wait'                   );
  ins( 267, 'index (re)build online merge'                               , 'Administrative' , 'N', 'object'                                  , 'mode'                       , 'wait'                   );
  ins( 268, 'securefile direct-read completion'                          , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 269, 'securefile direct-write completion'                         , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 270, 'SecureFile mutex'                                           , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins( 271, 'enq: WG - lock fso'                                         , 'Concurrency'    , 'N', 'name|mode'                               , 'kdlw lobid first half'      , 'kdlw lobid sec half'    );
  ins( 272, 'latch: row cache objects'                                   , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 273, 'row cache lock'                                             , 'Concurrency'    , 'N', 'cache id'                                , 'mode'                       , 'request'                );
  ins( 274, 'row cache read'                                             , 'Concurrency'    , 'N', 'cache id'                                , 'address'                    , 'times'                  );
  ins( 275, 'libcache interrupt action by LCK'                           , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins( 276, 'cursor: mutex X'                                            , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 277, 'cursor: mutex S'                                            , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 278, 'cursor: pin X'                                              , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 279, 'cursor: pin S'                                              , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 280, 'cursor: pin S wait on X'                                    , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 281, 'Global transaction acquire instance locks'                  , 'Configuration'  , 'N', 'retries'                                 , ''                           , ''                       );
  ins( 282, 'enq: BB - 2PC across RAC instances'                         , 'Commit'         , 'N', 'name|mode'                               , 'gtrid hash value'           , 'bqual hash value'       );
  ins( 283, 'latch: shared pool'                                         , 'Concurrency'    , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 284, 'library cache pin'                                          , 'Concurrency'    , 'N', 'handle address'                          , 'pin address'                , '100*mode+namespace'     );
  ins( 285, 'library cache lock'                                         , 'Concurrency'    , 'N', 'handle address'                          , 'lock address'               , '100*mode+namespace'     );
  ins( 286, 'library cache load lock'                                    , 'Concurrency'    , 'N', 'object address'                          , 'lock address'               , '100*mask+namespace'     );
  ins( 287, 'library cache: mutex X'                                     , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 288, 'library cache: mutex S'                                     , 'Concurrency'    , 'N', 'idn'                                     , 'value'                      , 'where'                  );
  ins( 289, 'BFILE read'                                                 , 'User I/O'       , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 290, 'resmgr:cpu quantum'                                         , 'Scheduler'      , 'N', 'location'                                , 'consumer group id'          , ' '                      );
  ins( 291, 'resmgr:large I/O queued'                                    , 'Scheduler'      , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 292, 'resmgr:small I/O queued'                                    , 'Scheduler'      , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 293, 'resmgr:internal state change'                               , 'Concurrency'    , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 294, 'resmgr:sessions to exit'                                    , 'Concurrency'    , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 295, 'resmgr:become active'                                       , 'Scheduler'      , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 296, 'resmgr:pq queued'                                           , 'Scheduler'      , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 297, 'TCP Socket (KGAS)'                                          , 'Network'        , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 298, 'utl_file I/O'                                               , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 299, 'alter system set dispatcher'                                , 'Administrative' , 'N', 'waited'                                  , ''                           , ''                       );
  ins( 300, 'virtual circuit wait'                                       , 'Network'        , 'N', 'circuit#'                                , 'type'                       , ''                       );
  ins( 301, 'shared server idle wait'                                    , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 302, 'dispatcher timer'                                           , 'Idle'           , 'Y', 'sleep time'                              , ''                           , ''                       );
  ins( 303, 'dispatcher listen timer'                                    , 'Network'        , 'N', 'sleep time'                              , ''                           , ''                       );
  ins( 304, 'dedicated server timer'                                     , 'Network'        , 'N', 'wait event'                              , ''                           , ''                       );
  ins( 305, 'cmon timer'                                                 , 'Idle'           , 'Y', 'sleep time'                              , ''                           , ''                       );
  ins( 306, 'pool server timer'                                          , 'Idle'           , 'Y', 'sleep time'                              , ''                           , ''                       );
  ins( 307, 'connection pool wait'                                       , 'Administrative' , 'N', 'op'                                      , 'num servers'                , ''                       );
  ins( 308, 'JOX Jit Process Sleep'                                      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 309, 'jobq slave wait'                                            , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 310, 'Wait for Table Lock'                                        , 'Application'    , 'N', ''                                        , ''                           , ''                       );
  ins( 311, 'pipe get'                                                   , 'Idle'           , 'Y', 'handle address'                          , 'buffer length'              , 'timeout'                );
  ins( 312, 'pipe put'                                                   , 'Concurrency'    , 'N', 'handle address'                          , 'record length'              , 'timeout'                );
  ins( 313, 'enq: DB - contention'                                       , 'Administrative' , 'N', 'name|mode'                               , 'EnqMode'                    , '0'                      );
  ins( 314, 'PX Deque wait'                                              , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 315, 'PX Idle Wait'                                               , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 316, 'PX Deq: Join ACK'                                           , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 317, 'PX Deq Credit: need buffer'                                 , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , 'qref'                   );
  ins( 318, 'PX Deq Credit: send blkd'                                   , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , 'qref'                   );
  ins( 319, 'PX Deq: Msg Fragment'                                       , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 320, 'PX Deq: Parse Reply'                                        , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 321, 'PX Deq: Execute Reply'                                      , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 322, 'PX Deq: Execution Msg'                                      , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 323, 'PX Deq: Table Q Normal'                                     , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 324, 'PX Deq: Table Q Sample'                                     , 'Idle'           , 'Y', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 325, 'external table read'                                        , 'User I/O'       , 'N', 'filectx'                                 , 'file#'                      , 'size'                   );
  ins( 326, 'external table write'                                       , 'User I/O'       , 'N', 'filectx'                                 , 'file#'                      , 'size'                   );
  ins( 327, 'external table open'                                        , 'User I/O'       , 'N', 'filectx'                                 , 'file#'                      , ''                       );
  ins( 328, 'external table seek'                                        , 'User I/O'       , 'N', 'filectx'                                 , 'file#'                      , 'pos'                    );
  ins( 329, 'external table misc IO'                                     , 'User I/O'       , 'N', 'filectx'                                 , 'iocode'                     , 'P3'                     );
  ins( 330, 'enq: RC - Result Cache: Contention'                         , 'Application'    , 'N', 'name|mode'                               , 'chunkNo'                    , 'blockNo'                );
  ins( 331, 'enq: JX - SQL statement queue'                              , 'Scheduler'      , 'N', 'name|mode'                               , 'sqlid'                      , 'execid'                 );
  ins( 332, 'enq: JX - cleanup of  queue'                                , 'Scheduler'      , 'N', 'name|mode'                               , 'sqlid'                      , 'execid'                 );
  ins( 333, 'PX Queuing: statement queue'                                , 'Scheduler'      , 'N', 'sleeptime'                               , 'passes'                     , ''                       );
  ins( 334, 'dbverify reads'                                             , 'User I/O'       , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins( 335, 'Streams fetch slave: waiting for txns'                      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 336, 'Streams capture: waiting for subscribers to catch up'       , 'Queueing'       , 'N', ''                                        , ''                           , ''                       );
  ins( 337, 'Streams: resolve low memory condition'                      , 'Queueing'       , 'N', ''                                        , ''                           , ''                       );
  ins( 338, 'Streams apply: waiting to commit'                           , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 339, 'Streams apply: waiting for dependency'                      , 'Concurrency'    , 'N', ''                                        , ''                           , ''                       );
  ins( 340, 'Streams: flow control'                                      , 'Queueing'       , 'N', ''                                        , ''                           , ''                       );
  ins( 341, 'Streams: waiting for messages'                              , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 342, 'Streams capture: waiting for archive log'                   , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 343, 'Streams capture: filter callback waiting for ruleset'       , 'Application'    , 'N', ''                                        , ''                           , ''                       );
  ins( 344, 'Streams: apply reader waiting for DDL to apply'             , 'Application'    , 'N', 'sleep time'                              , ''                           , ''                       );
  ins( 345, 'enq: ZG - contention'                                       , 'Administrative' , 'N', 'name|mode'                               , 'file group id'              , 'version id'             );
  ins( 346, 'single-task message'                                        , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 347, 'SQL*Net message to client'                                  , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 348, 'SQL*Net message to dblink'                                  , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 349, 'SQL*Net more data to client'                                , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 350, 'SQL*Net more data to dblink'                                , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 351, 'SQL*Net message from client'                                , 'Idle'           , 'Y', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 352, 'SQL*Net more data from client'                              , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 353, 'SQL*Net message from dblink'                                , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 354, 'SQL*Net more data from dblink'                              , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 355, 'SQL*Net vector message from client'                         , 'Idle'           , 'Y', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 356, 'SQL*Net vector message from dblink'                         , 'Idle'           , 'Y', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 357, 'SQL*Net vector data to client'                              , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 358, 'SQL*Net vector data from client'                            , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 359, 'SQL*Net vector data to dblink'                              , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 360, 'SQL*Net vector data from dblink'                            , 'Network'        , 'N', 'driver id'                               , '#bytes'                     , ''                       );
  ins( 361, 'SQL*Net break/reset to client'                              , 'Application'    , 'N', 'driver id'                               , 'break?'                     , ''                       );
  ins( 362, 'SQL*Net break/reset to dblink'                              , 'Application'    , 'N', 'driver id'                               , 'break?'                     , ''                       );
  ins( 363, 'External Procedure initial connection'                      , 'Application'    , 'N', ''                                        , ''                           , ''                       );
  ins( 364, 'External Procedure call'                                    , 'Application'    , 'N', ''                                        , ''                           , ''                       );
  ins( 365, 'PL/SQL lock timer'                                          , 'Idle'           , 'Y', 'duration'                                , ''                           , ''                       );
  ins( 366, 'enq: UL - contention'                                       , 'Application'    , 'N', 'name|mode'                               , 'id'                         , '0'                      );
  ins( 367, 'wait for EMON to process ntfns'                             , 'Configuration'  , 'N', ''                                        , ''                           , ''                       );
  ins( 368, 'Streams AQ: emn coordinator idle wait'                      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 369, 'EMON slave idle wait'                                       , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 370, 'Streams AQ: waiting for messages in the queue'              , 'Idle'           , 'Y', 'queue id'                                , 'process#'                   , 'wait time'              );
  ins( 371, 'Streams AQ: waiting for time management or cleanup tasks'   , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 372, 'Streams AQ: enqueue blocked on low memory'                  , 'Queueing'       , 'N', ''                                        , ''                           , ''                       );
  ins( 373, 'Streams AQ: enqueue blocked due to flow control'            , 'Queueing'       , 'N', ''                                        , ''                           , ''                       );
  ins( 374, 'Streams AQ: delete acknowledged messages'                   , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 375, 'Streams AQ: deallocate messages from Streams Pool'          , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 376, 'Streams AQ: qmn coordinator idle wait'                      , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 377, 'Streams AQ: qmn slave idle wait'                            , 'Idle'           , 'Y', 'Type'                                    , ''                           , ''                       );
  ins( 378, 'Streams AQ: RAC qmn coordinator idle wait'                  , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 379, 'HS message to agent'                                        , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 380, 'TEXT: URL_DATASTORE network wait'                           , 'Network'        , 'N', ''                                        , ''                           , ''                       );
  ins( 381, 'TEXT: File System I/O'                                      , 'User I/O'       , 'N', ''                                        , ''                           , ''                       );
  ins( 382, 'OLAP DML Sleep'                                             , 'Application'    , 'N', 'duration'                                , ''                           , ''                       );
  ins( 383, 'ASM COD rollback operation completion'                      , 'Administrative' , 'N', 'dismount force'                          , ''                           , ''                       );
  ins( 384, 'kfk: async disk IO'                                         , 'System I/O'     , 'N', 'count'                                   , 'intr'                       , 'timeout'                );
  ins( 385, 'ASM background timer'                                       , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 386, 'ASM Fixed Package I/O'                                      , 'User I/O'       , 'N', 'blkno'                                   , 'bytes'                      , 'filetype'               );
  ins( 387, 'ASM mount : wait for heartbeat'                             , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 388, 'ASM PST query : wait for [PM][grp][0] grant'                , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 389, 'lock remastering'                                           , 'Cluster'        , 'N', ''                                        , ''                           , ''                       );
  ins( 390, 'ASM Staleness File I/O'                                     , 'User I/O'       , 'N', 'blkno'                                   , '#blks'                      , 'diskno'                 );
  ins( 391, 'auto-sqltune: wait graph update'                            , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 392, 'WCR: replay client notify'                                  , 'Idle'           , 'Y', 'who am I'                                , ''                           , ''                       );
  ins( 393, 'WCR: replay clock'                                          , 'Idle'           , 'Y', 'wait for scn''s hi 4 bytes'              , 'wait for scn''s lo 4 bytes' , ''                       );
  ins( 394, 'WCR: replay lock order'                                     , 'Application'    , 'N', 'wait for scn''s hi 4 bytes'              , 'wait for scn''s lo 4 bytes' , ''                       );
  ins( 395, 'WCR: replay paused'                                         , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 396, 'JS kgl get object wait'                                     , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 397, 'JS external job'                                            , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 398, 'JS kill job wait'                                           , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 399, 'JS coord start wait'                                        , 'Administrative' , 'N', ''                                        , ''                           , ''                       );
  ins( 400, 'cell single block physical read'                            , 'User I/O'       , 'N', 'cellhash#'                               , 'diskhash#'                  , 'bytes'                  );
  ins( 401, 'cell multiblock physical read'                              , 'User I/O'       , 'N', 'cellhash#'                               , 'diskhash#'                  , 'bytes'                  );
  ins( 402, 'cell list of blocks physical read'                          , 'User I/O'       , 'N', 'cellhash#'                               , 'diskhash#'                  , 'blocks'                 );
  ins( 403, 'cell manager opening cell'                                  , 'System I/O'     , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 404, 'cell manager closing cell'                                  , 'System I/O'     , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 405, 'cell manager discovering disks'                             , 'System I/O'     , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 406, 'cell worker idle'                                           , 'Idle'           , 'Y', ''                                        , ''                           , ''                       );
  ins( 407, 'events in waitclass Other'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 408, 'enq: WM - WLM Plan activation'                              , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 409, 'latch free'                                                 , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 410, 'kslwait unit test event 1'                                  , 'Other'          , 'N', 'p1'                                      , 'p2'                         , 'p3'                     );
  ins( 411, 'kslwait unit test event 2'                                  , 'Other'          , 'N', 'p1'                                      , 'p2'                         , 'p3'                     );
  ins( 412, 'kslwait unit test event 3'                                  , 'Other'          , 'N', 'p1'                                      , 'p2'                         , 'p3'                     );
  ins( 413, 'unspecified wait event'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 414, 'latch activity'                                             , 'Other'          , 'N', 'address'                                 , 'number'                     , 'process#'               );
  ins( 415, 'wait list latch activity'                                   , 'Other'          , 'N', 'address'                                 , 'number'                     , 'process#'               );
  ins( 416, 'wait list latch free'                                       , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 417, 'global enqueue expand wait'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 418, 'free process state object'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 419, 'inactive session'                                           , 'Other'          , 'N', 'session#'                                , 'waited'                     , 'instance|serial'        );
  ins( 420, 'process terminate'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 421, 'latch: call allocation'                                     , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 422, 'latch: session allocation'                                  , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 423, 'check CPU wait times'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 424, 'enq: CI - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'opcode'                     , 'type'                   );
  ins( 425, 'enq: PR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 426, 'ksim generic wait event'                                    , 'Other'          , 'N', 'where'                                   , 'wait_count'                 , ''                       );
  ins( 427, 'debugger command'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 428, 'ksdxexeother'                                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 429, 'ksdxexeotherwait'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 430, 'enq: PE - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'parno'                      , '0'                      );
  ins( 431, 'enq: PG - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 432, 'ksbsrv'                                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 433, 'ksbcic'                                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 434, 'process startup'                                            , 'Other'          , 'N', 'type'                                    , 'process#'                   , 'waited'                 );
  ins( 435, 'process shutdown'                                           , 'Other'          , 'N', 'type'                                    , 'process#'                   , 'waited'                 );
  ins( 436, 'prior spawner clean up'                                     , 'Other'          , 'N', 'process_pid'                             , 'process_sno'                , ''                       );
  ins( 437, 'latch: messages'                                            , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 438, 'rdbms ipc message block'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 439, 'rdbms ipc reply'                                            , 'Other'          , 'N', 'from_process'                            , 'timeout'                    , ''                       );
  ins( 440, 'latch: enqueue hash chains'                                 , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 441, 'enq: FP - global fob contention'                            , 'Other'          , 'N', 'name|mode'                               , 'low file obj add'           , 'high file obj add'      );
  ins( 442, 'enq: RE - block repair contention'                          , 'Other'          , 'N', 'name|mode'                               , 'File Type or File number'   , 'block number'           );
  ins( 443, 'asynch descriptor resize'                                   , 'Other'          , 'N', 'outstanding #aio'                        , 'current aio limit'          , 'new aio limit'          );
  ins( 444, 'imm op'                                                     , 'Other'          , 'N', 'msg ptr'                                 , ''                           , ''                       );
  ins( 445, 'slave exit'                                                 , 'Other'          , 'N', 'nalive'                                  , 'sleeptime'                  , 'loop'                   );
  ins( 446, 'enq: KM - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'type'                       , 'type'                   );
  ins( 447, 'enq: KT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'plan #'                     , '0'                      );
  ins( 448, 'enq: CA - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 449, 'enq: KD - determine DBRM master'                            , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 450, 'reliable message'                                           , 'Other'          , 'N', 'channel context'                         , 'channel handle'             , 'broadcast message'      );
  ins( 451, 'broadcast mesg queue transition'                            , 'Other'          , 'N', 'channel handle'                          , 'message'                    , 'location'               );
  ins( 452, 'broadcast mesg recovery queue transition'                   , 'Other'          , 'N', 'channel handle'                          , 'message'                    , 'location'               );
  ins( 453, 'master exit'                                                , 'Other'          , 'N', 'alive slaves'                            , ''                           , ''                       );
  ins( 454, 'ksv slave avail wait'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 455, 'enq: PV - syncstart'                                        , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 456, 'enq: PV - syncshut'                                         , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 457, 'enq: SP - contention 1'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins( 458, 'enq: SP - contention 2'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins( 459, 'enq: SP - contention 3'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins( 460, 'enq: SP - contention 4'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins( 461, 'IPC send completion sync'                                   , 'Other'          , 'N', 'send count'                              , ''                           , ''                       );
  ins( 462, 'OSD IPC library'                                            , 'Other'          , 'N', 'rolling mig'                             , ''                           , ''                       );
  ins( 463, 'IPC wait for name service busy'                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 464, 'IPC busy async request'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 465, 'IPC waiting for OSD resources'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 466, 'ksxr poll remote instances'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 467, 'ksxr wait for mount shared'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 468, 'DBMS_LDAP: LDAP operation '                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 469, 'wait for FMON to come up'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 470, 'enq: FM - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 471, 'enq: XY - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins( 472, 'set director factor wait'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 473, 'enq: AS - service activation'                               , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 474, 'enq: PD - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'property name'              , 'key hash'               );
  ins( 475, 'cleanup of aborted process'                                 , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 476, 'enq: RU - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 477, 'enq: RU - waiting'                                          , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 478, 'rolling migration: cluster quiesce'                         , 'Other'          , 'N', 'location'                                , 'waits'                      , ''                       );
  ins( 479, 'LMON global data update'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 480, 'process diagnostic dump'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 481, 'enq: MX - sync storage server info'                         , 'Other'          , 'N', 'name|mode'                               , 'nodeid'                     , 'instanceid'             );
  ins( 482, 'master diskmon startup'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 483, 'master diskmon read'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 484, 'pmon dblkr tst event'                                       , 'Other'          , 'N', 'index'                                   , ''                           , ''                       );
  ins( 485, 'latch: ges resource hash list'                              , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 486, 'DFS lock handle'                                            , 'Other'          , 'N', 'type|mode'                               , 'id1'                        , 'id2'                    );
  ins( 487, 'ges LMD to shutdown'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 488, 'ges client process to exit'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 489, 'ges global resource directory to be frozen'                 , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 490, 'ges resource directory to be unfrozen'                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 491, 'gcs resource directory to be unfrozen'                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 492, 'ges LMD to inherit communication channels'                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 493, 'ges lmd sync during reconfig'                               , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 494, 'ges wait for lmon to be ready'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 495, 'ges cgs registration'                                       , 'Other'          , 'N', 'where'                                   , ''                           , ''                       );
  ins( 496, 'wait for master scn'                                        , 'Other'          , 'N', 'waittime'                                , 'startscn'                   , 'ackscn'                 );
  ins( 497, 'ges yield cpu in reconfig'                                  , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 498, 'ges2 proc latch in rm latch get 1'                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 499, 'ges2 proc latch in rm latch get 2'                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 500, 'ges lmd/lmses to freeze in rcfg'                            , 'Other'          , 'N', 'lmd/lms id'                              , ''                           , ''                       );
  ins( 501, 'ges lmd/lmses to unfreeze in rcfg'                          , 'Other'          , 'N', 'lmd/lms id'                              , ''                           , ''                       );
  ins( 502, 'ges lms sync during dynamic remastering and reconfig'       , 'Other'          , 'N', 'location'                                , 'lms id'                     , ''                       );
  ins( 503, 'ges LMON to join CGS group'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 504, 'ges pmon to exit'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 505, 'ges lmd and pmon to attach'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 506, 'gcs drm freeze begin'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 507, 'gcs retry nowait latch get'                                 , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 508, 'gcs remastering wait for read latch'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 509, 'ges cached resource cleanup'                                , 'Other'          , 'N', 'waittime'                                , ''                           , ''                       );
  ins( 510, 'ges generic event'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 511, 'ges retry query node'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 512, 'ges process with outstanding i/o'                           , 'Other'          , 'N', 'pid'                                     , ''                           , ''                       );
  ins( 513, 'ges user error'                                             , 'Other'          , 'N', 'error'                                   , ''                           , ''                       );
  ins( 514, 'ges enter server mode'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 515, 'gcs enter server mode'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 516, 'gcs drm freeze in enter server mode'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 517, 'gcs ddet enter server mode'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 518, 'ges cancel'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 519, 'ges resource cleanout during enqueue open'                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 520, 'ges resource cleanout during enqueue open-cvt'              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 521, 'ges master to get established for SCN op'                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 522, 'ges LMON to get to FTDONE '                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 523, 'ges1 LMON to wake up LMD - mrcvr'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 524, 'ges2 LMON to wake up LMD - mrcvr'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 525, 'ges2 LMON to wake up lms - mrcvr 2'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 526, 'ges2 LMON to wake up lms - mrcvr 3'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 527, 'ges inquiry response'                                       , 'Other'          , 'N', 'type|mode|where'                         , 'id1'                        , 'id2'                    );
  ins( 528, 'ges reusing os pid'                                         , 'Other'          , 'N', 'pid'                                     , 'count'                      , ''                       );
  ins( 529, 'ges LMON for send queues'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 530, 'ges LMD suspend for testing event'                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 531, 'ges performance test completion'                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 532, 'kjbopen wait for recovery domain attach'                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 533, 'kjudomatt wait for recovery domain attach'                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 534, 'kjudomdet wait for recovery domain detach'                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 535, 'kjbdomalc allocate recovery domain - retry'                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 536, 'kjbdrmcvtq lmon drm quiesce: ping completion'               , 'Other'          , 'N', 'location'                                , 'lms id'                     , ''                       );
  ins( 537, 'ges RMS0 retry add redo log'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 538, 'readable standby redo apply remastering'                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 539, 'ges DFS hang analysis phase 2 acks'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 540, 'ges/gcs diag dump'                                          , 'Other'          , 'N', 'location'                                , ''                           , ''                       );
  ins( 541, 'global plug and play automatic resource creation'           , 'Other'          , 'N', 'where'                                   , ''                           , ''                       );
  ins( 542, 'gcs lmon dirtydetach step completion'                       , 'Other'          , 'N', 'domain id'                               , 'location'                   , ''                       );
  ins( 543, 'recovery instance recovery completion '                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 544, 'KJC: Wait for msg sends to complete'                        , 'Other'          , 'N', 'msg'                                     , 'dest|rcvr'                  , 'mtype'                  );
  ins( 545, 'ges message buffer allocation'                              , 'Other'          , 'N', 'pool'                                    , 'request'                    , 'allocated'              );
  ins( 546, 'kjctssqmg: quick message send wait'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 547, 'kjctcisnd: Queue/Send client message'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 548, 'gcs domain validation'                                      , 'Other'          , 'N', 'cluinc'                                  , 'rcvinc'                     , ''                       );
  ins( 549, 'latch: gcs resource hash'                                   , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 550, 'affinity expansion in replay'                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 551, 'wait for sync ack'                                          , 'Other'          , 'N', 'cluinc'                                  , 'pending_nd'                 , ''                       );
  ins( 552, 'wait for verification ack'                                  , 'Other'          , 'N', 'cluinc'                                  , 'pending_insts'              , ''                       );
  ins( 553, 'wait for assert messages to be sent'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 554, 'wait for scn ack'                                           , 'Other'          , 'N', 'pending_nd'                              , 'scnwrp'                     , 'scnbas'                 );
  ins( 555, 'lms flush message acks'                                     , 'Other'          , 'N', 'id'                                      , ''                           , ''                       );
  ins( 556, 'name-service call wait'                                     , 'Other'          , 'N', 'waittime'                                , ''                           , ''                       );
  ins( 557, 'CGS wait for IPC msg'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 558, 'kjxgrtest'                                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 559, 'IMR mount phase II completion'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 560, 'IMR disk votes'                                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 561, 'IMR rr lock release'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 562, 'IMR net-check message ack'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 563, 'IMR rr update'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 564, 'IMR membership resolution'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 565, 'IMR CSS join retry'                                         , 'Other'          , 'N', 'retry count'                             , ''                           , ''                       );
  ins( 566, 'CGS skgxn join retry'                                       , 'Other'          , 'N', 'retry count'                             , ''                           , ''                       );
  ins( 567, 'gcs to be enabled'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 568, 'gcs log flush sync'                                         , 'Other'          , 'N', 'waittime'                                , 'poll'                       , 'event'                  );
  ins( 569, 'GCR ctx lock acquisition'                                   , 'Other'          , 'N', 'retry count'                             , ''                           , ''                       );
  ins( 570, 'GCR lock acquisition'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 571, 'GCR CSS join retry'                                         , 'Other'          , 'N', 'retry count'                             , ''                           , ''                       );
  ins( 572, 'GCR member Data from CSS '                                  , 'Other'          , 'N', 'retry count'                             , ''                           , ''                       );
  ins( 573, 'SGA: allocation forcing component growth'                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 574, 'SGA: sga_target resize'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 575, 'control file heartbeat'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 576, 'control file diagnostic dump'                               , 'Other'          , 'N', 'type'                                    , 'param'                      , ''                       );
  ins( 577, 'enq: CF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , 'operation'              );
  ins( 578, 'enq: SW - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 579, 'enq: DS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 580, 'enq: TC - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'checkpoint ID'              , '0'                      );
  ins( 581, 'enq: TC - contention2'                                      , 'Other'          , 'N', 'name|mode'                               , 'checkpoint ID'              , '0'                      );
  ins( 582, 'buffer exterminate'                                         , 'Other'          , 'N', 'file#'                                   , 'block#'                     , 'buf_ptr'                );
  ins( 583, 'buffer resize'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 584, 'latch: cache buffers lru chain'                             , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 585, 'enq: PW - perwarm status in dbw0'                           , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 586, 'latch: checkpoint queue latch'                              , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 587, 'latch: cache buffer handles'                                , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 588, 'kcbzps'                                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 589, 'DBWR range invalidation sync'                               , 'Other'          , 'N', 'dbwr#'                                   , ''                           , ''                       );
  ins( 590, 'buffer deadlock'                                            , 'Other'          , 'N', 'dba'                                     , 'class*10+mode'              , 'flag'                   );
  ins( 591, 'buffer latch'                                               , 'Other'          , 'N', 'latch addr'                              , 'chain#'                     , ''                       );
  ins( 592, 'cr request retry'                                           , 'Other'          , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins( 593, 'writes stopped by instance recovery or database suspension' , 'Other'          , 'N', 'by thread#'                              , 'our thread#'                , ''                       );
  ins( 594, 'lock escalate retry'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 595, 'lock deadlock retry'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 596, 'prewarm transfer retry'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 597, 'recovery buffer pinned'                                     , 'Other'          , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins( 598, 'TSE master key rekey'                                       , 'Other'          , 'N', 'ts#'                                     , ''                           , ''                       );
  ins( 599, 'TSE SSO wallet reopen'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 600, 'enq: CR - block range reuse ckpt'                           , 'Other'          , 'N', 'name|mode'                               , '2'                          , '0'                      );
END;
/
BEGIN
  ins( 601, 'wait for MTTR advisory state object'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 602, 'latch: object queue header operation'                       , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 603, 'Wait on stby instance close'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 604, 'ARCH wait for archivelog lock'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 605, 'enq: WL - Test access/locking'                              , 'Other'          , 'N', 'name|mode'                               , 'log # / thread id #'        , 'sequence #'             );
  ins( 606, 'FAL archive wait 1 sec for REOPEN minimum'                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 607, 'TEST: action sync'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 608, 'TEST: action hang'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 609, 'RSGA: RAC reconfiguration'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 610, 'enq: WL - RAC-wide SGA contention'                          , 'Other'          , 'N', 'name|mode'                               , 'log # / thread id #'        , 'sequence #'             );
  ins( 611, 'LGWR ORL/NoExp FAL archival'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 612, 'MRP wait on process start'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 613, 'MRP wait on process restart'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 614, 'MRP wait on startup clear'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 615, 'MRP inactivation'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 616, 'MRP termination'                                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 617, 'MRP state inspection'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 618, 'MRP wait on archivelog arrival'                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 619, 'MRP wait on archivelog archival'                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 620, 'log switch/archive'                                         , 'Other'          , 'N', 'thread#'                                 , ''                           , ''                       );
  ins( 621, 'ARCH wait on c/f tx acquire 1'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 622, 'RFS attach'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 623, 'RFS create'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 624, 'RFS close'                                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 625, 'RFS announce'                                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 626, 'RFS register'                                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 627, 'RFS detach'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 628, 'RFS ping'                                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 629, 'RFS dispatch'                                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 630, 'enq: WL - RFS global state contention'                      , 'Other'          , 'N', 'name|mode'                               , 'log # / thread id #'        , 'sequence #'             );
  ins( 631, 'LGWR simulation latency wait'                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 632, 'LNS simulation latency wait'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 633, 'Data Guard: RFS disk I/O'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 634, 'ARCH wait for process start 1'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 635, 'ARCH wait for process death 1'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 636, 'ARCH wait for process start 3'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 637, 'Data Guard: process exit'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 638, 'Data Guard: process clean up'                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 639, 'LGWR-LNS wait on channel'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 640, 'enq: WR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'thread id #'                , 'sequence #'             );
  ins( 641, 'LGWR wait for redo copy'                                    , 'Other'          , 'N', 'copy latch #'                            , ''                           , ''                       );
  ins( 642, 'latch: redo allocation'                                     , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 643, 'log file switch (clearing log file)'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 644, 'enq: WL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'log # / thread id #'        , 'sequence #'             );
  ins( 645, 'enq: RN - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'thread number'              , 'log number'             );
  ins( 646, 'DFS db file lock'                                           , 'Other'          , 'N', 'file#'                                   , ''                           , ''                       );
  ins( 647, 'enq: DF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , 'file #'                 );
  ins( 648, 'enq: IS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , 'type'                   );
  ins( 649, 'enq: FS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , 'type'                   );
  ins( 650, 'enq: DM - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'type'                       , 'type'                   );
  ins( 651, 'enq: RP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'file #'                     , '1 or block'             );
  ins( 652, 'latch: gc element'                                          , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 653, 'enq: RT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'redo thread'                , 'type'                   );
  ins( 654, 'enq: RT - thread internal enable/disable'                   , 'Other'          , 'N', 'name|mode'                               , 'redo thread'                , 'type'                   );
  ins( 655, 'enq: IR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0/1'                    );
  ins( 656, 'enq: IR - contention2'                                      , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0/1'                    );
  ins( 657, 'enq: MR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0 or file #'                , 'type'                   );
  ins( 658, 'enq: MR - standby role transition'                          , 'Other'          , 'N', 'name|mode'                               , '0 or file #'                , 'type'                   );
  ins( 659, 'shutdown after switchover to standby'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 660, 'parallel recovery coord wait for reply'                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 661, 'parallel recovery coord send blocked'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 662, 'parallel recovery slave wait for change'                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 663, 'enq: BR - file shrink'                                      , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 664, 'enq: BR - proxy-copy'                                       , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 665, 'enq: BR - multi-section restore header'                     , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 666, 'enq: BR - multi-section restore section'                    , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 667, 'enq: BR - space info datafile hdr update'                   , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 668, 'enq: BR - request autobackup'                               , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 669, 'enq: BR - perform autobackup'                               , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'file #'                 );
  ins( 670, 'enq: ID - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 671, 'Backup Restore Throttle sleep'                              , 'Other'          , 'N', 'session_id'                              , 'serial'                     , ''                       );
  ins( 672, 'Backup Restore Switch Bitmap sleep'                         , 'Other'          , 'N', 'session_id'                              , 'serial'                     , ''                       );
  ins( 673, 'Backup Restore Event 19778 sleep'                           , 'Other'          , 'N', 'session_id'                              , 'serial'                     , ''                       );
  ins( 674, 'enq: AB - ABMR process start/stop'                          , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 675, 'enq: AB - ABMR process initialized'                         , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 676, 'Auto BMR completion'                                        , 'Other'          , 'N', 'file#'                                   , 'block#'                     , ''                       );
  ins( 677, 'Auto BMR RPC standby catchup'                               , 'Other'          , 'N', 'wrap'                                    , 'base'                       , ''                       );
  ins( 678, 'enq: MN - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'session ID'                 , '0'                      );
  ins( 679, 'enq: PL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 680, 'enq: SB - logical standby metadata'                         , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 681, 'enq: SB - table instantiation'                              , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 682, 'Logical Standby Apply shutdown'                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 683, 'Logical Standby pin transaction'                            , 'Other'          , 'N', 'xidusn'                                  , 'xidslt'                     , 'xidsqn'                 );
  ins( 684, 'Logical Standby dictionary build'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 685, 'Logical Standby Terminal Apply'                             , 'Other'          , 'N', 'stage'                                   , ''                           , ''                       );
  ins( 686, 'Logical Standby Debug'                                      , 'Other'          , 'N', 'event'                                   , ''                           , ''                       );
  ins( 687, 'enq: XR - quiesce database'                                 , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , '0'                      );
  ins( 688, 'enq: XR - database force logging'                           , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , '0'                      );
  ins( 689, 'standby query scn advance'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 690, 'change tracking file synchronous read'                      , 'Other'          , 'N', 'block#'                                  , 'blocks'                     , ''                       );
  ins( 691, 'change tracking file synchronous write'                     , 'Other'          , 'N', 'block#'                                  , 'blocks'                     , ''                       );
  ins( 692, 'change tracking file parallel write'                        , 'Other'          , 'N', 'blocks'                                  , 'requests'                   , ''                       );
  ins( 693, 'block change tracking buffer space'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 694, 'CTWR media recovery checkpoint request'                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 695, 'enq: CT - global space management'                          , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 696, 'enq: CT - local space management'                           , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 697, 'enq: CT - change stream ownership'                          , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 698, 'enq: CT - state'                                            , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 699, 'enq: CT - state change gate 1'                              , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 700, 'enq: CT - state change gate 2'                              , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 701, 'enq: CT - CTWR process start/stop'                          , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 702, 'enq: CT - reading'                                          , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'operation parm'         );
  ins( 703, 'recovery area: computing dropped files'                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 704, 'recovery area: computing obsolete files'                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 705, 'recovery area: computing backed up files'                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 706, 'recovery area: computing applied logs'                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 707, 'enq: RS - file delete'                                      , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 708, 'enq: RS - record reuse'                                     , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 709, 'enq: RS - prevent file delete'                              , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 710, 'enq: RS - prevent aging list update'                        , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 711, 'enq: RS - persist alert level'                              , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 712, 'enq: RS - read alert level'                                 , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 713, 'enq: RS - write alert level'                                , 'Other'          , 'N', 'name|mode'                               , 'record type'                , 'record id'              );
  ins( 714, 'enq: FL - Flashback database log'                           , 'Other'          , 'N', 'name|mode'                               , 'Log #'                      , 'zero'                   );
  ins( 715, 'enq: FL - Flashback db command'                             , 'Other'          , 'N', 'name|mode'                               , 'Log #'                      , 'zero'                   );
  ins( 716, 'enq: FD - Marker generation'                                , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 717, 'enq: FD - Tablespace flashback on/off'                      , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 718, 'enq: FD - Flashback coordinator'                            , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 719, 'enq: FD - Flashback on/off'                                 , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 720, 'enq: FD - Restore point create/drop'                        , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 721, 'enq: FD - Flashback logical operations'                     , 'Other'          , 'N', 'name|mode'                               , 'Internal'                   , 'Internal'               );
  ins( 722, 'flashback free VI log'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 723, 'flashback log switch'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 724, 'RVWR wait for flashback copy'                               , 'Other'          , 'N', 'copy latch #'                            , ''                           , ''                       );
  ins( 725, 'parallel recovery read buffer free'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 726, 'parallel recovery change buffer free'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 727, 'cell smart flash unkeep'                                    , 'Other'          , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins( 728, 'datafile move cleanup during resize'                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 729, 'blocking txn id for DDL'                                    , 'Other'          , 'N', 'table'                                   , 'scn'                        , 'only dml'               );
  ins( 730, 'transaction'                                                , 'Other'          , 'N', 'undo seg#|slot#'                         , 'wrap#'                      , 'count'                  );
  ins( 731, 'inactive transaction branch'                                , 'Other'          , 'N', 'branch#'                                 , 'waited'                     , ''                       );
  ins( 732, 'txn to complete'                                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 733, 'PMON to cleanup pseudo-branches at svc stop time'           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 734, 'PMON to cleanup detached branches at shutdown'              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 735, 'test long ops'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 736, 'latch: undo global data'                                    , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 737, 'undo segment recovery'                                      , 'Other'          , 'N', 'segment#'                                , 'tx flags'                   , ''                       );
  ins( 738, 'unbound tx'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 739, 'wait for change'                                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 740, 'wait for another txn - undo rcv abort'                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 741, 'wait for another txn - txn abort'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 742, 'wait for another txn - rollback to savepoint'               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 743, 'undo_retention publish retry'                               , 'Other'          , 'N', 'where'                                   , 'retry_count'                , ''                       );
  ins( 744, 'enq: TA - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'undo segment # / other' );
  ins( 745, 'enq: TX - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'usn<<16 | slot'             , 'sequence'               );
  ins( 746, 'enq: US - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'undo segment #'             , '0'                      );
  ins( 747, 'wait for stopper event to be increased'                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 748, 'wait for a undo record'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 749, 'wait for a paralle reco to abort'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 750, 'enq: IM - contention for blr'                               , 'Other'          , 'N', 'name|mode'                               , 'pool #'                     , '0'                      );
  ins( 751, 'enq: TD - KTF dump entries'                                 , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 752, 'enq: TE - KTF broadcast'                                    , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 753, 'enq: CN - race with txn'                                    , 'Other'          , 'N', 'name|mode'                               , 'reg id'                     , '0'                      );
  ins( 754, 'enq: CN - race with reg'                                    , 'Other'          , 'N', 'name|mode'                               , 'reg id'                     , '0'                      );
  ins( 755, 'enq: CN - race with init'                                   , 'Other'          , 'N', 'name|mode'                               , 'reg id'                     , '0'                      );
  ins( 756, 'latch: Change Notification Hash table latch'                , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 757, 'enq: CO - master slave det'                                 , 'Other'          , 'N', 'name|mode'                               , 'inst id'                    , '0'                      );
  ins( 758, 'enq: FE - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 759, 'latch: change notification client cache latch'              , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 760, 'enq: TF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace #'               , 'relative file #'        );
  ins( 761, 'latch: lob segment hash table latch'                        , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 762, 'latch: lob segment query latch'                             , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 763, 'latch: lob segment dispenser latch'                         , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 764, 'Wait for shrink lock2'                                      , 'Other'          , 'N', 'object_id'                               , 'lock_mode'                  , ''                       );
  ins( 765, 'Wait for shrink lock'                                       , 'Other'          , 'N', 'object_id'                               , 'lock_mode'                  , ''                       );
  ins( 766, 'L1 validation'                                              , 'Other'          , 'N', 'seghdr'                                  , 'l1bmb'                      , ''                       );
  ins( 767, 'Wait for TT enqueue'                                        , 'Other'          , 'N', 'tsn'                                     , ''                           , ''                       );
  ins( 768, 'kttm2d'                                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 769, 'ktsambl'                                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 770, 'ktfbtgex'                                                   , 'Other'          , 'N', 'tsn'                                     , ''                           , ''                       );
  ins( 771, 'enq: DT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 772, 'enq: TS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace ID'              , 'dba'                    );
  ins( 773, 'enq: FB - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace #'               , 'dba'                    );
  ins( 774, 'enq: SK - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace #'               , 'dba'                    );
  ins( 775, 'enq: DW - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace #'               , 'dba'                    );
  ins( 776, 'enq: SU - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'table space #'              , '0'                      );
  ins( 777, 'enq: TT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace ID'              , 'operation'              );
  ins( 778, 'ktm: instance recovery'                                     , 'Other'          , 'N', 'undo segment#'                           , ''                           , ''                       );
  ins( 779, 'instance state change'                                      , 'Other'          , 'N', 'layer'                                   , 'value'                      , 'waited'                 );
  ins( 780, 'enq: SJ - Slave Task Cancel'                                , 'Other'          , 'N', 'name|mode'                               , 'Slave process id'           , 'task id'                );
  ins( 781, 'Space Manager: slave messages'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 782, 'index block split'                                          , 'Other'          , 'N', 'rootdba'                                 , 'level'                      , 'childdba'               );
  ins( 783, 'kdblil wait before retrying ORA-54'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 784, 'dupl. cluster key'                                          , 'Other'          , 'N', 'dba'                                     , ''                           , ''                       );
  ins( 785, 'kdic_do_merge'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 786, 'enq: DL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 787, 'enq: HQ - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , 'hash value'             );
  ins( 788, 'enq: HP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'tablespace #'               , 'dba'                    );
  ins( 789, 'enq: WG - delete fso'                                       , 'Other'          , 'N', 'name|mode'                               , 'kdlw lobid first half'      , 'kdlw lobid sec half'    );
  ins( 790, 'enq: SL - get lock'                                         , 'Other'          , 'N', 'name|mode'                               , 'kdlw lobid first half'      , 'kdlw lobid sec half'    );
  ins( 791, 'enq: SL - escalate lock'                                    , 'Other'          , 'N', 'name|mode'                               , 'kdlw lobid first half'      , 'kdlw lobid sec half'    );
  ins( 792, 'enq: SL - get lock for undo'                                , 'Other'          , 'N', 'name|mode'                               , 'kdlw lobid first half'      , 'kdlw lobid sec half'    );
  ins( 793, 'enq: ZH - compression analysis'                             , 'Other'          , 'N', 'name|mode'                               , 'obj#'                       , 'ulevel'                 );
  ins( 794, 'Compression analysis'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 795, 'row cache cleanup'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 796, 'row cache process'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 797, 'enq: DV - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 798, 'enq: SO - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 799, 'enq: TP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 800, 'enq: RW - MV metadata contention'                           , 'Other'          , 'N', 'name|mode'                               , 'table obj#'                 , '0'                      );
  ins( 801, 'enq: OC - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '1'                          , '2'                      );
  ins( 802, 'enq: OL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'hash value'                 , '0'                      );
  ins( 803, 'kkdlgon'                                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 804, 'kkdlsipon'                                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 805, 'kkdlhpon'                                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 806, 'kgltwait'                                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 807, 'kksfbc research'                                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 808, 'kksscl hash split'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 809, 'kksfbc child completion'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 810, 'enq: CU - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'handle'                     , 'handle'                 );
  ins( 811, 'enq: AE - lock'                                             , 'Other'          , 'N', 'name|mode'                               , 'edition obj#'               , '0'                      );
  ins( 812, 'enq: PF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 813, 'enq: IL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 814, 'enq: CL - drop label'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 815, 'enq: CL - compare labels'                                   , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 816, 'enq: MK - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 817, 'enq: OW - initialization'                                   , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 818, 'enq: OW - termination'                                      , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 819, 'enq: RK - set key'                                          , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 820, 'enq: RL - RAC wallet lock'                                  , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 821, 'enq: ZZ - update hash tables'                               , 'Other'          , 'N', 'name|mode'                               , 'KSBXIC Action'              , '0'                      );
  ins( 822, 'enq: ZA - add std audit table partition'                    , 'Other'          , 'N', 'name|mode'                               , 'KZAM Aud Partition'         , '0'                      );
  ins( 823, 'enq: ZF - add fga audit table partition'                    , 'Other'          , 'N', 'name|mode'                               , 'KZAM Fga Partition'         , '0'                      );
  ins( 824, 'enq: DX - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'transaction entry #'        , '0'                      );
  ins( 825, 'enq: DR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 826, 'pending global transaction(s)'                              , 'Other'          , 'N', 'scans'                                   , ''                           , ''                       );
  ins( 827, 'free global transaction table entry'                        , 'Other'          , 'N', 'tries'                                   , ''                           , ''                       );
  ins( 828, 'library cache revalidation'                                 , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 829, 'library cache shutdown'                                     , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 830, 'BFILE closure'                                              , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 831, 'BFILE check if exists'                                      , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 832, 'BFILE check if open'                                        , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 833, 'BFILE get length'                                           , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 834, 'BFILE get name object'                                      , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 835, 'BFILE get path object'                                      , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 836, 'BFILE open'                                                 , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 837, 'BFILE internal seek'                                        , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 838, 'waiting to get CAS latch'                                   , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 839, 'waiting to get RM CAS latch'                                , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 840, 'resmgr:internal state cleanup'                              , 'Other'          , 'N', 'location'                                , ' '                          , ' '                      );
  ins( 841, 'xdb schema cache initialization'                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 842, 'ASM cluster file access'                                    , 'Other'          , 'N', 'lock'                                    , 'mode'                       , ''                       );
  ins( 843, 'CSS initialization'                                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 844, 'CSS group registration'                                     , 'Other'          , 'N', 'group_name1'                             , 'group_name2'                , 'mem_id'                 );
  ins( 845, 'CSS group membership query'                                 , 'Other'          , 'N', 'group_name1'                             , 'group_name2'                , 'group_name3'            );
  ins( 846, 'CSS operation: data query'                                  , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 847, 'CSS operation: data update'                                 , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 848, 'CSS Xgrp shared operation'                                  , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 849, 'CSS operation: query'                                       , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 850, 'CSS operation: action'                                      , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 851, 'CSS operation: diagnostic'                                  , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 852, 'GIPC operation: dump'                                       , 'Other'          , 'N', 'function_id'                             , ''                           , ''                       );
  ins( 853, 'GPnP Initialization'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 854, 'GPnP Termination'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 855, 'GPnP Get Item'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 856, 'GPnP Set Item'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 857, 'GPnP Get Error'                                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 858, 'ADR file lock'                                              , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 859, 'ADR block file read'                                        , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 860, 'ADR block file write'                                       , 'Other'          , 'N', ' '                                       , ' '                          , ' '                      );
  ins( 861, 'CRS call completion'                                        , 'Other'          , 'N', 'clsrrestype'                             , 'kjha_action'                , ''                       );
  ins( 862, 'dispatcher shutdown'                                        , 'Other'          , 'N', 'waited'                                  , ''                           , ''                       );
  ins( 863, 'listener registration dump'                                 , 'Other'          , 'N', 'dump'                                    , ''                           , ''                       );
  ins( 864, 'latch: virtual circuit queues'                              , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 865, 'listen endpoint status'                                     , 'Other'          , 'N', 'end-point#'                              , 'status'                     , ''                       );
  ins( 866, 'OJVM: Generic'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 867, 'select wait'                                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 868, 'jobq slave shutdown wait'                                   , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 869, 'jobq slave TJ process wait'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 870, 'job scheduler coordinator slave wait'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 871, 'enq: JD - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 872, 'enq: JQ - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 873, 'enq: OD - Serializing DDLs'                                 , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 874, 'kkshgnc reloop'                                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 875, 'optimizer stats update retry'                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 876, 'wait active processes'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 877, 'SUPLOG PL wait for inflight pragma-d PL/SQL'                , 'Other'          , 'N', 'session_num'                             , ''                           , ''                       );
  ins( 878, 'enq: MD - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'master object #'            , '0'                      );
  ins( 879, 'enq: MS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'master object #'            , '0'                      );
  ins( 880, 'wait for kkpo ref-partitioning *TEST EVENT*'                , 'Other'          , 'N', 'where'                                   , ''                           , ''                       );
  ins( 881, 'PX slave connection'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 882, 'PX slave release'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 883, 'PX Send Wait'                                               , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 884, 'PX qref latch'                                              , 'Other'          , 'N', 'function'                                , 'sleeptime'                  , 'qref'                   );
  ins( 885, 'PX server shutdown'                                         , 'Other'          , 'N', 'nalive'                                  , 'sleeptime'                  , 'loop'                   );
  ins( 886, 'PX create server'                                           , 'Other'          , 'N', 'nservers'                                , 'sleeptime'                  , 'enqueue'                );
  ins( 887, 'PX signal server'                                           , 'Other'          , 'N', 'serial'                                  , 'error'                      , 'nbusy'                  );
  ins( 888, 'PX Deq Credit: free buffer'                                 , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , 'qref'                   );
  ins( 889, 'PX Deq: Test for msg'                                       , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 890, 'PX Deq: Test for credit'                                    , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 891, 'PX Deq: Signal ACK RSG'                                     , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 892, 'PX Deq: Signal ACK EXT'                                     , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 893, 'PX Deq: reap credit'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 894, 'PX Nsq: PQ descriptor query'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 895, 'PX Nsq: PQ load info query'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 896, 'PX Deq Credit: Session Stats'                               , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 897, 'PX Deq: Slave Session Stats'                                , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 898, 'PX Deq: Slave Join Frag'                                    , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 899, 'enq: PI - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'serial #'               );
  ins( 900, 'enq: PS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'instance'                   , 'slave ID'               );
  ins( 901, 'latch: parallel query alloc buffer'                         , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 902, 'kxfxse'                                                     , 'Other'          , 'N', 'kxfxse debug wait: stalling for slave 0' , ''                           , ''                       );
  ins( 903, 'kxfxsp'                                                     , 'Other'          , 'N', 'kxfxsp debug wait: stalling for slave 0' , ''                           , ''                       );
  ins( 904, 'PX Deq: Table Q qref'                                       , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 905, 'PX Deq: Table Q Get Keys'                                   , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 906, 'PX Deq: Table Q Close'                                      , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 907, 'GV$: slave acquisition retry wait time'                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 908, 'PX hash elem being inserted'                                , 'Other'          , 'N', 'hashid'                                  , 'element'                    , 'insert/update'          );
  ins( 909, 'latch: PX hash array latch'                                 , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins( 910, 'enq: AY - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'Op1'                        , 'Op2'                    );
  ins( 911, 'enq: TO - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '1'                      );
  ins( 912, 'enq: IT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 913, 'enq: BF - allocation contention'                            , 'Other'          , 'N', 'name|mode'                               , 'node#/parallelizer#'        , 'bloom#'                 );
  ins( 914, 'enq: BF - PMON Join Filter cleanup'                         , 'Other'          , 'N', 'name|mode'                               , 'node#/parallelizer#'        , 'bloom#'                 );
  ins( 915, 'enq: RD - RAC load'                                         , 'Other'          , 'N', 'name|mode'                               , 'sqlid'                      , 'execid'                 );
  ins( 916, 'timer in sksawat'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 917, 'scginq AST call'                                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 918, 'kupp process wait'                                          , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 919, 'Kupp process shutdown'                                      , 'Other'          , 'N', 'nalive'                                  , 'sleeptime'                  , 'loop'                   );
  ins( 920, 'Data Pump slave startup'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 921, 'Data Pump slave init'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 922, 'enq: KP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 923, 'Replication Dequeue '                                       , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 924, 'knpc_acwm_AwaitChangedWaterMark'                            , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 925, 'knpc_anq_AwaitNonemptyQueue'                                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 926, 'knpsmai'                                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 927, 'enq: SR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'sequence # / apply #'   );
  ins( 928, 'Streams capture: waiting for database startup'              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 929, 'Streams miscellaneous event'                                , 'Other'          , 'N', 'TYPE'                                    , ''                           , ''                       );
  ins( 930, 'enq: SI - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'object #'                   , '0'                      );
  ins( 931, 'Streams: RAC waiting for inter instance ack'                , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 932, 'enq: IA - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 933, 'enq: JI - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'view object #'              , '0'                      );
  ins( 934, 'qerex_gdml'                                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 935, 'enq: AT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 936, 'opishd'                                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 937, 'kpodplck wait before retrying ORA-54'                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 938, 'enq: CQ - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 939, 'Streams AQ: emn coordinator waiting for slave to start'     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 940, 'wait for EMON to spawn'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 941, 'EMON termination'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 942, 'EMON slave messages'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 943, 'enq: SE - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'Session-id'                 , 'Serial#'                );
  ins( 944, 'tsm with timeout'                                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 945, 'Streams AQ: waiting for busy instance for instance_name'    , 'Other'          , 'N', 'where'                                   , 'wait_count'                 , ''                       );
  ins( 946, 'enq: TQ - TM contention'                                    , 'Other'          , 'N', 'name|mode'                               , 'QT_OBJ#'                    , '0'                      );
  ins( 947, 'enq: TQ - DDL contention'                                   , 'Other'          , 'N', 'name|mode'                               , 'QT_OBJ#'                    , '0'                      );
  ins( 948, 'enq: TQ - INI contention'                                   , 'Other'          , 'N', 'name|mode'                               , 'QT_OBJ#'                    , '0'                      );
  ins( 949, 'enq: TQ - DDL-INI contention'                               , 'Other'          , 'N', 'name|mode'                               , 'QT_OBJ#'                    , '0'                      );
  ins( 950, 'AQ propagation connection'                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 951, 'enq: DP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 952, 'enq: MH - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 953, 'enq: ML - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 954, 'enq: PH - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 955, 'enq: SF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 956, 'enq: XH - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 957, 'enq: WA - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 958, 'Streams AQ: QueueTable kgl locks'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 959, 'AQ spill debug idle'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 960, 'queue slave messages'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 961, 'Streams AQ: qmn coordinator waiting for slave to start'     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 962, 'enq: CX - TEXT: Index Specific Lock'                        , 'Other'          , 'N', 'name|mode'                               , 'Index Id'                   , 'Partition Id'           );
  ins( 963, 'enq: OT - TEXT: Generic Lock'                               , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins( 964, 'XDB SGA initialization'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins( 965, 'enq: XC - XDB Configuration'                                , 'Other'          , 'N', 'name|mode'                               , ''                           , ''                       );
  ins( 966, 'NFS read delegation outstanding'                            , 'Other'          , 'N', 'delegation count'                        , ''                           , ''                       );
  ins( 967, 'Data Guard Broker Wait'                                     , 'Other'          , 'N', 'Broker Component'                        , 'Wait Argument 1'            , 'Wait Argument 2'        );
  ins( 968, 'enq: RF - synch: DG Broker metadata'                        , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 969, 'enq: RF - atomicity'                                        , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 970, 'enq: RF - synchronization: aifo master'                     , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 971, 'enq: RF - new AI'                                           , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 972, 'enq: RF - synchronization: critical ai'                     , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 973, 'enq: RF - RF - Database Automatic Disable'                  , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 974, 'enq: RF - FSFO Observer Heartbeat'                          , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 975, 'enq: RF - DG Broker Current File ID'                        , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 976, 'enq: RF - FSFO Primary Shutdown suspended'                  , 'Other'          , 'N', 'name|mode'                               , 'lock operation'             , 'lock value'             );
  ins( 977, 'PX Deq: OLAP Update Reply'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 978, 'PX Deq: OLAP Update Execute'                                , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 979, 'PX Deq: OLAP Update Close'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 980, 'OLAP Parallel Type Deq'                                     , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 981, 'OLAP Parallel Temp Grow Request'                            , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 982, 'OLAP Parallel Temp Grow Wait'                               , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 983, 'OLAP Parallel Temp Grew'                                    , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 984, 'OLAP Null PQ Reason'                                        , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 985, 'OLAP Aggregate Master Enq'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 986, 'OLAP Aggregate Client Enq'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 987, 'OLAP Aggregate Master Deq'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 988, 'OLAP Aggregate Client Deq'                                  , 'Other'          , 'N', 'sleeptime/senderid'                      , 'passes'                     , ''                       );
  ins( 989, 'enq: AW - AW$ table lock'                                   , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'workspace #'            );
  ins( 990, 'enq: AW - AW state lock'                                    , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'workspace #'            );
  ins( 991, 'enq: AW - user access for AW'                               , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'workspace #'            );
  ins( 992, 'enq: AW - AW generation lock'                               , 'Other'          , 'N', 'name|mode'                               , 'operation'                  , 'workspace #'            );
  ins( 993, 'enq: AG - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'workspace #'                , 'generation'             );
  ins( 994, 'enq: AO - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'workspace #'                , 'object #'               );
  ins( 995, 'enq: OQ - xsoqhiAlloc'                                      , 'Other'          , 'N', 'name|mode'                               , 'resource id'                , '0'                      );
  ins( 996, 'enq: OQ - xsoqhiFlush'                                      , 'Other'          , 'N', 'name|mode'                               , 'resource id'                , '0'                      );
  ins( 997, 'enq: OQ - xsoq*histrecb'                                    , 'Other'          , 'N', 'name|mode'                               , 'resource id'                , '0'                      );
  ins( 998, 'enq: OQ - xsoqhiClose'                                      , 'Other'          , 'N', 'name|mode'                               , 'resource id'                , '0'                      );
  ins( 999, 'enq: OQ - xsoqhistrecb'                                     , 'Other'          , 'N', 'name|mode'                               , 'resource id'                , '0'                      );
  ins(1000, 'enq: AM - client registration'                              , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1001, 'enq: AM - shutdown'                                         , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1002, 'enq: AM - rollback COD reservation'                         , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1003, 'enq: AM - background COD reservation'                       , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1004, 'enq: AM - ASM cache freeze'                                 , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1005, 'enq: AM - ASM ACD Relocation'                               , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1006, 'enq: AM - group use'                                        , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1007, 'enq: AM - group block'                                      , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1008, 'enq: AM - ASM File Destroy'                                 , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1009, 'enq: AM - ASM User'                                         , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1010, 'enq: AM - ASM Password File Update'                         , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1011, 'enq: AM - ASM Amdu Dump'                                    , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1012, 'enq: AM - disk offline'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1013, 'enq: AM - ASM reserved'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1014, 'enq: AM - block repair'                                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1015, 'enq: AM - ASM disk based alloc/dealloc'                     , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1016, 'enq: AM - ASM file descriptor'                              , 'Other'          , 'N', 'name|mode'                               , 'id1'                        , 'id2'                    );
  ins(1017, 'ASM internal hang test'                                     , 'Other'          , 'N', 'test #'                                  , ''                           , ''                       );
  ins(1018, 'ASM Instance startup'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1019, 'buffer busy'                                                , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1020, 'buffer freelistbusy'                                        , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1021, 'buffer rememberlist busy'                                   , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1022, 'buffer writeList full'                                      , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1023, 'no free buffers'                                            , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1024, 'buffer write wait'                                          , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1025, 'buffer invalidation wait'                                   , 'Other'          , 'N', 'group#'                                  , 'obj#'                       , 'block#'                 );
  ins(1026, 'buffer dirty disabled'                                      , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1027, 'ASM metadata cache frozen'                                  , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1028, 'enq: CM - gate'                                             , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'type'                   );
  ins(1029, 'enq: CM - instance'                                         , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'type'                   );
  ins(1030, 'enq: CM - diskgroup dismount'                               , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'type'                   );
  ins(1031, 'enq: XQ - recovery'                                         , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'unused'                 );
  ins(1032, 'enq: XQ - relocation'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'unused'                 );
  ins(1033, 'enq: XQ - purification'                                     , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'unused'                 );
  ins(1034, 'enq: AD - allocate AU'                                      , 'Other'          , 'N', 'name|mode'                               , 'group and disk number'      , 'AU number'              );
  ins(1035, 'enq: AD - deallocate AU'                                    , 'Other'          , 'N', 'name|mode'                               , 'group and disk number'      , 'AU number'              );
  ins(1036, 'enq: AD - relocate AU'                                      , 'Other'          , 'N', 'name|mode'                               , 'group and disk number'      , 'AU number'              );
  ins(1037, 'enq: DO - disk online'                                      , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'disk #'                 );
  ins(1038, 'enq: DO - disk online recovery'                             , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'disk #'                 );
  ins(1039, 'enq: DO - Staleness Registry create'                        , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'disk #'                 );
  ins(1040, 'enq: DO - startup of MARK process'                          , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'disk #'                 );
  ins(1041, 'enq: DO - disk online operation'                            , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'disk #'                 );
  ins(1042, 'extent map load/unlock'                                     , 'Other'          , 'N', 'group'                                   , 'file'                       , 'extent'                 );
  ins(1043, 'enq: XL - fault extent map'                                 , 'Other'          , 'N', 'name|mode'                               , 'map id'                     , 'nothing'                );
  ins(1044, 'Sync ASM rebalance'                                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1045, 'enq: DG - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'type'                   );
  ins(1046, 'enq: DD - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'type'                   );
  ins(1047, 'enq: HD - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , '0'                      );
  ins(1048, 'enq: DN - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1049, 'Cluster stabilization wait'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1050, 'Cluster Suspension wait'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1051, 'ASM background starting'                                    , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1052, 'ASM db client exists'                                       , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1053, 'ASM file metadata operation'                                , 'Other'          , 'N', 'msgop'                                   , 'locn'                       , ''                       );
  ins(1054, 'ASM network foreground exits'                               , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1055, 'enq: FA - access file'                                      , 'Other'          , 'N', 'name|mode'                               , 'disk group number'          , 'file number'            );
  ins(1056, 'enq: RX - relocate extent'                                  , 'Other'          , 'N', 'name|mode'                               , 'disk group #:file #'        , 'virtual extent number'  );
  ins(1057, 'log write(odd)'                                             , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1058, 'log write(even)'                                            , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1059, 'checkpoint advanced'                                        , 'Other'          , 'N', 'group#'                                  , ''                           , ''                       );
  ins(1060, 'enq: FR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1061, 'enq: FR - use the thread'                                   , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1062, 'enq: FR - recover the thread'                               , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1063, 'enq: FG - serialize ACD relocate'                           , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'type'                   );
  ins(1064, 'enq: FG - FG redo generation enq race'                      , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'type'                   );
  ins(1065, 'enq: FG - LGWR redo generation enq race'                    , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'type'                   );
  ins(1066, 'enq: FT - allow LGWR writes'                                , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1067, 'enq: FT - disable LGWR writes'                              , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1068, 'enq: FC - open an ACD thread'                               , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1069, 'enq: FC - recover an ACD thread'                            , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'thread'                 );
  ins(1070, 'enq: FX - issue ACD Xtnt Relocation CIC'                    , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , 'unused'                 );
  ins(1071, 'rollback operations block full'                             , 'Other'          , 'N', 'max operations'                          , ''                           , ''                       );
  ins(1072, 'rollback operations active'                                 , 'Other'          , 'N', 'operation count'                         , ''                           , ''                       );
  ins(1073, 'enq: RB - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group'                 , '0'                      );
  ins(1074, 'ASM: MARK subscribe to msg channel'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1075, 'enq: PT - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'disk group #'               , 'type'                   );
  ins(1076, 'ASM PST operation'                                          , 'Other'          , 'N', 'op'                                      , ''                           , ''                       );
  ins(1077, 'global cache busy'                                          , 'Other'          , 'N', 'group'                                   , 'file#'                      , 'block#'                 );
  ins(1078, 'lock release pending'                                       , 'Other'          , 'N', 'group'                                   , 'file#'                      , 'block#'                 );
  ins(1079, 'dma prepare busy'                                           , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1080, 'GCS lock cancel'                                            , 'Other'          , 'N', 'le'                                      , ''                           , ''                       );
  ins(1081, 'GCS lock open S'                                            , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1082, 'GCS lock open X'                                            , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1083, 'GCS lock open'                                              , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1084, 'GCS lock cvt S'                                             , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1085, 'GCS lock cvt X'                                             , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1086, 'GCS lock esc X'                                             , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1087, 'GCS lock esc'                                               , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1088, 'GCS recovery lock open'                                     , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1089, 'GCS recovery lock convert'                                  , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1090, 'kfcl: instance recovery'                                    , 'Other'          , 'N', 'group'                                   , 'obj#'                       , 'block#'                 );
  ins(1091, 'no free locks'                                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1092, 'lock close'                                                 , 'Other'          , 'N', 'group'                                   , 'lms#'                       , ''                       );
  ins(1093, 'enq: KQ - access ASM attribute'                             , 'Other'          , 'N', 'name|mode'                               , 'group'                      , 'entry'                  );
  ins(1094, 'ASM Volume Background'                                      , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1095, 'ASM DG Unblock'                                             , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1096, 'enq: AV - persistent DG number'                             , 'Other'          , 'N', 'name|mode'                               , 'persistent DG number'       , 'non-DG number enqs'     );
  ins(1097, 'enq: AV - volume relocate'                                  , 'Other'          , 'N', 'name|mode'                               , 'persistent DG number'       , 'non-DG number enqs'     );
  ins(1098, 'enq: AV - AVD client registration'                          , 'Other'          , 'N', 'name|mode'                               , 'persistent DG number'       , 'non-DG number enqs'     );
  ins(1099, 'enq: AV - add/enable first volume in DG'                    , 'Other'          , 'N', 'name|mode'                               , 'persistent DG number'       , 'non-DG number enqs'     );
  ins(1100, 'ASM: OFS Cluster membership update'                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1101, 'enq: WF - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1102, 'enq: WP - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1103, 'enq: FU - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1104, 'enq: MW - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'Schedule Id'                , '0'                      );
  ins(1105, 'AWR Flush'                                                  , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1106, 'AWR Metric Capture'                                         , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1107, 'enq: TB - SQL Tuning Base Cache Update'                     , 'Other'          , 'N', 'name|mode'                               , '1'                          , '2'                      );
  ins(1108, 'enq: TB - SQL Tuning Base Cache Load'                       , 'Other'          , 'N', 'name|mode'                               , '1'                          , '2'                      );
  ins(1109, 'enq: SH - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1110, 'enq: AF - task serialization'                               , 'Other'          , 'N', 'name|mode'                               , 'task id'                    , '0'                      );
  ins(1111, 'MMON slave messages'                                        , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1112, 'MMON (Lite) shutdown'                                       , 'Other'          , 'N', 'process#'                                , 'waited'                     , ''                       );
  ins(1113, 'enq: MO - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1114, 'enq: TL - contention'                                       , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1115, 'enq: TH - metric threshold evaluation'                      , 'Other'          , 'N', 'name|mode'                               , '0'                          , '0'                      );
  ins(1116, 'enq: TK - Auto Task Serialization'                          , 'Other'          , 'N', 'name|mode'                               , '0-MMON, 1-MMON Slave'       , '0'                      );
  ins(1117, 'enq: TK - Auto Task Slave Lockout'                          , 'Other'          , 'N', 'name|mode'                               , '0-MMON, 1-MMON Slave'       , '0'                      );
  ins(1118, 'enq: RR - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'lock#'                      , 'not used'               );
  ins(1119, 'WCR: RAC message context busy'                              , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1120, 'WCR: capture file IO write'                                 , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1121, 'WCR: Sync context busy'                                     , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1122, 'latch: WCR: sync'                                           , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins(1123, 'latch: WCR: processes HT'                                   , 'Other'          , 'N', 'address'                                 , 'number'                     , 'tries'                  );
  ins(1124, 'enq: JS - contention'                                       , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1125, 'enq: JS - job run lock - synchronize'                       , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1126, 'enq: JS - job recov lock'                                   , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1127, 'enq: JS - queue lock'                                       , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1128, 'enq: JS - sch locl enqs'                                    , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1129, 'enq: JS - q mem clnup lck'                                  , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1130, 'enq: JS - evtsub add'                                       , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1131, 'enq: JS - evtsub drop'                                      , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1132, 'enq: JS - wdw op'                                           , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1133, 'enq: JS - evt notify'                                       , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1134, 'enq: JS - aq sync'                                          , 'Other'          , 'N', 'name|mode'                               , 'service ID'                 , 'queue type'             );
  ins(1135, 'enq: XD - ASM disk drop/add'                                , 'Other'          , 'N', 'name|mode'                               , 'opcode'                     , 'notused'                );
  ins(1136, 'enq: XD - ASM disk ONLINE'                                  , 'Other'          , 'N', 'name|mode'                               , 'opcode'                     , 'notused'                );
  ins(1137, 'enq: XD - ASM disk OFFLINE'                                 , 'Other'          , 'N', 'name|mode'                               , 'opcode'                     , 'notused'                );
  ins(1138, 'cell worker online completion'                              , 'Other'          , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins(1139, 'cell worker retry '                                         , 'Other'          , 'N', 'cellhash#'                               , ''                           , ''                       );
  ins(1140, 'cell manager cancel work request'                           , 'Other'          , 'N', ''                                        , ''                           , ''                       );
  ins(1141, 'secondary event'                                            , 'Other'          , 'N', 'event #'                                 , 'wait time'                  , ''                       );
END;
/
SET DEF ON;

DROP PROCEDURE ins;

/* ------------------------------------------------------------------------- */

-- one row per execution of the Trace Analyzer
CREATE TABLE trca$_tool_execution (
  id                          INTEGER        NOT NULL, -- pk trca$_tool_execution_id_s
  file_name                   VARCHAR2(4000),          -- trace or control text file
  directory_alias             VARCHAR2(30),            -- fk dba_directories.directory_name
  directory_path              VARCHAR2(4000),          -- input directory path
  parse_start                 DATE,
  parse_end                   DATE,
  file_count                  INTEGER,
  file_bytes                  INTEGER,
  start_tim                   INTEGER,                 -- first tim in file(s)
  end_tim                     INTEGER,                 -- last tim in file(s)
  dep                         INTEGER,                 -- max recursive depth
  accounted_for_response_time INTEGER,                 -- total for all file(s)
  elapsed_time                INTEGER,                 -- total for all file(s)
  cpu_time                    INTEGER,                 -- total for all file(s)
  log_file_name               VARCHAR2(4000),
  html_file_name              VARCHAR2(4000),
  text_file_name              VARCHAR2(4000)
);

CREATE UNIQUE INDEX trca$_tool_execution_pk ON trca$_tool_execution(id);
ALTER TABLE trca$_tool_execution ADD (CONSTRAINT trca$_tool_execution_pk PRIMARY KEY (id));

GRANT SELECT ON trca$_tool_execution TO &&role_name.;

/* ------------------------------------------------------------------------- */

-- one row per file created by Trace Analyzer
CREATE TABLE trca$_file (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  file_type                  VARCHAR2(24),            -- HTML, TEXT, LOG, 10053, 10046
  filename                   VARCHAR2(128),
  file_date                  DATE,
  file_size                  NUMBER,
  username                   VARCHAR2(30),
  file_text                  CLOB
);

GRANT SELECT ON trca$_file TO &&role_name.;

/* ------------------------------------------------------------------------- */

-- staging table to hold names of traces inside control file
CREATE GLOBAL TEMPORARY TABLE trca$_files (
  filename                   VARCHAR2(256),
  directory_alias            VARCHAR2(30),
  order_by                   NUMBER
) ON COMMIT PRESERVE ROWS;

/*------------------------------------------------------------------*/

-- one row for each file analyzed during one execution
CREATE TABLE trca$_trace (
  id                         INTEGER        NOT NULL, -- pk trca$_trace_id_s
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  file_name                  VARCHAR2(4000) NOT NULL, -- trace file name
  file_len                   INTEGER,                 -- bytes in trace
  status                     VARCHAR2(30),            -- PARSING/COMPLETED/ERROR
  parse_start                DATE,
  parse_end                  DATE,
  parsed_lines               INTEGER,                 -- current parsing trace line
  parsed_bytes               INTEGER                  -- current parsing trace byte
);

CREATE UNIQUE INDEX trca$_trace_pk ON trca$_trace(id);
ALTER TABLE trca$_trace ADD (CONSTRAINT trca$_trace_pk PRIMARY KEY (id));

CREATE INDEX trca$_trace_n1 ON trca$_trace(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- all trace lines before first SESSION
CREATE TABLE trca$_trace_header (
  id                         INTEGER        NOT NULL, -- pk trca$_header_id_s
  trace_id                   INTEGER        NOT NULL, -- fk trca$_trace.id
  piece                      INTEGER        NOT NULL, -- header line number
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  text                       VARCHAR2(4000)
);

CREATE UNIQUE INDEX trca$_trace_header_pk ON trca$_trace_header(id);
ALTER TABLE trca$_trace_header ADD (CONSTRAINT trca$_trace_header_pk PRIMARY KEY (id));

CREATE INDEX trca$_trace_header_n1 ON trca$_trace_header(tool_execution_id, trace_id);

/* ------------------------------------------------------------------------- */

-- one row for each SESSION found in trace
CREATE TABLE trca$_session (
  id                         INTEGER        NOT NULL, -- pk trca$_session_id_s
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  trace_id                   INTEGER        NOT NULL, -- fk trca$_trace.id
  sid                        INTEGER        NOT NULL,
  serial#                    INTEGER        NOT NULL,
  session_timestamp          TIMESTAMP      NOT NULL,
  session_tim                INTEGER,
  read_only_committed        INTEGER,
  read_only_rollbacked       INTEGER,
  update_committed           INTEGER,
  update_rollbacked          INTEGER
);

CREATE UNIQUE INDEX trca$_session_pk ON trca$_session(id);
ALTER TABLE trca$_session ADD (CONSTRAINT trca$_session_pk PRIMARY KEY (id));

CREATE INDEX trca$_session_n1 ON trca$_session(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- one row for each GAP (***) found in trace
CREATE TABLE trca$_gap (
  id                         INTEGER        NOT NULL, -- pk trca$_gap_id_s
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  trace_id                   INTEGER        NOT NULL, -- fk trca$_trace.id
  gap_timestamp              TIMESTAMP      NOT NULL,
  tim_before                 INTEGER,
  tim_after                  INTEGER,
  ela_after                  INTEGER,
  wait_call_after            CHAR(1),                -- (W)wait/(C)all
  call_id_after              INTEGER                 -- fk trca$_call.id
);

CREATE UNIQUE INDEX trca$_gap_pk ON trca$_gap(id);
ALTER TABLE trca$_gap ADD (CONSTRAINT trca$_gap_pk PRIMARY KEY (id));

CREATE INDEX trca$_gap_n1 ON trca$_gap(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- call stack for calls associated to gaps
CREATE TABLE trca$_gap_call (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  gap_id                     INTEGER        NOT NULL, -- fk trca$_gap.id
  call_id                    INTEGER        NOT NULL, -- fk trca$_call.id
  call                       INTEGER        NOT NULL, -- call enumerator
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  dep                        INTEGER,
  c                          INTEGER,
  e                          INTEGER,
  tim                        INTEGER,
  parent_dep_id              INTEGER                  -- fk trca$_call.dep_id
);

CREATE INDEX trca$_gap_call_n1 ON trca$_gap_call(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- one row for each unique SQL statement accoss all executions of this tool
CREATE TABLE trca$_statement (
  id                         INTEGER        NOT NULL, -- pk trca$_statement_id_s
  len                        INTEGER        NOT NULL,
  hv                         INTEGER        NOT NULL,
  sqlid                      VARCHAR2(16),            -- 11g+
  oct                        INTEGER        NOT NULL, -- fk trca$_audit_actions.action
  sql_text                   VARCHAR2(1000) NOT NULL,
  sql_fulltext               CLOB           NOT NULL
);

INSERT INTO trca$_statement VALUES (-1, 7, -1, '-1', 0, 'UNKNOWN', 'UNKNOWN');

CREATE UNIQUE INDEX trca$_statement_pk ON trca$_statement(id);
ALTER TABLE trca$_statement ADD (CONSTRAINT trca$_statement_pk PRIMARY KEY (id));

CREATE INDEX trca$_statement_n1 ON trca$_statement(len, hv, oct, sqlid);

/* ------------------------------------------------------------------------- */

-- one row for each PARSING trace line
CREATE &&global_temporary. TABLE trca$_cursor (
  id                         INTEGER        NOT NULL, -- pk trca$_cursor_id_s
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  trace_id                   INTEGER        NOT NULL, -- fk trca$_trace.id
  statement_id               INTEGER        NOT NULL, -- fk trca$_statement.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id (cursor_id or statement_id as per g_aggregate)
  cursor_num                 INTEGER        NOT NULL, -- as shown on trace
  dep                        INTEGER        NOT NULL,
  uid#                       INTEGER        NOT NULL,
  lid                        INTEGER        NOT NULL,
  tim                        INTEGER,
  ad                         VARCHAR2(32),
  err                        INTEGER,
  session_id                 NUMBER,                  -- fk trca$_session.id
  trace_line                 INTEGER
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- one row for each PARSING for each unique SQL as per g_aggregate
CREATE TABLE trca$_group (
  id                         INTEGER        NOT NULL, -- pk cursor_id or statement_id (fk trca$_cursor.id or trca$_statement.id)
  tool_execution_id          INTEGER        NOT NULL, -- pk fk trca$_tool_execution.id
  statement_id               INTEGER        NOT NULL, -- fk trca$_statement.id
  first_cursor_id            INTEGER        NOT NULL, -- fk trca$_cursor.id
  uid#                       INTEGER        NOT NULL, -- trca$_cursor.uid# (denormalized)
  lid                        INTEGER        NOT NULL, -- trca$_cursor.lid (denormalized)
  dep                        INTEGER        NOT NULL, -- trca$_cursor.dep (denormalized)
  plh                        INTEGER,                 -- 11.1.0.7+
  err                        INTEGER,                 -- trca$_cursor.err (denormalized)
  first_exec_id              INTEGER,                 -- fk trca$_exec.id
  last_exec_id               INTEGER,                 -- fk trca$_exec.id
  exec_count                 INTEGER,                 -- trca$_group_call.call_count
  response_time_self         INTEGER,                 -- elapsed plus wait idle
  response_time_progeny      INTEGER,                 -- ditto for recursive sql
  contribution               NUMBER,                  -- fraction between 0 and 1 over trace(s) total response time
  rank                       INTEGER,                 -- one means first top sql as per response time
  top_sql                    CHAR(1),                 -- Y/N as per reposnse time
  elapsed_time_self          INTEGER,                 -- elapsed time
  elapsed_time_progeny       INTEGER,                 -- ditto for recursive sql
  contribution_et            NUMBER,                  -- fraction between 0 and 1 over trace(s) total elapsed time
  rank_et                    INTEGER,                 -- one means first top sql as per elapsed time
  top_sql_et                 CHAR(1),                 -- Y/N as per elapsed time
  cpu_time_self              INTEGER,                 -- cpu time
  cpu_time_progeny           INTEGER,                 -- ditto for recursive sql
  contribution_ct            NUMBER,                  -- fraction between 0 and 1 over trace(s) total cpu time
  rank_ct                    INTEGER,                 -- one means first top sql as per cpu time
  top_sql_ct                 CHAR(1),                 -- Y/N as per cpu time
  include_details            CHAR(1),                 -- Y/N
  trca_plan_hash_value       INTEGER,                 -- for explain plan
  rows_processed             INTEGER
);

CREATE UNIQUE INDEX trca$_group_pk ON trca$_group(id, tool_execution_id);
ALTER TABLE trca$_group ADD (CONSTRAINT trca$_group_pk PRIMARY KEY (id, tool_execution_id));

CREATE INDEX trca$_group_n1 ON trca$_group(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- one row for selected executions according to trca$t.compute_exec_rank (not all executions are materialized here)
CREATE TABLE trca$_exec (
  id                         INTEGER        NOT NULL, -- pk trca$_exec_id_s
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  dep                        INTEGER        NOT NULL,
  plh                        INTEGER,                 -- 11.1.0.7+
  start_tim                  INTEGER        NOT NULL, -- of first call
  end_tim                    INTEGER        NOT NULL, -- of last call
  response_time_self         INTEGER        NOT NULL, -- elapsed plus wait idle
  response_time_progeny      INTEGER        NOT NULL, -- ditto for recursive sql
  grp_contribution           NUMBER,                  -- fraction between 0 and 1 over group total response time
  trc_contribution           NUMBER,                  -- fraction between 0 and 1 over trace total response time
  first_exec                 CHAR(1)        NOT NULL, -- Y/N
  last_exec                  CHAR(1)        NOT NULL, -- Y/N
  top_exec                   CHAR(1)        NOT NULL, -- Y/N
  rank                       INTEGER        NOT NULL  -- one means first top execution for one group
);

CREATE UNIQUE INDEX trca$_exec_pk ON trca$_exec(id);
ALTER TABLE trca$_exec ADD (CONSTRAINT trca$_exec_pk PRIMARY KEY (id));

CREATE INDEX trca$_exec_n1 ON trca$_exec(tool_execution_id, group_id);

/* ------------------------------------------------------------------------- */

-- one row per distinct wait event name found in one execution of this tool
CREATE TABLE trca$_wait_event_name (
  tool_execution_id          INTEGER        NOT NULL, -- pk (fk trca$_tool_execution.id)
  event#                     INTEGER        NOT NULL, -- pk (fk trca$_event_name.event# if source is V)
  name                       VARCHAR2(64)   NOT NULL,
  wait_class                 VARCHAR2(64)   NOT NULL,
  idle                       CHAR(1)        NOT NULL, -- Y/N
  source                     CHAR(1)        NOT NULL, -- (V)iew, (T)trace
  parameter1v                VARCHAR2(64),            -- sourced by trca$_event_name
  parameter2v                VARCHAR2(64),
  parameter3v                VARCHAR2(64),
  parameter1t                VARCHAR2(64),            -- sourced by trace p1, p2 and p3 names
  parameter2t                VARCHAR2(64),
  parameter3t                VARCHAR2(64)
);

CREATE UNIQUE INDEX trca$_wait_event_name_pk ON trca$_wait_event_name(tool_execution_id, event#);
ALTER TABLE trca$_wait_event_name ADD (CONSTRAINT trca$_wait_event_name_pk PRIMARY KEY (tool_execution_id, event#));

/* ------------------------------------------------------------------------- */

-- one row for each WAIT line
CREATE &&global_temporary. TABLE trca$_wait (
  call_id                    INTEGER        NOT NULL, -- fk trca$_call.id
  event#                     INTEGER        NOT NULL, -- fk trca$_wait_event_name.event#
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  ela                        INTEGER        NOT NULL,
  p1                         INTEGER,
  p2                         INTEGER,
  p3                         INTEGER,
  obj#                       INTEGER,
  tim                        INTEGER
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- one row for each Bind line. Several columns are disabled to save storage space.
CREATE &&global_temporary. TABLE trca$_bind (
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  group_id                   INTEGER,                 -- fk trca$_group.id (can be null)
  --cursor_id                  INTEGER        NOT NULL, -- fk trca$_cursor.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  bind                       INTEGER        NOT NULL,
  oacdef                     CHAR(1)        NOT NULL, -- Y/N. N=(No oacdef for this bind)
  oacdty                     INTEGER,
  --mxl                        INTEGER,
  --pmxl                       INTEGER,
  --mxlc                       INTEGER,
  --mal                        INTEGER,
  --scl                        INTEGER,
  --pre                        INTEGER,
  --oacflg                     INTEGER,
  --oacf12                     VARCHAR2(32), -- oacfl2 can be b38f0000000001 on 9.2.0.6 HP-UX
  --frm                        INTEGER,
  --csi                        INTEGER,
  --siz                        INTEGER,
  --offset                     INTEGER,
  --kxsbbbfp                   VARCHAR2(32),
  --bln                        INTEGER,
  avl                        INTEGER,
  --flg                        INTEGER,
  value                      VARCHAR2(4000)
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- bind variable values per execution
CREATE TABLE trca$_exec_binds (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id
  bind                       INTEGER        NOT NULL,
  data_type_code             INTEGER,
  data_type_name             VARCHAR2(128),
  actual_value_length        INTEGER,
  oacdef                     CHAR(1)        NOT NULL, -- Y/N. N=(No oacdef for this bind)
  value                      VARCHAR2(4000)
);

CREATE INDEX trca$_exec_binds_n1 ON trca$_exec_binds(tool_execution_id, group_id, exec_id);

/* ------------------------------------------------------------------------- */

-- one row for each STAT line. Some columns are disabled to save storage space.
CREATE &&global_temporary. TABLE trca$_stat (
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  cursor_id                  INTEGER        NOT NULL, -- fk trca$_cursor.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  --cursor_num                 INTEGER,
  session_id                 NUMBER,                  -- fk trca$_session.id (denormalized)
  id                         INTEGER        NOT NULL,
  cnt                        INTEGER,
  pid                        INTEGER,
  pos                        INTEGER,
  obj                        INTEGER,
  op                         VARCHAR2(4000),
  cr                         INTEGER,
  pr                         INTEGER,
  pw                         INTEGER,
  time                       INTEGER,
  cost                       INTEGER,
  siz                        INTEGER,
  card                       INTEGER
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- summary table with one row per execution set, indicating its plan hash value
CREATE TABLE trca$_stat_exec (
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  trca_plan_hash_value       INTEGER        NOT NULL
);

CREATE UNIQUE INDEX trca$_stat_exec_pk ON trca$_stat_exec(exec_id);
ALTER TABLE trca$_stat_exec ADD (CONSTRAINT trca$_stat_exec_pk PRIMARY KEY (exec_id));

CREATE INDEX trca$_stat_exec_n1 ON trca$_stat_exec(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- plan hash values per group (normally one). This is for row source plans.
CREATE TABLE trca$_group_row_source_plan (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  trca_plan_hash_value       INTEGER        NOT NULL, -- zero, one or many per group
  first_exec_id              INTEGER        NOT NULL  -- first_exec_id for given trca_plan_hash_value
);

CREATE UNIQUE INDEX trca$_group_row_source_plan_pk ON trca$_group_row_source_plan(tool_execution_id, group_id, trca_plan_hash_value);
ALTER TABLE trca$_group_row_source_plan ADD (CONSTRAINT trca$_group_row_source_plan_pk PRIMARY KEY (tool_execution_id, group_id, trca_plan_hash_value));

/* ------------------------------------------------------------------------- */

-- aggregate row source plan per trca_plan_hash_value.
CREATE TABLE trca$_row_source_plan (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  trca_plan_hash_value       INTEGER        NOT NULL, -- row source plan
  id                         INTEGER        NOT NULL,
  pid                        INTEGER,
  depth                      INTEGER,
  cnt                        INTEGER,
  pos                        INTEGER,
  obj                        INTEGER,
  op                         VARCHAR2(4000),
  cr                         INTEGER,
  pr                         INTEGER,
  pw                         INTEGER,
  time                       INTEGER,
  cost                       INTEGER,
  siz                        INTEGER,
  card                       INTEGER,
  sessions                   INTEGER
);

CREATE UNIQUE INDEX trca$_row_source_plan_pk ON trca$_row_source_plan(tool_execution_id, group_id, trca_plan_hash_value, id);
ALTER TABLE trca$_row_source_plan ADD (CONSTRAINT trca$_row_source_plan_pk PRIMARY KEY (tool_execution_id, group_id, trca_plan_hash_value, id));

/* ------------------------------------------------------------------------- */

-- aggregate row source plan per trca_plan_hash_value and session.
CREATE TABLE trca$_row_source_plan_session (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  trca_plan_hash_value       INTEGER        NOT NULL, -- row source plan
  session_id                 INTEGER        NOT NULL,
  id                         INTEGER        NOT NULL,
  pid                        INTEGER,
  depth                      INTEGER,
  cnt                        INTEGER,
  pos                        INTEGER,
  obj                        INTEGER,
  op                         VARCHAR2(4000),
  cr                         INTEGER,
  pr                         INTEGER,
  pw                         INTEGER,
  time                       INTEGER,
  cost                       INTEGER,
  siz                        INTEGER,
  card                       INTEGER
);

CREATE UNIQUE INDEX trca$_row_source_plan_sess_pk ON trca$_row_source_plan_session(tool_execution_id, group_id, trca_plan_hash_value, session_id, id);
ALTER TABLE trca$_row_source_plan_session ADD (CONSTRAINT trca$_row_source_plan_sess_pk PRIMARY KEY (tool_execution_id, group_id, trca_plan_hash_value, session_id, id));

/* ------------------------------------------------------------------------- */

-- one row per ERROR line. Some columns are disabled for consistency with other tables.
CREATE TABLE trca$_error (
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  group_id                   INTEGER,                 -- fk trca$_group.id (can be null)
  --cursor_id                  INTEGER        NOT NULL, -- fk trca$_cursor.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  --cursor_num                 INTEGER,
  err                        INTEGER,
  tim                        INTEGER
);

CREATE INDEX trca$_error_n1 ON trca$_error(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- one row per db call (PARSE, EXEC, FETCH, UNMAP, SORT_UNMAP)
CREATE &&global_temporary. TABLE trca$_call (
  id                         INTEGER        NOT NULL, -- pk trca$_call_id_s
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  --cursor_id                  INTEGER        NOT NULL, -- fk trca$_cursor.id
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  -- parse
  call                       CHAR(1)        NOT NULL, -- CALL enumerator (trca$g)
  -- aggregate stats (this plus direct children)
  c                          INTEGER,
  e                          INTEGER,
  p                          INTEGER,
  cr                         INTEGER,
  cu                         INTEGER,
  -- non aggregate stats (this)
  mis                        INTEGER,
  r                          INTEGER,
  -- attributes
  dep                        INTEGER,
  --og                         INTEGER,                -- irrelevant
  plh                        INTEGER,                 -- 11.1.0.7+
  tim                        INTEGER,
  -- recursive control (call dependencies)
  dep_id                     INTEGER,                 -- unique. null if leaf (no children underneath)
  parent_dep_id              INTEGER,                 -- null if dep=0 (root)
  -- recursive call metrics (direct children)
  recu_c                     INTEGER,
  recu_e                     INTEGER,
  recu_p                     INTEGER,
  recu_cr                    INTEGER,
  recu_cu                    INTEGER,
  recu_call_count            INTEGER,
  recu_mis                   INTEGER,
  recu_r                     INTEGER,
  -- recursive waits metrics
  self_wait_count_idle       INTEGER,
  self_wait_count_non_idle   INTEGER,
  self_wait_ela_idle         INTEGER,
  self_wait_ela_non_idle     INTEGER,
  recu_wait_count_idle       INTEGER,
  recu_wait_count_non_idle   INTEGER,
  recu_wait_ela_idle         INTEGER,
  recu_wait_ela_non_idle     INTEGER
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- edges of the call dependency tree (sql genealogy). nodes are defined by dep_id and parent_dep_id.
CREATE &&global_temporary. TABLE trca$_call_tree (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id (denormalized)
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id (denormalized)
  parent_group_id            INTEGER,                 -- fk trca$_group.id (denormalized)
  dep_id                     INTEGER,                 -- fk trca$_call.dep_id (lower node) null for leafs
  parent_dep_id              INTEGER,                 -- fk trca$_call.parent_dep_id (upper node) null for roots
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected). this is the edge
  response_time_self         INTEGER        NOT NULL, -- elapsed plus wait idle for this edge
  response_time_progeny      INTEGER        NOT NULL  -- ditto for recursive sql (branches underneeth)
) &&on_commit_preserve_rows.;

CREATE INDEX trca$_call_tree_n1 ON trca$_call_tree(tool_execution_id, parent_dep_id);

/* ------------------------------------------------------------------------- */

-- edges of the call dependency tree (sql genealogy). nodes are defined by dep_id and parent_dep_id.
CREATE &&global_temporary. TABLE trca$_exec_tree (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  root_group_id              INTEGER        NOT NULL, -- non-recursive sql
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id (lower node) null for leafs
  parent_group_id            INTEGER,                 -- fk trca$_group.id (upper node) null for roots
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected). this is the edge
  response_time_self         INTEGER        NOT NULL, -- elapsed plus wait idle for this edge
  response_time_progeny      INTEGER        NOT NULL, -- ditto for recursive sql (branches underneeth)
  dep                        INTEGER        NOT NULL,
  path                       VARCHAR2(256)            -- from root to leaf excluding this
) &&on_commit_preserve_rows.;

/* ------------------------------------------------------------------------- */

-- aggregate of execution tree edges since same edge can have multiple executions
CREATE &&global_temporary. TABLE trca$_genealogy_edge (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  root_group_id              INTEGER        NOT NULL, -- non-recursive sql
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id (lower node) null for leafs
  parent_group_id            INTEGER,                 -- fk trca$_group.id (upper node) null for roots
  first_exec_id              INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected). this is the edge
  exec_count                 INTEGER        NOT NULL, -- number of executions for this edge
  response_time_self         INTEGER        NOT NULL, -- elapsed plus wait idle for this edge
  response_time_progeny      INTEGER        NOT NULL, -- ditto for recursive sql (branches underneeth)
  dep                        INTEGER        NOT NULL,
  path                       VARCHAR2(256)            -- from root to leaf excluding this
) &&on_commit_preserve_rows.;

CREATE INDEX trca$_genealogy_edge_n1 ON trca$_genealogy_edge(tool_execution_id, root_group_id, group_id);
CREATE INDEX trca$_genealogy_edge_n2 ON trca$_genealogy_edge(tool_execution_id, root_group_id, parent_group_id);

/* ------------------------------------------------------------------------- */

-- same as trca$_genealogy_edge but tree order is incorporated (id)
CREATE TABLE trca$_genealogy (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  root_group_id              INTEGER        NOT NULL, -- non-recursive sql
  id                         INTEGER        NOT NULL, -- order in this tree
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id (lower node) null for leafs
  parent_group_id            INTEGER,                 -- fk trca$_group.id (upper node) null for roots
  first_exec_id              INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected). this is the edge
  exec_count                 INTEGER        NOT NULL, -- number of executions for this edge
  response_time_self         INTEGER        NOT NULL, -- elapsed plus wait idle for this edge
  response_time_progeny      INTEGER        NOT NULL, -- ditto for recursive sql (branches underneeth)
  dep                        INTEGER        NOT NULL,
  path                       VARCHAR2(256)            -- from root to leaf excluding this
);

CREATE INDEX trca$_genealogy_n1 ON trca$_genealogy(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- calls aggregate per execution
CREATE TABLE trca$_group_exec_call (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  call                       CHAR(1)        NOT NULL, -- CALL enumerator (trca$g)
  -- aggregate stats (this plus direct children)
  c                          INTEGER,
  e                          INTEGER,
  p                          INTEGER,
  cr                         INTEGER,
  cu                         INTEGER,
  -- non aggregate stats (this)
  call_count                 INTEGER,
  mis                        INTEGER,
  r                          INTEGER,
  -- attributes
  dep                        INTEGER,
  plh                        INTEGER, -- 11.1.0.7+
  -- recursive call metrics (direct children)
  recu_c                     INTEGER,
  recu_e                     INTEGER,
  recu_p                     INTEGER,
  recu_cr                    INTEGER,
  recu_cu                    INTEGER,
  recu_call_count            INTEGER,
  recu_mis                   INTEGER,
  recu_r                     INTEGER,
  -- recursive waits metrics
  self_wait_count_idle       INTEGER,
  self_wait_count_non_idle   INTEGER,
  self_wait_ela_idle         INTEGER,
  self_wait_ela_non_idle     INTEGER,
  recu_wait_count_idle       INTEGER,
  recu_wait_count_non_idle   INTEGER,
  recu_wait_ela_idle         INTEGER,
  recu_wait_ela_non_idle     INTEGER
);

CREATE UNIQUE INDEX trca$_group_exec_call_pk ON trca$_group_exec_call(tool_execution_id, group_id, exec_id, call);
ALTER TABLE trca$_group_exec_call ADD (CONSTRAINT trca$_group_exec_call_pk PRIMARY KEY (tool_execution_id, group_id, exec_id, call));

/* ------------------------------------------------------------------------- */

-- calls aggregate per group
CREATE TABLE trca$_group_call (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  call                       CHAR(1)        NOT NULL, -- CALL enumerator (trca$g)
  -- aggregate stats (this plus direct children)
  c                          INTEGER,
  e                          INTEGER,
  p                          INTEGER,
  cr                         INTEGER,
  cu                         INTEGER,
  -- non aggregate stats (this)
  call_count                 INTEGER,
  mis                        INTEGER,
  r                          INTEGER,
  -- attributes
  dep                        INTEGER,
  plh                        INTEGER, -- 11.1.0.7+
  -- recursive call metrics (direct children)
  recu_c                     INTEGER,
  recu_e                     INTEGER,
  recu_p                     INTEGER,
  recu_cr                    INTEGER,
  recu_cu                    INTEGER,
  recu_call_count            INTEGER,
  recu_mis                   INTEGER,
  recu_r                     INTEGER,
  -- recursive waits metrics
  self_wait_count_idle       INTEGER,
  self_wait_count_non_idle   INTEGER,
  self_wait_ela_idle         INTEGER,
  self_wait_ela_non_idle     INTEGER,
  recu_wait_count_idle       INTEGER,
  recu_wait_count_non_idle   INTEGER,
  recu_wait_ela_idle         INTEGER,
  recu_wait_ela_non_idle     INTEGER
);

CREATE UNIQUE INDEX trca$_group_call_pk ON trca$_group_call(tool_execution_id, group_id, call);
ALTER TABLE trca$_group_call ADD (CONSTRAINT trca$_group_call_pk PRIMARY KEY (tool_execution_id, group_id, call));

/* ------------------------------------------------------------------------- */

-- calls aggregate per tool execution (overall totals)
CREATE TABLE trca$_tool_exec_call (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  recursive                  CHAR(1)        NOT NULL, -- Y/N
  call                       CHAR(1)        NOT NULL, -- CALL enumerator (trca$g)
  -- aggregate stats (this plus direct children)
  c                          INTEGER,
  e                          INTEGER,
  p                          INTEGER,
  cr                         INTEGER,
  cu                         INTEGER,
  -- non aggregate stats (this)
  call_count                 INTEGER,
  mis                        INTEGER,
  r                          INTEGER,
  -- recursive call metrics (direct children)
  recu_c                     INTEGER,
  recu_e                     INTEGER,
  recu_p                     INTEGER,
  recu_cr                    INTEGER,
  recu_cu                    INTEGER,
  recu_call_count            INTEGER,
  recu_mis                   INTEGER,
  recu_r                     INTEGER,
  -- recursive waits metrics
  self_wait_count_idle       INTEGER,
  self_wait_count_non_idle   INTEGER,
  self_wait_ela_idle         INTEGER,
  self_wait_ela_non_idle     INTEGER,
  recu_wait_count_idle       INTEGER,
  recu_wait_count_non_idle   INTEGER,
  recu_wait_ela_idle         INTEGER,
  recu_wait_ela_non_idle     INTEGER
);

CREATE UNIQUE INDEX trca$_tool_exec_call_pk ON trca$_tool_exec_call(tool_execution_id, recursive, call);
ALTER TABLE trca$_tool_exec_call ADD (CONSTRAINT trca$_tool_exec_call_pk PRIMARY KEY (tool_execution_id, recursive, call));

/* ------------------------------------------------------------------------- */

-- waits aggregate per tool execution (overall totals)
CREATE TABLE trca$_tool_wait (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  dep                        INTEGER        NOT NULL,
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  blocks                     INTEGER
);

CREATE UNIQUE INDEX trca$_tool_wait_pk ON trca$_tool_wait(tool_execution_id, dep, event#);
ALTER TABLE trca$_tool_wait ADD (CONSTRAINT trca$_tool_wait_pk PRIMARY KEY (tool_execution_id, dep, event#));

/* ------------------------------------------------------------------------- */

-- waits aggregate per group
CREATE TABLE trca$_group_wait (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  blocks                     INTEGER
);

CREATE UNIQUE INDEX trca$_group_wait_pk ON trca$_group_wait(tool_execution_id, group_id, event#);
ALTER TABLE trca$_group_wait ADD (CONSTRAINT trca$_group_wait_pk PRIMARY KEY (tool_execution_id, group_id, event#));

/* ------------------------------------------------------------------------- */

-- waits aggregate per execution
CREATE TABLE trca$_group_exec_wait (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  blocks                     INTEGER
);

CREATE UNIQUE INDEX trca$_group_exec_wait_pk ON trca$_group_exec_wait(tool_execution_id, group_id, exec_id, event#);
ALTER TABLE trca$_group_exec_wait ADD (CONSTRAINT trca$_group_exec_wait_pk PRIMARY KEY (tool_execution_id, group_id, exec_id, event#));

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_objects (
  object_id                  VARCHAR2(255),
  object_type                VARCHAR2(255),
  owner                      VARCHAR2(255),
  object_name                VARCHAR2(255),
  subobject_name             VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_OBJECTS.bad'
  LOGFILE 'TRCA_OBJECTS.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_OBJECTS.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_objects');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_objects seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_objects$ (
  object_id                  INTEGER        NOT NULL,
  object_type                VARCHAR2(20),
  owner                      VARCHAR2(30),
  object_name                VARCHAR2(128),
  subobject_name             VARCHAR2(30)
);

CREATE UNIQUE INDEX trca$_objects$_u1 ON trca$_objects$(object_id);

/* ------------------------------------------------------------------------- */

-- objects referenced by waits
CREATE TABLE trca$_objects (
  tool_execution_id          INTEGER        NOT NULL, -- pk fk trca$_tool_execution.id
  object_id                  INTEGER        NOT NULL, -- pk fk dba_objects.object_id
  object_type                VARCHAR2(20),
  owner                      VARCHAR2(30),
  object_name                VARCHAR2(128),
  subobject_name             VARCHAR2(30)
);

CREATE UNIQUE INDEX trca$_objects_pk ON trca$_objects(tool_execution_id, object_id);
ALTER TABLE trca$_objects ADD (CONSTRAINT trca$_objects_pk PRIMARY KEY (tool_execution_id, object_id));

/* ------------------------------------------------------------------------- */

-- wait segments per tool execution (overall totals)
CREATE TABLE trca$_tool_wait_segment (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  dep                        INTEGER        NOT NULL,
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  start_tim                  INTEGER,
  end_tim                    INTEGER,
  blocks                     INTEGER,
  obj#                       INTEGER,
  segment_type               VARCHAR2(20),
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30)
);

CREATE INDEX trca$_tool_wait_segment_n1 ON trca$_tool_wait_segment(tool_execution_id, dep, event#);

/* ------------------------------------------------------------------------- */

-- wait segments per group
CREATE TABLE trca$_group_wait_segment (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  start_tim                  INTEGER,
  end_tim                    INTEGER,
  blocks                     INTEGER,
  obj#                       INTEGER,
  segment_type               VARCHAR2(20),
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30)
);

CREATE INDEX trca$_group_wait_segment_n1 ON trca$_group_wait_segment(tool_execution_id, group_id, event#);

/* ------------------------------------------------------------------------- */

-- wait segments per execution
CREATE TABLE trca$_group_exec_wait_segment (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER        NOT NULL, -- fk trca$_group.id
  exec_id                    INTEGER        NOT NULL, -- fk trca$_exec.id (orphans are expected)
  event#                     INTEGER        NOT NULL, -- trca$_wait_event_name.event#
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  start_tim                  INTEGER,
  end_tim                    INTEGER,
  blocks                     INTEGER,
  obj#                       INTEGER,
  segment_type               VARCHAR2(20),
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30)
);

CREATE INDEX trca$_group_exec_wait_seg_n1 ON trca$_group_exec_wait_segment(tool_execution_id, group_id, exec_id, event#);

/* ------------------------------------------------------------------------- */

-- hot blocks
CREATE TABLE trca$_hot_block (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  p1                         INTEGER        NOT NULL,
  p2                         INTEGER        NOT NULL,
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  obj#                       INTEGER
);

CREATE INDEX trca$_hot_block_n1 ON trca$_hot_block(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- hot blocks with segment details
CREATE TABLE trca$_hot_block_segment (
  tool_execution_id          INTEGER        NOT NULL, -- fk trca$_tool_execution.id
  p1                         INTEGER        NOT NULL,
  p2                         INTEGER        NOT NULL,
  ela                        INTEGER        NOT NULL,
  wait_count                 INTEGER        NOT NULL,
  max_ela                    INTEGER        NOT NULL,
  obj#                       INTEGER,
  segment_type               VARCHAR2(20),
  owner                      VARCHAR2(30),
  segment_name               VARCHAR2(128),
  partition_name             VARCHAR2(30)
);

CREATE INDEX trca$_hot_block_segment_n1 ON trca$_hot_block_segment(tool_execution_id);

/* ------------------------------------------------------------------------- */

-- lookup table
CREATE TABLE trca$_audit_actions AS
SELECT * FROM audit_actions;

CREATE UNIQUE INDEX trca$_audit_actions_pk ON trca$_audit_actions(action);
ALTER TABLE trca$_audit_actions ADD (CONSTRAINT trca$_audit_actions_pk PRIMARY KEY (action));

/* ------------------------------------------------------------------------- */

-- for explain plans
CREATE TABLE trca$_plan_table (
  statement_id               VARCHAR2(30),
  plan_id                    NUMBER,
  timestamp                  DATE,
  remarks                    VARCHAR2(4000),
  operation                  VARCHAR2(30),
  options                    VARCHAR2(255),
  object_node                VARCHAR2(128),
  object_owner               VARCHAR2(30),
  object_name                VARCHAR2(30),
  object_alias               VARCHAR2(65),
  object_instance            NUMERIC,
  object_type                VARCHAR2(30),
  optimizer                  VARCHAR2(255),
  search_columns             NUMBER,
  id                         NUMERIC,
  parent_id                  NUMERIC,
  depth                      NUMERIC,
  position                   NUMERIC,
  cost                       NUMERIC,
  cardinality                NUMERIC,
  bytes                      NUMERIC,
  other_tag                  VARCHAR2(255),
  partition_start            VARCHAR2(255),
  partition_stop             VARCHAR2(255),
  partition_id               NUMERIC,
  other                      CLOB,
  distribution               VARCHAR2(30),
  cpu_cost                   NUMERIC,
  io_cost                    NUMERIC,
  temp_space                 NUMERIC,
  access_predicates          VARCHAR2(4000),
  filter_predicates          VARCHAR2(4000),
  projection                 VARCHAR2(4000),
  time                       NUMERIC,
  qblock_name                VARCHAR2(30),
  other_xml                  CLOB,
  -- trca
  tool_execution_id          INTEGER, -- fk trca$_tool_execution.id
  group_id                   INTEGER, -- fk trca$_group.id
  actual_rows                INTEGER
);

CREATE INDEX trca$_plan_table_n1 ON trca$_plan_table(tool_execution_id, group_id);

GRANT INSERT ON trca$_plan_table TO &&role_name.;

/* ------------------------------------------------------------------------- */

-- staging table to get tables and indexes out of explain plan and row source plan
CREATE GLOBAL TEMPORARY TABLE trca$_pivot (
  object_name                VARCHAR2(30),
  object_owner               VARCHAR2(30)
) ON COMMIT DELETE ROWS;

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_tables (
  owner                      VARCHAR2(255),
  table_name                 VARCHAR2(255),
  num_rows                   VARCHAR2(255),
  blocks                     VARCHAR2(255),
  empty_blocks               VARCHAR2(255),
  avg_space                  VARCHAR2(255),
  chain_cnt                  VARCHAR2(255),
  avg_row_len                VARCHAR2(255),
  sample_size                VARCHAR2(255),
  last_analyzed              VARCHAR2(255), -- YYYYMMDDHHMISS
  partitioned                VARCHAR2(255),
  temporary                  VARCHAR2(255),
  global_stats               VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_TABLES.bad'
  LOGFILE 'TRCA_TABLES.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_TABLES.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_tables');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_tables seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_tables$ (
  owner                      VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  num_rows                   NUMBER,
  blocks                     NUMBER,
  empty_blocks               NUMBER,
  avg_space                  NUMBER,
  chain_cnt                  NUMBER,
  avg_row_len                NUMBER,
  sample_size                NUMBER,
  last_analyzed              DATE,
  partitioned                VARCHAR2(3),
  temporary                  VARCHAR2(1),
  global_stats               VARCHAR2(3)
);

CREATE UNIQUE INDEX trca$_tables$_u1 ON trca$_tables$(table_name, owner);

/* ------------------------------------------------------------------------- */

-- tables refernced by explain plan and row source plan. derived from dba_tables.
CREATE TABLE trca$_tables (
  owner                      VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  num_rows                   NUMBER,
  blocks                     NUMBER,
  empty_blocks               NUMBER,
  avg_space                  NUMBER,
  chain_cnt                  NUMBER,
  avg_row_len                NUMBER,
  sample_size                NUMBER,
  last_analyzed              DATE,
  partitioned                VARCHAR2(3),
  temporary                  VARCHAR2(1),
  global_stats               VARCHAR2(3),
  -- trca
  tool_execution_id          INTEGER NOT NULL, -- fk trca$_tool_execution.id
  actual_rows                INTEGER,
  actual_rows_suffix         CHAR(1)
);

CREATE UNIQUE INDEX trca$_tables_pk ON trca$_tables(tool_execution_id, owner, table_name);
ALTER TABLE trca$_tables ADD (CONSTRAINT trca$_tables_pk PRIMARY KEY (tool_execution_id, owner, table_name));

GRANT SELECT, UPDATE ON trca$_tables TO &&role_name.;

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_indexes (
  owner                      VARCHAR2(255),
  index_name                 VARCHAR2(255),
  index_type                 VARCHAR2(255),
  table_owner                VARCHAR2(255),
  table_name                 VARCHAR2(255),
  uniqueness                 VARCHAR2(255),
  blevel                     VARCHAR2(255),
  leaf_blocks                VARCHAR2(255),
  distinct_keys              VARCHAR2(255),
  avg_leaf_blocks_per_key    VARCHAR2(255),
  avg_data_blocks_per_key    VARCHAR2(255),
  clustering_factor          VARCHAR2(255),
  num_rows                   VARCHAR2(255),
  sample_size                VARCHAR2(255),
  last_analyzed              VARCHAR2(255), -- YYYYMMDDHHMISS
  partitioned                VARCHAR2(255),
  temporary                  VARCHAR2(255),
  global_stats               VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_INDEXES.bad'
  LOGFILE 'TRCA_INDEXES.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_INDEXES.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_indexes');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_tables seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_indexes$ (
  owner                      VARCHAR2(30) NOT NULL,
  index_name                 VARCHAR2(30) NOT NULL,
  index_type                 VARCHAR2(27),
  table_owner                VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  uniqueness                 VARCHAR2(9),
  blevel                     NUMBER,
  leaf_blocks                NUMBER,
  distinct_keys              NUMBER,
  avg_leaf_blocks_per_key    NUMBER,
  avg_data_blocks_per_key    NUMBER,
  clustering_factor          NUMBER,
  num_rows                   NUMBER,
  sample_size                NUMBER,
  last_analyzed              DATE,
  partitioned                VARCHAR2(3),
  temporary                  VARCHAR2(1),
  global_stats               VARCHAR2(3)
);

CREATE UNIQUE INDEX trca$_indexes$_u1 ON trca$_indexes$(index_name, owner);
CREATE INDEX trca$_indexes$_n1 ON trca$_indexes$(table_name, table_owner);

/* ------------------------------------------------------------------------- */

-- indexes for tables refernced by explain plan and row source plan. derived from dba_indexes.
CREATE TABLE trca$_indexes (
  owner                      VARCHAR2(30) NOT NULL,
  index_name                 VARCHAR2(30) NOT NULL,
  index_type                 VARCHAR2(27),
  table_owner                VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  uniqueness                 VARCHAR2(9),
  blevel                     NUMBER,
  leaf_blocks                NUMBER,
  distinct_keys              NUMBER,
  avg_leaf_blocks_per_key    NUMBER,
  avg_data_blocks_per_key    NUMBER,
  clustering_factor          NUMBER,
  num_rows                   NUMBER,
  sample_size                NUMBER,
  last_analyzed              DATE,
  partitioned                VARCHAR2(3),
  temporary                  VARCHAR2(1),
  global_stats               VARCHAR2(3),
  -- trca
  tool_execution_id          INTEGER NOT NULL,
  indexed_columns            VARCHAR2(4000),
  columns_count              INTEGER
);

CREATE UNIQUE INDEX trca$_indexes_pk ON trca$_indexes(tool_execution_id, owner, index_name);
ALTER TABLE trca$_indexes ADD (CONSTRAINT trca$_indexes_pk PRIMARY KEY (tool_execution_id, owner, index_name));

CREATE INDEX trca$_indexes_n1 ON trca$_indexes(tool_execution_id, table_owner, table_name);

/* ------------------------------------------------------------------------- */

-- tables refernced by explain plan and row source plan for a given group
CREATE TABLE trca$_group_tables (
  tool_execution_id          INTEGER      NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER      NOT NULL, -- fk trca$_group.id
  owner                      VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  in_row_source_plan         CHAR(1)      NOT NULL,
  in_explain_plan            CHAR(1)      NOT NULL
);

CREATE UNIQUE INDEX trca$_group_tables_pk ON trca$_group_tables(tool_execution_id, group_id, owner, table_name);
ALTER TABLE trca$_group_tables ADD (CONSTRAINT trca$_group_tables_pk PRIMARY KEY (tool_execution_id, group_id, owner, table_name));

/* ------------------------------------------------------------------------- */

-- indexes for tables refernced by explain plan and row source plan for a given group
CREATE TABLE trca$_group_indexes (
  tool_execution_id          INTEGER      NOT NULL, -- fk trca$_tool_execution.id
  group_id                   INTEGER      NOT NULL, -- fk trca$_group.id
  owner                      VARCHAR2(30) NOT NULL,
  index_name                 VARCHAR2(30) NOT NULL,
  table_owner                VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  in_row_source_plan         CHAR(1)      NOT NULL,
  in_explain_plan            CHAR(1)      NOT NULL
);

CREATE UNIQUE INDEX trca$_group_indexes_pk ON trca$_group_indexes(tool_execution_id, group_id, owner, index_name);
ALTER TABLE trca$_group_indexes ADD (CONSTRAINT trca$_group_indexes_pk PRIMARY KEY (tool_execution_id, group_id, owner, index_name));

CREATE INDEX trca$_group_indexes_n1 ON trca$_group_indexes(tool_execution_id, group_id, table_owner, table_name);

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_tab_cols (
  owner                      VARCHAR2(255),
  table_name                 VARCHAR2(255),
  column_name                VARCHAR2(255),
  column_id                  VARCHAR2(255),
  num_distinct               VARCHAR2(255),
  density                    VARCHAR2(255),
  num_nulls                  VARCHAR2(255),
  num_buckets                VARCHAR2(255),
  last_analyzed              VARCHAR2(255), -- YYYYMMDDHHMISS
  sample_size                VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_TAB_COLS.bad'
  LOGFILE 'TRCA_TAB_COLS.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_TAB_COLS.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_tab_cols');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_tab_cols seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_tab_cols$ (
  owner                      VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  column_name                VARCHAR2(30) NOT NULL,
  column_id                  NUMBER,
  num_distinct               NUMBER,
  density                    NUMBER,
  num_nulls                  NUMBER,
  num_buckets                NUMBER,
  last_analyzed              DATE,
  sample_size                NUMBER
);

CREATE INDEX trca$_tab_cols$_n1 ON trca$_tab_cols$(table_name, owner);

/* ------------------------------------------------------------------------- */

-- table columns for trca$_tables
CREATE TABLE trca$_tab_cols (
  owner                      VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  column_name                VARCHAR2(30) NOT NULL,
  column_id                  NUMBER,
  num_distinct               NUMBER,
  density                    NUMBER,
  num_nulls                  NUMBER,
  num_buckets                NUMBER,
  last_analyzed              DATE,
  sample_size                NUMBER,
  -- trca
  tool_execution_id          INTEGER NOT NULL -- fk trca$_tool_execution.id
);

CREATE INDEX trca$_tab_cols_n1 ON trca$_tab_cols(tool_execution_id, owner, table_name, column_name);

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_ind_columns (
  index_owner                VARCHAR2(255),
  index_name                 VARCHAR2(255),
  table_owner                VARCHAR2(255),
  table_name                 VARCHAR2(255),
  column_name                VARCHAR2(4000),
  column_position            VARCHAR2(255),
  descend                    VARCHAR2(255)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_IND_COLUMNS.bad'
  LOGFILE 'TRCA_IND_COLUMNS.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_IND_COLUMNS.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_ind_columns');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of dba_ind_columns seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_ind_columns$ (
  index_owner                VARCHAR2(30) NOT NULL,
  index_name                 VARCHAR2(30) NOT NULL,
  table_owner                VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  column_name                VARCHAR2(4000),
  column_position            NUMBER NOT NULL,
  descend                    VARCHAR2(4)
);

CREATE INDEX trca$_ind_columns$_n1 ON trca$_ind_columns$(index_name, index_owner);

/* ------------------------------------------------------------------------- */

-- indexed columns for trca$_index
CREATE TABLE trca$_ind_columns (
  index_owner                VARCHAR2(30) NOT NULL,
  index_name                 VARCHAR2(30) NOT NULL,
  table_owner                VARCHAR2(30) NOT NULL,
  table_name                 VARCHAR2(30) NOT NULL,
  column_name                VARCHAR2(4000),
  column_position            NUMBER NOT NULL,
  descend                    VARCHAR2(4),
  -- trca
  tool_execution_id          INTEGER NOT NULL -- fk trca$_tool_execution.id
);

CREATE INDEX trca$_ind_columns_n1 ON trca$_ind_columns(tool_execution_id, index_owner, index_name);

/* ------------------------------------------------------------------------- */

-- external dict table generated by trca/dict/trcadictexp.sql in SOURCE
CREATE TABLE trca_parameter2 (
  name                       VARCHAR2(255),
  value                      VARCHAR2(4000)
) ORGANIZATION EXTERNAL
( TYPE ORACLE_LOADER
  DEFAULT DIRECTORY TRCA$STAGE
  ACCESS PARAMETERS
( RECORDS DELIMITED BY 0x'0A'
  BADFILE 'TRCA_PARAMETER2.bad'
  LOGFILE 'TRCA_PARAMETER2.log'
  FIELDS TERMINATED BY ','
  MISSING FIELD VALUES ARE NULL
  REJECT ROWS WITH ALL NULL FIELDS
) LOCATION ('TRCA_PARAMETER2.txt')
);

WHENEVER SQLERROR CONTINUE;
EXEC DBMS_STATS.LOCK_TABLE_STATS(USER, 'trca_parameter2');
WHENEVER SQLERROR EXIT SQL.SQLCODE;

/* ------------------------------------------------------------------------- */

-- dict snaphot of v$parameter2 seeded by trca$t.refresh_trca$_dict_from
CREATE TABLE trca$_parameter2$ (
  name                       VARCHAR2(80),
  value                      VARCHAR2(4000)
);

/* ------------------------------------------------------------------------- */

SPOOL OFF;
UNDEFINE ON_COMMIT_PRESERVE_ROWS GLOBAL_TEMPORARY TEMPORARY_OR_PERMANENT;
SET ECHO OFF TERM ON VER ON;
PRO TACOBJ completed.
